@@ -297,7 +297,6 @@ static int get_domain_info(struct ipa_extdom_ctx *ctx, const char *domain_name,
                                    domain_info->sid, &range);
     }
     if (err != IDMAP_SUCCESS) {
-        free_domain_info(domain_info);
         ret = EFAULT;
         goto done;
     }
@@ -333,10 +332,13 @@ int handle_request(struct ipa_extdom_ctx *ctx, struct extdom_req *req,
     enum idmap_error_code err;
     char *sid_str;
 
-    ret = get_domain_info(ctx, req->data.name.domain_name, &domain_info);
-    if (ret != 0) {
-        return LDAP_OPERATIONS_ERROR;
+    if (req->input_type != INP_SID) {
+        ret = get_domain_info(ctx, req->data.name.domain_name, &domain_info);
+        if (ret != 0) {
+            return LDAP_OPERATIONS_ERROR;
+        }
     }
+
     if (req->input_type == INP_POSIX_UID || req->input_type == INP_POSIX_GID) {
         if (req->input_type == INP_POSIX_UID) {
             id = req->data.posix_uid.uid;
@@ -375,6 +377,13 @@ int handle_request(struct ipa_extdom_ctx *ctx, struct extdom_req *req,
                 goto done;
             }
 
+            if (req->input_type == INP_SID) {
+                ret = get_domain_info(ctx, domain_name, &domain_info);
+                if (ret != 0) {
+                    return LDAP_OPERATIONS_ERROR;
+                }
+            }
+
             ret = create_response(req, domain_info, domain_name, name, &sid,
                                   name_type, res);
             if (ret != 0) {
@@ -423,7 +432,8 @@ int create_response(struct extdom_req *req, struct domain_info *domain_info,
     struct extdom_res *res;
     uint32_t id;
     enum idmap_error_code err;
-    char sid_str[WBC_SID_STRING_BUFLEN + 1];
+    char *sid_str;
+    wbcErr werr;
 
     res = malloc(sizeof(struct extdom_res));
     if (res == NULL) {
@@ -439,11 +449,12 @@ int create_response(struct extdom_req *req, struct domain_info *domain_info,
                     res->data.name.object_name = name;
                     break;
                 case INP_NAME:
+                case INP_POSIX_UID:
+                case INP_POSIX_GID:
                     res->response_type = RESP_SID;
 
-                    len = wbcSidToStringBuf(sid, sid_str,
-                                            WBC_SID_STRING_BUFLEN);
-                    if (len + 1 > WBC_SID_STRING_BUFLEN) {
+                    werr = wbcSidToString(sid, &sid_str);
+                    if (!WBC_ERROR_IS_OK(werr)) {
                         ret = EINVAL;
                         goto done;
                     }
@@ -456,13 +467,14 @@ int create_response(struct extdom_req *req, struct domain_info *domain_info,
             }
             break;
         case REQ_FULL:
-            len = wbcSidToStringBuf(sid, sid_str, WBC_SID_STRING_BUFLEN);
-            if (len + 1 > WBC_SID_STRING_BUFLEN) {
+            len = wbcSidToString(sid, &sid_str);
+            if (!WBC_ERROR_IS_OK(werr)) {
                 ret = EINVAL;
                 goto done;
             }
 
             err = sss_idmap_sid_to_unix(domain_info->idmap_ctx, sid_str, &id);
+            wbcFreeMemory(sid_str);
             if (err != IDMAP_SUCCESS) {
                 ret = EINVAL;
                 goto done;
@@ -557,6 +569,7 @@ int pack_response(struct extdom_res *res, struct berval **ret_val)
     switch (res->response_type) {
         case RESP_SID:
             ret = ber_printf(ber,"{es}", res->response_type, res->data.sid);
+            wbcFreeMemory(res->data.sid);
             break;
         case RESP_NAME:
             ret = ber_printf(ber,"{e{ss}}", res->response_type,
