@@ -87,6 +87,30 @@ Slapi_PluginDesc ipapwd_plugin_desc = {
 void *ipapwd_plugin_id;
 static int usetxn = 0;
 
+static Slapi_DN *_ConfigAreaDN = NULL;
+static Slapi_DN *_PluginDN = NULL;
+static bool g_plugin_started = false;
+
+void *ipapwd_get_plugin_id(void)
+{
+    return ipapwd_plugin_id;
+}
+
+Slapi_DN *ipapwd_get_otp_config_area(void)
+{
+    return _ConfigAreaDN;
+}
+
+Slapi_DN *ipapwd_get_plugin_sdn(void)
+{
+    return _PluginDN;
+}
+
+bool ipapwd_get_plugin_started(void)
+{
+    return g_plugin_started;
+}
+
 static int filter_keys(struct ipapwd_krbcfg *krbcfg,
                        struct ipapwd_keyset *kset)
 {
@@ -240,22 +264,6 @@ static int ipapwd_chpwop(Slapi_PBlock *pb, struct ipapwd_krbcfg *krbcfg)
 	}
 
 parse_req_done:
-
-	if (usetxn) {
-                Slapi_DN *sdn = slapi_sdn_new_dn_byref(dn);
-                Slapi_Backend *be = slapi_be_select(sdn);
-                slapi_sdn_free(&sdn);
-                if (be) {
-			chpwop_pb = slapi_pblock_new();
-			slapi_pblock_set(chpwop_pb, SLAPI_BACKEND, be);
-			rc = slapi_back_transaction_begin(chpwop_pb);
-			if (rc) {
-				LOG_FATAL("failed to start transaction\n");
-			}
-		} else {
-			LOG_FATAL("failed to get be backend from %s\n", dn);
-		}
-	}
 	/* Uncomment for debugging, otherwise we don't want to leak the
 	 * password values into the log... */
 	/* LDAPDebug( LDAP_DEBUG_ARGS, "passwd: dn (%s), oldPasswd (%s),
@@ -309,6 +317,26 @@ parse_req_done:
 		goto free_and_return;
 	 }
 
+	if (usetxn) {
+                Slapi_DN *sdn = slapi_sdn_new_dn_byref(dn);
+                Slapi_Backend *be = slapi_be_select(sdn);
+                slapi_sdn_free(&sdn);
+                if (be) {
+			chpwop_pb = slapi_pblock_new();
+			if (slapi_pblock_set(chpwop_pb, SLAPI_BACKEND, be)) {
+				LOG_FATAL("slapi_pblock_set failed!\n");
+				rc = LDAP_OPERATIONS_ERROR;
+				goto free_and_return;
+			}
+			rc = slapi_back_transaction_begin(chpwop_pb);
+			if (rc) {
+				LOG_FATAL("failed to start transaction\n");
+			}
+		} else {
+			LOG_FATAL("failed to get be backend from %s\n", dn);
+		}
+	}
+
 	 /* Now we have the DN, look for the entry */
 	 ret = ipapwd_getEntry(dn, &targetEntry, attrlist);
 	 /* If we can't find the entry, then that's an error */
@@ -1191,6 +1219,35 @@ Slapi_Filter *ipapwd_string2filter(char *strfilter)
 	return ret;
 }
 
+/* Loads the OTP config entry, parses it, and applies it. */
+static inline bool ipapwd_load_otp_config(void)
+{
+    char *config_attrs[] = { IPA_USER_AUTH_TYPE, NULL };
+    Slapi_Entry *config_entry = NULL;
+    Slapi_DN *config_sdn = NULL;
+
+    /* If we are using an alternate config area, check it for our
+     * configuration, otherwise we just use our main plug-in config
+     * entry. */
+    if ((config_sdn = ipapwd_get_otp_config_area()) == NULL) {
+        config_sdn = ipapwd_get_plugin_sdn();
+    }
+
+    slapi_log_error(SLAPI_LOG_PLUGIN, IPAPWD_PLUGIN_NAME,
+                    "Looking for config settings in \"%s\".\n",
+                    config_sdn ? slapi_sdn_get_ndn(config_sdn) : "null");
+
+    /* Fetch the config entry. */
+    slapi_search_internal_get_entry(config_sdn, config_attrs, &config_entry,
+                                    ipapwd_plugin_id);
+
+    /* Parse and apply the config. */
+    ipapwd_parse_otp_config_entry(config_entry, true);
+
+    slapi_entry_free(config_entry);
+    return true;
+}
+
 /* Init data structs */
 static int ipapwd_start( Slapi_PBlock *pb )
 {
@@ -1199,8 +1256,37 @@ static int ipapwd_start( Slapi_PBlock *pb )
     char *realm = NULL;
     char *config_dn;
     Slapi_Entry *config_entry = NULL;
+    Slapi_DN *plugindn = NULL;
+    char *config_area = NULL;
     int ret;
 
+    /* Check if we're already started */
+    if (g_plugin_started) {
+        return LDAP_SUCCESS;
+    }
+
+    /* Get the plug-in target dn from the system and store for future use. */
+    slapi_pblock_get(pb, SLAPI_TARGET_SDN, &plugindn);
+    if (plugindn == NULL || slapi_sdn_get_ndn_len(plugindn) == 0) {
+        LOG_FATAL("No plugin dn?\n");
+        return LDAP_OPERATIONS_ERROR;
+    }
+    _PluginDN = slapi_sdn_dup(plugindn);
+
+    /* Set the alternate config area if one is defined. */
+    slapi_pblock_get(pb, SLAPI_PLUGIN_CONFIG_AREA, &config_area);
+    if (config_area != NULL) {
+        _ConfigAreaDN = slapi_sdn_new_normdn_byval(config_area);
+    }
+
+    /*
+     * Load the config.
+     */
+    if (!ipapwd_load_otp_config()) {
+        LOG_FATAL("Unable to load plug-in config\n");
+        return LDAP_OPERATIONS_ERROR;
+    }
+
     krberr = krb5_init_context(&krbctx);
     if (krberr) {
         LOG_FATAL("krb5_init_context failed\n");
@@ -1269,6 +1355,7 @@ static int ipapwd_start( Slapi_PBlock *pb )
     }
 
     ret = LDAP_SUCCESS;
+    g_plugin_started = true;
 
 done:
     free(realm);
@@ -1277,7 +1364,25 @@ done:
     return ret;
 }
 
+/* Clean up any resources allocated at startup. */
+static int ipapwd_close(Slapi_PBlock * pb)
+{
+    if (!g_plugin_started) {
+        goto done;
+    }
 
+    g_plugin_started = false;
+
+    /* We are not guaranteed that other threads are finished accessing
+     * PluginDN or ConfigAreaDN, so we don't want to free them.  This is
+     * only a one-time leak at shutdown, so it should be fine.
+     * slapi_sdn_free(&_PluginDN);
+     * slapi_sdn_free(&_ConfigAreaDN);
+     */
+
+done:
+    return 0;
+}
 
 static char *ipapwd_oid_list[] = {
 	EXOP_PASSWD_OID,
@@ -1324,12 +1429,13 @@ int ipapwd_init( Slapi_PBlock *pb )
      * plug-in function that handles the operation identified by
      * OID 1.3.6.1.4.1.4203.1.11.1 .  Also specify the version of the server
      * plug-in */
-    ret = slapi_pblock_set(pb, SLAPI_PLUGIN_VERSION, SLAPI_PLUGIN_VERSION_01);
+    ret = slapi_pblock_set(pb, SLAPI_PLUGIN_VERSION, SLAPI_PLUGIN_VERSION_03);
     if (!ret) ret = slapi_pblock_set(pb, SLAPI_PLUGIN_START_FN, (void *)ipapwd_start);
     if (!ret) ret = slapi_pblock_set(pb, SLAPI_PLUGIN_DESCRIPTION, (void *)&ipapwd_plugin_desc);
     if (!ret) ret = slapi_pblock_set(pb, SLAPI_PLUGIN_EXT_OP_OIDLIST, ipapwd_oid_list);
     if (!ret) ret = slapi_pblock_set(pb, SLAPI_PLUGIN_EXT_OP_NAMELIST, ipapwd_name_list);
     if (!ret) ret = slapi_pblock_set(pb, SLAPI_PLUGIN_EXT_OP_FN, (void *)ipapwd_extop);
+    if (!ret) ret = slapi_pblock_set(pb, SLAPI_PLUGIN_CLOSE_FN, (void *)ipapwd_close);
     if (ret) {
         LOG("Failed to set plug-in version, function, and OID.\n" );
         return -1;
@@ -1357,5 +1463,10 @@ int ipapwd_init( Slapi_PBlock *pb )
                           "IPA pwd post ops", NULL,
                           ipapwd_plugin_id);
 
+    slapi_register_plugin("internalpostoperation", 1,
+                          "ipapwd_intpost_init", ipapwd_intpost_init,
+                          "IPA pwd internal post ops", NULL,
+                          ipapwd_plugin_id);
+
     return 0;
 }
