@@ -18,6 +18,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
+define(['freeipa/ipa', 'freeipa/jquery', 'freeipa/field', 'freeipa/widget',
+        'freeipa/entity'],
+       function(IPA, $) {  return function() {
 
 var widget_container;
 var widget;
@@ -350,11 +353,4 @@ test("IPA.radio_widget" ,function(){
 
 });
 
-
-
-
-
-
-
-
-
+};});
\ No newline at end of file
