@@ -22,20 +22,20 @@ Base classes for all front-end plugins.
 """
 
 import re
-import inspect
-from base import lock, check_name, NameSpace
-from plugable import Plugin, is_production_mode
-from parameters import create_param, parse_param_spec, Param, Str, Flag, Password
-from output import Output, Entry, ListOfEntries
-from text import _, ngettext
-
-from errors import (ZeroArgumentError, MaxArgumentError, OverlapError,
-    RequiresRoot, VersionError, RequirementError, OptionError)
-from errors import InvocationError
-from constants import TYPE_ERROR
-from ipapython.version import API_VERSION
 from distutils import version
 
+from ipapython.version import API_VERSION
+from ipapython.ipa_log_manager import root_logger
+from base import NameSpace
+from plugable import Plugin, is_production_mode
+from parameters import create_param, Param, Str, Flag, Password
+from output import Output, Entry, ListOfEntries
+from text import _
+from errors import (ZeroArgumentError, MaxArgumentError, OverlapError,
+    VersionError, OptionError, InvocationError,
+    ValidationError, ConversionError)
+from ipalib import messages
+
 
 RULE_FLAG = 'validation_rule'
 
@@ -478,10 +478,10 @@ class Command(HasParam):
         >>> list(c._repr_iter(login=u'Okay.', passwd=u'Private!'))
         ["u'Okay.'", "passwd=u'********'"]
         """
-        for arg in self.args():         #pylint: disable=E1102
+        for arg in self.args():
             value = params.get(arg.name, None)
             yield repr(arg.safe_value(value))
-        for option in self.options():   #pylint: disable=E1102
+        for option in self.options():
             if option.name not in params:
                 continue
             value = params[option.name]
@@ -506,7 +506,7 @@ class Command(HasParam):
 
     def __args_2_params(self, values):
         multivalue = False
-        for (i, arg) in enumerate(self.args()): #pylint: disable=E1102
+        for (i, arg) in enumerate(self.args()):
             assert not multivalue
             if len(values) > i:
                 if arg.multivalue:
@@ -526,7 +526,6 @@ class Command(HasParam):
                 yield (name, options.pop(name))
         # If any options remain, they are either internal or unknown
         unused_keys = set(options).difference(self.internal_options)
-        unused_keys.discard('version')
         if unused_keys:
             raise OptionError(_('Unknown option: %(option)s'),
                 option=unused_keys.pop())
@@ -560,25 +559,31 @@ class Command(HasParam):
             if name in params:
                 yield(name, params[name])
 
-    def split_csv(self, **kw):
+    def prompt_param(self, param, default=None, optional=False, kw=dict(),
+                     label=None):
         """
-        Return a dictionary of values where values are decoded from CSV.
+        Prompts the user for the value of given parameter.
 
-        For example:
-
-        >>> class my_command(Command):
-        ...     takes_options = (
-        ...         Param('flags', multivalue=True, csv=True),
-        ...     )
-        ...
-        >>> c = my_command()
-        >>> c.finalize()
-        >>> c.split_csv(flags=u'public,replicated')
-        {'flags': (u'public', u'replicated')}
+        Returns the parameter instance.
         """
-        return dict(
-            (k, self.params[k].split_csv(v)) for (k, v) in kw.iteritems()
-        )
+
+        if label is None:
+            label = param.label
+
+        while True:
+            raw = self.Backend.textui.prompt(label, default, optional=optional)
+
+            # Backend.textui.prompt does not fill in the default value,
+            # we have to do it ourselves
+            if not raw.strip():
+                raw = default
+
+            try:
+                return param(raw, **kw)
+            except (ValidationError, ConversionError), e:
+                # Display error and prompt again
+                self.Backend.textui.print_prompt_attribute_error(unicode(label),
+                                                             unicode(e.error))
 
     def normalize(self, **kw):
         """
@@ -741,10 +746,17 @@ class Command(HasParam):
         performs is executed remotely.
         """
         if self.api.env.in_server:
-            if 'version' in options:
+            version_provided = 'version' in options
+            if version_provided:
                 self.verify_client_version(options['version'])
-                del options['version']
-            return self.execute(*args, **options)
+            else:
+                options['version'] = API_VERSION
+            result = self.execute(*args, **options)
+            if not version_provided:
+                messages.add_message(
+                    API_VERSION, result,
+                    messages.VersionMissing(server_version=API_VERSION))
+            return result
         return self.forward(*args, **options)
 
     def execute(self, *args, **kw):
@@ -784,7 +796,7 @@ class Command(HasParam):
         else:
             self.max_args = None
         self._create_param_namespace('options')
-        params_nosort = tuple(self.args()) + tuple(self.options()) #pylint: disable=E1102
+        params_nosort = tuple(self.args()) + tuple(self.options())
         def get_key(p):
             if p.required:
                 if p.sortorder < 0:
@@ -897,12 +909,12 @@ class Command(HasParam):
                     exclude='webui',
                     flags=['no_output'],
                 )
-                yield Str('version?',
-                    doc=_('Client version. Used to determine if server will accept request.'),
-                    exclude='webui',
-                    flags=['no_option', 'no_output'],
-                )
-                return
+                break
+        yield Str('version?',
+            doc=_('Client version. Used to determine if server will accept request.'),
+            exclude='webui',
+            flags=['no_option', 'no_output'],
+        )
 
     def validate_output(self, output):
         """
@@ -914,7 +926,7 @@ class Command(HasParam):
                 nice, dict, type(output), output)
             )
         expected_set = set(self.output)
-        actual_set = set(output)
+        actual_set = set(output) - set(['messages'])
         if expected_set != actual_set:
             missing = expected_set - actual_set
             if missing:
@@ -945,6 +957,21 @@ class Command(HasParam):
                 continue
             yield param
 
+    def log_messages(self, output, logger):
+        logger_functions = dict(
+            debug=logger.debug,
+            info=logger.info,
+            warning=logger.warning,
+            error=logger.error,
+        )
+        for message in output.get('messages', ()):
+            try:
+                function = logger_functions[message['type']]
+            except KeyError:
+                logger.error('Server sent a message with a wrong type')
+                function = logger.error
+            function(message.get('message'))
+
     def output_for_cli(self, textui, output, *args, **options):
         """
         Generic output method. Prints values the output argument according
@@ -963,7 +990,9 @@ class Command(HasParam):
 
         rv = 0
 
-        order = [p.name for p in self.output_params()]  #pylint: disable=E1102
+        self.log_messages(output, root_logger)
+
+        order = [p.name for p in self.output_params()]
         if options.get('all', False):
             order.insert(0, 'dn')
             print_all = True
@@ -973,8 +1002,8 @@ class Command(HasParam):
         if options.get('raw', False):
             labels = None
         else:
-            labels = dict((p.name, unicode(p.label)) for p in self.output_params()) #pylint: disable=E1102
-        flags = dict((p.name, p.flags) for p in self.output_params())   #pylint: disable=E1102
+            labels = dict((p.name, unicode(p.label)) for p in self.output_params())
+        flags = dict((p.name, p.flags) for p in self.output_params())
 
         for o in self.output:
             outp = self.output[o]
@@ -1112,7 +1141,7 @@ class Object(HasParam):
             self.__get_attrs('Property'), sort=False, name_attr='attr_name'
         )
         self._create_param_namespace('params')
-        pkeys = filter(lambda p: p.primary_key, self.params())  #pylint: disable=E1102
+        pkeys = filter(lambda p: p.primary_key, self.params())
         if len(pkeys) > 1:
             raise ValueError(
                 '%s (Object) has multiple primary keys: %s' % (
@@ -1123,7 +1152,7 @@ class Object(HasParam):
         if len(pkeys) == 1:
             self.primary_key = pkeys[0]
             self.params_minus_pk = NameSpace(
-                filter(lambda p: not p.primary_key, self.params()), sort=False  #pylint: disable=E1102
+                filter(lambda p: not p.primary_key, self.params()), sort=False
             )
         else:
             self.primary_key = None
@@ -1141,7 +1170,7 @@ class Object(HasParam):
         if len(names) == 1 and not isinstance(names[0], (Param, str)):
             names = names[0]
         minus = frozenset(names)
-        for param in self.params(): #pylint: disable=E1102
+        for param in self.params():
             if param.name in minus or param in minus:
                 continue
             yield param
@@ -1282,7 +1311,7 @@ class Method(Attribute, Command):
     >>> from ipalib import create_api
     >>> api = create_api()
     >>> class user_add(Method):
-    ...     def run(self):
+    ...     def run(self, **options):
     ...             return dict(result='Added the user!')
     ...
     >>> class user(Object):
