@@ -34,6 +34,8 @@ from os import path
 import subprocess
 import optparse
 import errors
+import textwrap
+
 from config import Env
 import util
 import text
@@ -454,7 +456,7 @@ class API(DictProxy):
     def isdone(self, name):
         return name in self.__done
 
-    def bootstrap(self, **overrides):
+    def bootstrap(self, parser=None, **overrides):
         """
         Initialize environment variables and logging.
         """
@@ -488,6 +490,11 @@ class API(DictProxy):
                                           stream=sys.stderr,
                                           level=level,
                                           format=LOGGING_FORMAT_STDERR)])
+
+        if not parser:
+            parser = self.build_global_parser()
+        object.__setattr__(self, 'parser', parser)
+
         # Add file handler:
         if self.env.mode in ('dummy', 'unit_test'):
             return  # But not if in unit-test mode
@@ -501,7 +508,6 @@ class API(DictProxy):
                 log.error('Could not create log_dir %r', log_dir)
                 return
 
-
         level = 'info'
         if self.env.debug:
             level = 'debug'
@@ -519,8 +525,23 @@ class API(DictProxy):
         Add global options to an optparse.OptionParser instance.
         """
         if parser is None:
-            parser = optparse.OptionParser()
+            parser = optparse.OptionParser(
+                add_help_option=False,
+                formatter=IPAHelpFormatter(),
+                usage='%prog [global-options] COMMAND [command-options]',
+                description='Manage an IPA domain',
+                epilog='\n'.join([
+                    'See "ipa help topics" for available help topics.',
+                    'See "ipa help <TOPIC>" for more information on a '
+                        'specific topic.',
+                    'See "ipa help commands" for the full list of commands.',
+                    'See "ipa <COMMAND> --help" for more information on a '
+                        'specific command.',
+                ]))
             parser.disable_interspersed_args()
+            parser.add_option("-h", "--help", action="help",
+                help='Show this help message and exit')
+
         parser.add_option('-e', dest='env', metavar='KEY=VAL', action='append',
             help='Set environment variable KEY to VAL',
         )
@@ -548,12 +569,6 @@ class API(DictProxy):
                 dest='fallback',
                 help='Only use the server configured in /etc/ipa/default.conf'
             )
-        topics = optparse.OptionGroup(parser, "Available help topics",
-                    "ipa help topics")
-        cmds = optparse.OptionGroup(parser, "Available commands",
-                    "ipa help commands")
-        parser.add_option_group(topics)
-        parser.add_option_group(cmds)
 
         return parser
 
@@ -581,7 +596,7 @@ class API(DictProxy):
             overrides['webui_prod'] = options.prod
         if context is not None:
             overrides['context'] = context
-        self.bootstrap(**overrides)
+        self.bootstrap(parser, **overrides)
         return (options, args)
 
     def load_plugins(self):
@@ -730,3 +745,15 @@ class API(DictProxy):
         object.__setattr__(self, 'plugins',
             tuple(PluginInfo(p) for p in plugins.itervalues())
         )
+
+
+class IPAHelpFormatter(optparse.IndentedHelpFormatter):
+    def format_epilog(self, text):
+        text_width = self.width - self.current_indent
+        indent = " " * self.current_indent
+        lines = text.splitlines()
+        result = '\n'.join(
+            textwrap.fill(line, text_width, initial_indent=indent,
+                subsequent_indent=indent)
+            for line in lines)
+        return '\n%s\n' % result
