@@ -101,10 +101,10 @@ command-line now (see last example).
    ipa aci-add --permissions=write --attrs=member --targetgroup=admins --group=editors --prefix=none "Editors manage admins"
 
  Add an ACI that allows members of the admins group to manage the street and zip code of those in the editors group:
-   ipa aci-add --permissions=write --memberof=editors --group=admins --attrs=street,postalcode --prefix=none "admins edit the address of editors"
+   ipa aci-add --permissions=write --memberof=editors --group=admins --attrs=street --attrs=postalcode --prefix=none "admins edit the address of editors"
 
  Add an ACI that allows the admins group manage the street and zipcode of those who work for the boss:
-   ipa aci-add --permissions=write --group=admins --attrs=street,postalcode --filter="(manager=uid=boss,cn=users,cn=accounts,dc=example,dc=com)" --prefix=none "Edit the address of those who work for the boss"
+   ipa aci-add --permissions=write --group=admins --attrs=street --attrs=postalcode --filter="(manager=uid=boss,cn=users,cn=accounts,dc=example,dc=com)" --prefix=none "Edit the address of those who work for the boss"
 
  Add an entirely new kind of record to IPA that isn't covered by any of the --type options, creating a permission:
    ipa permission-add  --permissions=add --subtree="cn=*,cn=orange,cn=accounts,dc=example,dc=com" --desc="Add Orange Entries" add_orange
@@ -245,7 +245,10 @@ def _make_aci(ldap, current, aciname, kw):
             if 'test' in kw and not kw.get('test'):
                 raise e
             else:
-                entry_attrs = {'dn': DN(('cn', kw['permission']), api.env.container_permission)}
+                entry_attrs = {
+                    'dn': DN(('cn', kw['permission']),
+                             api.env.container_permission, api.env.basedn),
+                }
     elif group:
         # Not so friendly with groups. This will raise
         try:
@@ -366,7 +369,8 @@ def _aci_to_kw(ldap, a, test=False, pkey_only=False):
             except errors.NotFound, e:
                 # FIXME, use real name here
                 if test:
-                    dn = DN(('cn', 'test'), api.env.container_permission)
+                    dn = DN(('cn', 'test'), api.env.container_permission,
+                            api.env.basedn)
                     entry_attrs = {'cn': [u'test']}
             if api.env.container_permission in dn:
                 kw['permission'] = entry_attrs['cn'][0]
@@ -392,21 +396,18 @@ def _find_aci_by_name(acis, aciprefix, aciname):
             return a
     raise errors.NotFound(reason=_('ACI with name "%s" not found') % aciname)
 
-def validate_permissions(ugettext, permissions):
-    valid_permissions = []
-    permissions = permissions.split(',')
-    for p in permissions:
-        p = p.strip().lower()
-        if not p in _valid_permissions_values:
-             return '"%s" is not a valid permission' % p
 
-def _normalize_permissions(permissions):
+def validate_permissions(ugettext, perm):
+    perm = perm.strip().lower()
+    if perm not in _valid_permissions_values:
+        return '"%s" is not a valid permission' % perm
+
+
+def _normalize_permissions(perm):
     valid_permissions = []
-    permissions = permissions.split(',')
-    for p in permissions:
-        p = p.strip().lower()
-        if p not in valid_permissions:
-            valid_permissions.append(p)
+    perm = perm.strip().lower()
+    if perm not in valid_permissions:
+        valid_permissions.append(perm)
     return ','.join(valid_permissions)
 
 _prefix_option = StrEnum('aciprefix',
@@ -447,7 +448,7 @@ class aci(Object):
         Str('permissions+', validate_permissions,
             cli_name='permissions',
             label=_('Permissions'),
-            doc=_('comma-separated list of permissions to grant' \
+            doc=_('Permissions to grant' \
                 '(read, write, add, delete, all)'),
             csv=True,
             normalizer=_normalize_permissions,
@@ -455,8 +456,8 @@ class aci(Object):
         ),
         Str('attrs*',
             cli_name='attrs',
-            label=_('Attributes'),
-            doc=_('Comma-separated list of attributes'),
+            label=_('Attributes to which the permission applies'),
+            doc=_('Attributes'),
             csv=True,
             flags=('virtual_attribute',),
         ),
@@ -566,7 +567,7 @@ class aci_del(crud.Delete):
 
     takes_options = (_prefix_option,)
 
-    def execute(self, aciname, aciprefix):
+    def execute(self, aciname, aciprefix, **options):
         """
         Execute the aci-delete operation.
 
