@@ -21,7 +21,6 @@ Base classes for LDAP plugins.
 """
 
 import re
-import json
 import time
 from copy import deepcopy
 import base64
@@ -29,13 +28,14 @@ import base64
 from ipalib import api, crud, errors
 from ipalib import Method, Object, Command
 from ipalib import Flag, Int, Str
-from ipalib.base import NameSpace
-from ipalib.cli import to_cli, from_cli
+from ipalib.cli import to_cli
 from ipalib import output
 from ipalib.text import _
 from ipalib.util import json_serialize, validate_hostname
 from ipapython.dn import DN, RDN
 
+DNA_MAGIC = -1
+
 global_output_params = (
     Flag('has_password',
         label=_('Password'),
@@ -228,9 +228,18 @@ def entry_from_entry(entry, newentry):
         del entry[e]
 
     # Re-populate it with new wentry
-    for e in newentry:
+    for e in newentry.keys():
         entry[e] = newentry[e]
 
+def entry_to_dict(entry, **options):
+    if options.get('raw', False):
+        result = dict(entry)
+    else:
+        result = dict((k.lower(), v) for (k, v) in entry.iteritems())
+    if options.get('all', False):
+        result['dn'] = entry.dn
+    return result
+
 def wait_for_value(ldap, dn, attr, value):
     """
     389-ds postoperation plugins are executed after the data has been
@@ -382,6 +391,10 @@ def remove_external_post_callback(memberattr, membertype, externalattr, ldap, co
                     external_entries.remove(entry[0])
                 completed_external += 1
             else:
+                msg = unicode(errors.NotGroupMember().message)
+                newerror = (entry[0], msg)
+                ind = failed[memberattr][membertype].index(entry)
+                failed[memberattr][membertype][ind] = newerror
                 failed_entries.append(membername)
 
         if completed_external:
@@ -417,7 +430,6 @@ class LDAPObject(Object):
 
     parent_object = ''
     container_dn = ''
-    normalize_dn = True
     object_name = _('entry')
     object_name_plural = _('entries')
     object_class = []
@@ -463,12 +475,12 @@ class LDAPObject(Object):
         if self.parent_object:
             parent_dn = self.api.Object[self.parent_object].get_dn(*keys[:-1])
         else:
-            parent_dn = self.container_dn
+            parent_dn = DN(self.container_dn, api.env.basedn)
         if self.rdn_attribute:
             try:
                 (dn, entry_attrs) = self.backend.find_entry_by_attr(
                     self.primary_key.name, keys[-1], self.object_class, [''],
-                    self.container_dn
+                    DN(self.container_dn, api.env.basedn)
                 )
             except errors.NotFound:
                 pass
@@ -527,7 +539,8 @@ class LDAPObject(Object):
             for member in entry_attrs.setdefault(attr, []):
                 for ldap_obj_name in self.attribute_members[attr]:
                     ldap_obj = self.api.Object[ldap_obj_name]
-                    if ldap_obj.container_dn in member:
+                    container_dn = DN(ldap_obj.container_dn, api.env.basedn)
+                    if member.endswith(container_dn):
                         new_attr = '%s_%s' % (attr, ldap_obj.name)
                         entry_attrs.setdefault(new_attr, []).append(
                             ldap_obj.get_primary_key_from_dn(member)
@@ -768,12 +781,6 @@ last, after all sets and adds."""),
                 # None means "delete this attribute"
                 value = None
 
-            if ldap.has_dn_syntax(attr):
-                try:
-                    value = DN(value)
-                except ValueError:
-                    raise errors.InvalidSyntax(attr=attr)
-
             if attr in newdict:
                 if type(value) in (tuple,):
                     newdict[attr] += list(value)
@@ -860,7 +867,7 @@ last, after all sets and adds."""),
         if needldapattrs:
             try:
                 (dn, old_entry) = self._exc_wrapper(keys, options, ldap.get_entry)(
-                    dn, needldapattrs, normalize=self.obj.normalize_dn
+                    dn, needldapattrs
                 )
             except errors.NotFound:
                 self.obj.handle_not_found(*keys)
@@ -868,7 +875,7 @@ last, after all sets and adds."""),
             # Provide a nice error message when user tries to delete an
             # attribute that does not exist on the entry (and user is not
             # adding it)
-            names = set(n.lower() for n in old_entry)
+            names = set(n.lower() for n in old_entry.keys())
             del_nonexisting = delattrs - (names | setattrs | addattrs)
             if del_nonexisting:
                 raise errors.ValidationError(name=del_nonexisting.pop(),
@@ -980,6 +987,7 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
         ldap = self.obj.backend
 
         entry_attrs = self.args_options_2_entry(*keys, **options)
+        entry_attrs = ldap.make_entry(DN(), entry_attrs)
 
         self.process_attr_options(entry_attrs, None, keys, options)
 
@@ -1004,7 +1012,8 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
             if dn_attr != self.obj.primary_key.name:
                 self.obj.handle_duplicate_entry(*keys)
             dn = ldap.make_dn(
-                entry_attrs, self.obj.rdn_attribute, self.obj.container_dn
+                entry_attrs, self.obj.rdn_attribute,
+                DN(self.obj.container_dn, api.env.basedn)
             )
 
         if options.get('all', False):
@@ -1024,7 +1033,7 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
         _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.disallow_object_classes), entry_attrs.keys(), allow_only=False)
 
         try:
-            self._exc_wrapper(keys, options, ldap.add_entry)(dn, entry_attrs, normalize=self.obj.normalize_dn)
+            self._exc_wrapper(keys, options, ldap.add_entry)(dn, entry_attrs)
         except errors.NotFound:
             parent = self.obj.parent_object
             if parent:
@@ -1051,12 +1060,12 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
                     object_class = None
                 (dn, entry_attrs) = self._exc_wrapper(keys, options, ldap.find_entry_by_attr)(
                     self.obj.primary_key.name, keys[-1], object_class, attrs_list,
-                    self.obj.container_dn
+                    DN(self.obj.container_dn, api.env.basedn)
                 )
                 assert isinstance(dn, DN)
             else:
                 (dn, entry_attrs) = self._exc_wrapper(keys, options, ldap.get_entry)(
-                    dn, attrs_list, normalize=self.obj.normalize_dn
+                    dn, attrs_list
                 )
                 assert isinstance(dn, DN)
         except errors.NotFound:
@@ -1065,10 +1074,12 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
         for callback in self.get_callbacks('post'):
             dn = callback(self, ldap, dn, entry_attrs, *keys, **options)
 
+        self.obj.convert_attribute_members(entry_attrs, *keys, **options)
+
         assert isinstance(dn, DN)
+        entry_attrs = entry_to_dict(entry_attrs, **options)
         entry_attrs['dn'] = dn
 
-        self.obj.convert_attribute_members(entry_attrs, *keys, **options)
         if self.obj.primary_key and keys[-1] is not None:
             return dict(result=entry_attrs, value=keys[-1])
         return dict(result=entry_attrs, value=u'')
@@ -1178,7 +1189,7 @@ class LDAPRetrieve(LDAPQuery):
 
         try:
             (dn, entry_attrs) = self._exc_wrapper(keys, options, ldap.get_entry)(
-                dn, attrs_list, normalize=self.obj.normalize_dn
+                dn, attrs_list
             )
             assert isinstance(dn, DN)
         except errors.NotFound:
@@ -1192,8 +1203,11 @@ class LDAPRetrieve(LDAPQuery):
             assert isinstance(dn, DN)
 
         self.obj.convert_attribute_members(entry_attrs, *keys, **options)
+
         assert isinstance(dn, DN)
+        entry_attrs = entry_to_dict(entry_attrs, **options)
         entry_attrs['dn'] = dn
+
         if self.obj.primary_key and keys[-1] is not None:
             return dict(result=entry_attrs, value=keys[-1])
         return dict(result=entry_attrs, value=u'')
@@ -1255,6 +1269,7 @@ class LDAPUpdate(LDAPQuery, crud.Update):
         assert isinstance(dn, DN)
 
         entry_attrs = self.args_options_2_entry(**options)
+        entry_attrs = ldap.make_entry(dn, entry_attrs)
 
         self.process_attr_options(entry_attrs, dn, keys, options)
 
@@ -1299,7 +1314,7 @@ class LDAPUpdate(LDAPQuery, crud.Update):
             # mean an error occurred, just that there were no other updates to
             # perform.
             assert isinstance(dn, DN)
-            self._exc_wrapper(keys, options, ldap.update_entry)(dn, entry_attrs, normalize=self.obj.normalize_dn)
+            self._exc_wrapper(keys, options, ldap.update_entry)(dn, entry_attrs)
         except errors.EmptyModlist, e:
             if not rdnupdate:
                 raise e
@@ -1308,7 +1323,7 @@ class LDAPUpdate(LDAPQuery, crud.Update):
 
         try:
             (dn, entry_attrs) = self._exc_wrapper(keys, options, ldap.get_entry)(
-                dn, attrs_list, normalize=self.obj.normalize_dn
+                dn, attrs_list
             )
         except errors.NotFound:
             raise errors.MidairCollision(
@@ -1323,6 +1338,9 @@ class LDAPUpdate(LDAPQuery, crud.Update):
             assert isinstance(dn, DN)
 
         self.obj.convert_attribute_members(entry_attrs, *keys, **options)
+
+        entry_attrs = entry_to_dict(entry_attrs, **options)
+
         if self.obj.primary_key and keys[-1] is not None:
             return dict(result=entry_attrs, value=keys[-1])
         return dict(result=entry_attrs, value=u'')
@@ -1376,12 +1394,12 @@ class LDAPDelete(LDAPMultiQuery):
                         for (dn_, entry_attrs) in subentries:
                             delete_subtree(dn_)
                 try:
-                    self._exc_wrapper(nkeys, options, ldap.delete_entry)(base_dn, normalize=self.obj.normalize_dn)
+                    self._exc_wrapper(nkeys, options, ldap.delete_entry)(base_dn)
                 except errors.NotFound:
                     self.obj.handle_not_found(*nkeys)
 
             try:
-                self._exc_wrapper(nkeys, options, ldap.delete_entry)(dn, normalize=self.obj.normalize_dn)
+                self._exc_wrapper(nkeys, options, ldap.delete_entry)(dn)
             except errors.NotFound:
                 self.obj.handle_not_found(*nkeys)
             except errors.NotAllowedOnNonLeaf:
@@ -1436,7 +1454,7 @@ class LDAPModMember(LDAPQuery):
     Base class for member manipulation.
     """
     member_attributes = ['member']
-    member_param_doc = _('comma-separated list of %s')
+    member_param_doc = _('%s')
     member_count_out = ('%i member processed.', '%i members processed.')
 
     def get_options(self):
@@ -1478,7 +1496,7 @@ class LDAPAddMember(LDAPModMember):
     """
     Add other LDAP entries to members.
     """
-    member_param_doc = _('comma-separated list of %s to add')
+    member_param_doc = _('%s to add')
     member_count_out = ('%i member added.', '%i members added.')
     allow_same = False
 
@@ -1535,7 +1553,7 @@ class LDAPAddMember(LDAPModMember):
 
         try:
             (dn, entry_attrs) = self._exc_wrapper(keys, options, ldap.get_entry)(
-                dn, attrs_list, normalize=self.obj.normalize_dn
+                dn, attrs_list
             )
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
@@ -1546,9 +1564,12 @@ class LDAPAddMember(LDAPModMember):
                 **options)
             assert isinstance(dn, DN)
 
-        assert isinstance(dn, DN)
-        entry_attrs['dn'] = dn
         self.obj.convert_attribute_members(entry_attrs, *keys, **options)
+
+        assert isinstance(dn, DN)
+        entry_attrs = entry_to_dict(entry_attrs, **options)
+        entry_attrs['dn'] = dn
+
         return dict(
             completed=completed,
             failed=failed,
@@ -1574,7 +1595,7 @@ class LDAPRemoveMember(LDAPModMember):
     """
     Remove LDAP entries from members.
     """
-    member_param_doc = _('comma-separated list of %s to remove')
+    member_param_doc = _('%s to remove')
     member_count_out = ('%i member removed.', '%i members removed.')
 
     has_output = (
@@ -1633,7 +1654,7 @@ class LDAPRemoveMember(LDAPModMember):
 
         try:
             (dn, entry_attrs) = self._exc_wrapper(keys, options, ldap.get_entry)(
-                dn, attrs_list, normalize=self.obj.normalize_dn
+                dn, attrs_list
             )
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
@@ -1644,10 +1665,12 @@ class LDAPRemoveMember(LDAPModMember):
                 **options)
             assert isinstance(dn, DN)
 
+        self.obj.convert_attribute_members(entry_attrs, *keys, **options)
+
         assert isinstance(dn, DN)
+        entry_attrs = entry_to_dict(entry_attrs, **options)
         entry_attrs['dn'] = dn
 
-        self.obj.convert_attribute_members(entry_attrs, *keys, **options)
         return dict(
             completed=completed,
             failed=failed,
@@ -1785,7 +1808,7 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
         if self.obj.parent_object:
             base_dn = self.api.Object[self.obj.parent_object].get_dn(*args[:-1])
         else:
-            base_dn = self.obj.container_dn
+            base_dn = DN(self.obj.container_dn, api.env.basedn)
         assert isinstance(base_dn, DN)
 
         search_kw = self.args_options_2_entry(**options)
@@ -1858,10 +1881,9 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
             for e in entries:
                 self.obj.convert_attribute_members(e[1], *args, **options)
 
-        for e in entries:
-            assert isinstance(e[0], DN)
-            e[1]['dn'] = e[0]
-        entries = [e for (dn, e) in entries]
+        for (i, e) in enumerate(entries):
+            entries[i] = entry_to_dict(e, **options)
+            entries[i]['dn'] = e.dn
 
         return dict(
             result=entries,
@@ -1899,7 +1921,7 @@ class LDAPModReverseMember(LDAPQuery):
     Base class for reverse member manipulation.
     """
     reverse_attributes = ['member']
-    reverse_param_doc = _('comma-separated list of %s')
+    reverse_param_doc = _('%s')
     reverse_count_out = ('%i member processed.', '%i members processed.')
 
     has_output_params = global_output_params
@@ -1924,7 +1946,7 @@ class LDAPAddReverseMember(LDAPModReverseMember):
     The call looks like "add A to B" but in fact executes
     add B to A to handle reverse membership.
     """
-    member_param_doc = _('comma-separated list of %s to add')
+    member_param_doc = _('%s to add')
     member_count_out = ('%i member added.', '%i members added.')
 
     show_command = None
@@ -1996,7 +2018,9 @@ class LDAPAddReverseMember(LDAPModReverseMember):
             assert isinstance(dn, DN)
 
         assert isinstance(dn, DN)
+        entry_attrs = entry_to_dict(entry_attrs, **options)
         entry_attrs['dn'] = dn
+
         return dict(
             completed=completed,
             failed=failed,
@@ -2024,7 +2048,7 @@ class LDAPRemoveReverseMember(LDAPModReverseMember):
     The call looks like "remove A from B" but in fact executes
     remove B from A to handle reverse membership.
     """
-    member_param_doc = _('comma-separated list of %s to remove')
+    member_param_doc = _('%s to remove')
     member_count_out = ('%i member removed.', '%i members removed.')
 
     show_command = None
@@ -2096,7 +2120,9 @@ class LDAPRemoveReverseMember(LDAPModReverseMember):
             assert isinstance(dn, DN)
 
         assert isinstance(dn, DN)
+        entry_attrs = entry_to_dict(entry_attrs, **options)
         entry_attrs['dn'] = dn
+
         return dict(
             completed=completed,
             failed=failed,
