@@ -194,13 +194,13 @@ class config(LDAPObject):
             cli_name='pac_type',
             label=_('Default PAC types'),
             doc=_('Default types of PAC supported for services'),
-            values=(u'MS-PAC', u'PAD'),
+            values=(u'MS-PAC', u'PAD', u'nfs:NONE'),
             csv=True,
         ),
     )
 
     def get_dn(self, *keys, **kwargs):
-        return DN(('cn', 'ipaconfig'), ('cn', 'etc'))
+        return DN(('cn', 'ipaconfig'), ('cn', 'etc'), api.env.basedn)
 
 api.register(config)
 
