@@ -43,7 +43,7 @@ EXAMPLES:
 
  When managing the list of attributes you need to include all attributes
  in the list, including existing ones. Add postalCode to the list:
-   ipa delegation-mod --attrs=street,postalCode --group=managers --membergroup=employees "managers edit employees' street"
+   ipa delegation-mod --attrs=street --attrs=postalCode --group=managers --membergroup=employees "managers edit employees' street"
 
  Display our updated rule:
    ipa delegation-show "managers edit employees' street"
@@ -81,14 +81,13 @@ class delegation(Object):
         Str('permissions*',
             cli_name='permissions',
             label=_('Permissions'),
-            doc=_('Comma-separated list of permissions to grant ' \
-                '(read, write). Default is write.'),
+            doc=_('Permissions to grant (read, write). Default is write.'),
             csv=True,
         ),
         Str('attrs+',
             cli_name='attrs',
             label=_('Attributes'),
-            doc=_('Comma-separated list of attributes'),
+            doc=_('Attributes to which the delegation applies'),
             csv=True,
             normalizer=lambda value: value.lower(),
         ),
