@@ -29,9 +29,9 @@ import string
 from ipalib import api, errors, util
 from ipalib import Str, Flag, Bytes
 from ipalib.plugins.baseldap import *
-from ipalib.plugins.service import split_principal
-from ipalib.plugins.service import validate_certificate
-from ipalib.plugins.service import set_certificate_attrs
+from ipalib.plugins.service import (split_principal, validate_certificate,
+    set_certificate_attrs, ticket_flags_params, update_krbticketflags,
+    set_kerberos_attrs)
 from ipalib.plugins.dns import (dns_container_exists, _record_types,
         add_records_for_host_validation, add_records_for_host,
         _hostname_validator, get_reverse_zone)
@@ -139,6 +139,12 @@ def update_sshfp_record(zone, record, entry_attrs):
             continue
         if sshfp is not None:
             sshfps.append(sshfp)
+        try:
+            sshfp = SSHPublicKey(pubkey).fingerprint_dns_sha256()
+        except ValueError, UnicodeDecodeError:
+            continue
+        if sshfp is not None:
+            sshfps.append(sshfp)
 
     try:
         api.Command['dnsrecord_mod'](zone, record, sshfprecord=sshfps)
@@ -224,6 +230,7 @@ class host(LDAPObject):
         'fqdn', 'description', 'l', 'nshostlocation', 'krbprincipalname',
         'nshardwareplatform', 'nsosversion', 'usercertificate', 'memberof',
         'managedby', 'memberindirect', 'memberofindirect', 'macaddress',
+        'userclass'
     ]
     uuid_attribute = 'ipauniqueid'
     attribute_members = {
@@ -317,7 +324,13 @@ class host(LDAPObject):
             csv=True,
             flags=['no_search'],
         ),
-    )
+        Str('userclass*',
+            cli_name='class',
+            label=_('Class'),
+            doc=_('Host category (semantics placed on this attribute are for '
+                  'local interpretation)'),
+        ),
+    ) + ticket_flags_params
 
     def get_dn(self, *keys, **options):
         hostname = keys[-1]
@@ -328,8 +341,7 @@ class host(LDAPObject):
             try:
                 (dn, entry_attrs) = self.backend.find_entry_by_attr(
                     'serverhostname', hostname, self.object_class, [''],
-                    self.container_dn
-                )
+                    DN(self.container_dn, api.env.basedn))
             except errors.NotFound:
                 pass
         return dn
@@ -341,8 +353,9 @@ class host(LDAPObject):
         managed_hosts = []
 
         try:
-            (hosts, truncated) = ldap.find_entries(base_dn=self.container_dn,
-                                    filter=host_filter, attrs_list=host_attrs)
+            (hosts, truncated) = ldap.find_entries(
+                base_dn=DN(self.container_dn, api.env.basedn),
+                filter=host_filter, attrs_list=host_attrs)
 
             for host in hosts:
                 managed_hosts.append(host[0])
@@ -358,7 +371,7 @@ class host(LDAPObject):
         """
         ng_container = DN(api.env.container_netgroup, api.env.basedn)
         if 'memberofindirect' in entry_attrs:
-            for member in entry_attrs['memberofindirect']:
+            for member in list(entry_attrs['memberofindirect']):
                 memberdn = DN(member)
                 if memberdn.endswith(ng_container):
                     try:
@@ -406,7 +419,6 @@ class host_add(LDAPCreate):
             util.validate_host_dns(self.log, keys[-1])
         if 'locality' in entry_attrs:
             entry_attrs['l'] = entry_attrs['locality']
-            del entry_attrs['locality']
         entry_attrs['cn'] = keys[-1]
         entry_attrs['serverhostname'] = keys[-1].split('.', 1)[0]
         if 'userpassword' not in entry_attrs and not options.get('random', False):
@@ -434,6 +446,9 @@ class host_add(LDAPCreate):
         entry_attrs['managedby'] = dn
         entry_attrs['objectclass'].append('ieee802device')
         entry_attrs['objectclass'].append('ipasshhost')
+        update_krbticketflags(ldap, entry_attrs, attrs_list, options, False)
+        if 'krbticketflags' in entry_attrs:
+            entry_attrs['objectclass'].append('krbticketpolicyaux')
         return dn
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
@@ -468,6 +483,7 @@ class host_add(LDAPCreate):
                 reason=_('The host was added but the DNS update failed with: %(exc)s') % dict(exc=exc)
             )
         set_certificate_attrs(entry_attrs)
+        set_kerberos_attrs(entry_attrs, options)
 
         if options.get('all', False):
             entry_attrs['managing'] = self.obj.get_managed_hosts(dn)
@@ -560,34 +576,36 @@ class host_del(LDAPDelete):
                                             **delkw)
                             break
 
-        try:
-            (dn, entry_attrs) = ldap.get_entry(dn, ['usercertificate'])
-        except errors.NotFound:
-            self.obj.handle_not_found(*keys)
-
-        if 'usercertificate' in entry_attrs:
-            cert = x509.normalize_certificate(entry_attrs.get('usercertificate')[0])
+        if self.api.env.enable_ra:
             try:
-                serial = unicode(x509.get_serial_number(cert, x509.DER))
+                (dn, entry_attrs) = ldap.get_entry(dn, ['usercertificate'])
+            except errors.NotFound:
+                self.obj.handle_not_found(*keys)
+            cert = entry_attrs.single_value('usercertificate', None)
+            if cert:
+                cert = x509.normalize_certificate(cert)
                 try:
-                    result = api.Command['cert_show'](unicode(serial))['result'
-]
-                    if 'revocation_reason' not in result:
-                        try:
-                            api.Command['cert_revoke'](unicode(serial), revocation_reason=4)
-                        except errors.NotImplementedError:
-                            # some CA's might not implement revoke
-                            pass
-                except errors.NotImplementedError:
-                    # some CA's might not implement revoke
-                    pass
-            except NSPRError, nsprerr:
-                if nsprerr.errno == -8183:
-                    # If we can't decode the cert them proceed with
-                    # removing the host.
-                    self.log.info("Problem decoding certificate %s" % nsprerr.args[1])
-                else:
-                    raise nsprerr
+                    serial = unicode(x509.get_serial_number(cert, x509.DER))
+                    try:
+                        result = api.Command['cert_show'](serial)['result']
+                        if 'revocation_reason' not in result:
+                            try:
+                                api.Command['cert_revoke'](serial,
+                                                           revocation_reason=4)
+                            except errors.NotImplementedError:
+                                # some CA's might not implement revoke
+                                pass
+                    except errors.NotImplementedError:
+                        # some CA's might not implement revoke
+                        pass
+                except NSPRError, nsprerr:
+                    if nsprerr.errno == -8183:
+                        # If we can't decode the cert them proceed with
+                        # removing the host.
+                        self.log.info("Problem decoding certificate %s" %
+                                      nsprerr.args[1])
+                    else:
+                        raise nsprerr
 
         return dn
 
@@ -629,7 +647,6 @@ class host_mod(LDAPUpdate):
             raise errors.ACIError(info=_('cn is immutable'))
         if 'locality' in entry_attrs:
             entry_attrs['l'] = entry_attrs['locality']
-            del entry_attrs['locality']
         if 'krbprincipalname' in entry_attrs:
             (dn, entry_attrs_old) = ldap.get_entry(
                 dn, ['objectclass', 'krbprincipalname']
@@ -643,36 +660,42 @@ class host_mod(LDAPUpdate):
                 entry_attrs['objectclass'] = obj_classes
         cert = x509.normalize_certificate(entry_attrs.get('usercertificate'))
         if cert:
-            x509.verify_cert_subject(ldap, keys[-1], cert)
-            (dn, entry_attrs_old) = ldap.get_entry(dn, ['usercertificate'])
-            if 'usercertificate' in entry_attrs_old:
-                oldcert = x509.normalize_certificate(entry_attrs_old.get('usercertificate')[0])
-                try:
-                    serial = unicode(x509.get_serial_number(oldcert, x509.DER))
+            if self.api.env.enable_ra:
+                x509.verify_cert_subject(ldap, keys[-1], cert)
+                (dn, entry_attrs_old) = ldap.get_entry(dn, ['usercertificate'])
+                oldcert = entry_attrs_old.single_value('usercertificate', None)
+                if oldcert:
+                    oldcert = x509.normalize_certificate(oldcert)
                     try:
-                        result = api.Command['cert_show'](unicode(serial))['result']
-                        if 'revocation_reason' not in result:
-                            try:
-                                api.Command['cert_revoke'](unicode(serial), revocation_reason=4)
-                            except errors.NotImplementedError:
-                                # some CA's might not implement revoke
-                                pass
-                    except errors.NotImplementedError:
-                        # some CA's might not implement revoke
-                        pass
-                except NSPRError, nsprerr:
-                    if nsprerr.errno == -8183:
-                        # If we can't decode the cert them proceed with
-                        # modifying the host.
-                        self.log.info("Problem decoding certificate %s" % nsprerr.args[1])
-                    else:
-                        raise nsprerr
+                        serial = x509.get_serial_number(oldcert, x509.DER)
+                        serial = unicode(serial)
+                        try:
+                            result = api.Command['cert_show'](serial)['result']
+                            if 'revocation_reason' not in result:
+                                try:
+                                    api.Command['cert_revoke'](
+                                        serial, revocation_reason=4)
+                                except errors.NotImplementedError:
+                                    # some CA's might not implement revoke
+                                    pass
+                        except errors.NotImplementedError:
+                            # some CA's might not implement revoke
+                            pass
+                    except NSPRError, nsprerr:
+                        if nsprerr.errno == -8183:
+                            # If we can't decode the cert them proceed with
+                            # modifying the host.
+                            self.log.info("Problem decoding certificate %s" %
+                                          nsprerr.args[1])
+                        else:
+                            raise nsprerr
 
             entry_attrs['usercertificate'] = cert
 
         if options.get('random'):
             entry_attrs['userpassword'] = ipa_generate_password(characters=host_pwd_chars)
             setattr(context, 'randompassword', entry_attrs['userpassword'])
+
         if 'macaddress' in entry_attrs:
             if 'objectclass' in entry_attrs:
                 obj_classes = entry_attrs['objectclass']
@@ -704,6 +727,15 @@ class host_mod(LDAPUpdate):
             if 'ipasshhost' not in obj_classes:
                 obj_classes.append('ipasshhost')
 
+        update_krbticketflags(ldap, entry_attrs, attrs_list, options, True)
+
+        if 'krbticketflags' in entry_attrs:
+            if 'objectclass' not in entry_attrs:
+                entry_attrs_old = ldap.get_entry(dn, ['objectclass'])
+                entry_attrs['objectclass'] = entry_attrs_old['objectclass']
+            if 'krbticketpolicyaux' not in entry_attrs['objectclass']:
+                entry_attrs['objectclass'].append('krbticketpolicyaux')
+
         return dn
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
@@ -711,6 +743,7 @@ class host_mod(LDAPUpdate):
         if options.get('random', False):
             entry_attrs['randompassword'] = unicode(getattr(context, 'randompassword'))
         set_certificate_attrs(entry_attrs)
+        set_kerberos_attrs(entry_attrs, options)
         self.obj.get_password_attributes(ldap, dn, entry_attrs)
         if entry_attrs['has_password']:
             # If an OTP is set there is no keytab, at least not one
@@ -797,6 +830,7 @@ class host_find(LDAPSearch):
         for entry in entries:
             (dn, entry_attrs) = entry
             set_certificate_attrs(entry_attrs)
+            set_kerberos_attrs(entry_attrs, options)
             self.obj.get_password_attributes(ldap, dn, entry_attrs)
             self.obj.suppress_netgroup_memberof(entry_attrs)
             if entry_attrs['has_password']:
@@ -835,6 +869,7 @@ class host_show(LDAPRetrieve):
             entry_attrs['has_keytab'] = False
 
         set_certificate_attrs(entry_attrs)
+        set_kerberos_attrs(entry_attrs, options)
 
         if options.get('all', False):
             entry_attrs['managing'] = self.obj.get_managed_hosts(dn)
@@ -882,12 +917,6 @@ class host_disable(LDAPQuery):
         # See if we actually do anthing here, and if not raise an exception
         done_work = False
 
-        dn = self.obj.get_dn(*keys, **options)
-        try:
-            (dn, entry_attrs) = ldap.get_entry(dn, ['usercertificate'])
-        except errors.NotFound:
-            self.obj.handle_not_found(*keys)
-
         truncated = True
         while truncated:
             try:
@@ -906,28 +935,38 @@ class host_disable(LDAPQuery):
                             done_work = True
                         except errors.AlreadyInactive:
                             pass
-        if 'usercertificate' in entry_attrs:
-            cert = x509.normalize_certificate(entry_attrs.get('usercertificate')[0])
-            try:
-                serial = unicode(x509.get_serial_number(cert, x509.DER))
+
+        dn = self.obj.get_dn(*keys, **options)
+        try:
+            (dn, entry_attrs) = ldap.get_entry(dn, ['usercertificate'])
+        except errors.NotFound:
+            self.obj.handle_not_found(*keys)
+        cert = entry_attrs.single_value('usercertificate', None)
+        if cert:
+            if self.api.env.enable_ra:
+                cert = x509.normalize_certificate(cert)
                 try:
-                    result = api.Command['cert_show'](unicode(serial))['result']
-                    if 'revocation_reason' not in result:
-                        try:
-                            api.Command['cert_revoke'](unicode(serial), revocation_reason=4)
-                        except errors.NotImplementedError:
-                            # some CA's might not implement revoke
-                            pass
-                except errors.NotImplementedError:
-                    # some CA's might not implement revoke
-                    pass
-            except NSPRError, nsprerr:
-                if nsprerr.errno == -8183:
-                    # If we can't decode the cert them proceed with
-                    # disabling the host.
-                    self.log.info("Problem decoding certificate %s" % nsprerr.args[1])
-                else:
-                    raise nsprerr
+                    serial = unicode(x509.get_serial_number(cert, x509.DER))
+                    try:
+                        result = api.Command['cert_show'](serial)['result']
+                        if 'revocation_reason' not in result:
+                            try:
+                                api.Command['cert_revoke'](serial,
+                                                           revocation_reason=4)
+                            except errors.NotImplementedError:
+                                # some CA's might not implement revoke
+                                pass
+                    except errors.NotImplementedError:
+                        # some CA's might not implement revoke
+                        pass
+                except NSPRError, nsprerr:
+                    if nsprerr.errno == -8183:
+                        # If we can't decode the cert them proceed with
+                        # disabling the host.
+                        self.log.info("Problem decoding certificate %s" %
+                                      nsprerr.args[1])
+                    else:
+                        raise nsprerr
 
             # Remove the usercertificate altogether
             ldap.update_entry(dn, {'usercertificate': None})
