@@ -37,8 +37,8 @@ EXAMPLES:
  Add another new host group:
    ipa hostgroup-add --desc="Maryland hosts" maryland
 
- Add members to the hostgroup:
-   ipa hostgroup-add-member --hosts=box1,box2,box3 baltimore
+ Add members to the hostgroup (using Bash brace expansion):
+   ipa hostgroup-add-member --hosts={box1,box2,box3} baltimore
 
  Add a hostgroup as a member of another hostgroup:
    ipa hostgroup-add-member --hostgroups=baltimore maryland
@@ -99,7 +99,7 @@ class hostgroup(LDAPObject):
         """
         if 'memberof' in entry_attrs:
             hgdn = DN(dn)
-            for member in entry_attrs['memberof']:
+            for member in list(entry_attrs['memberof']):
                 ngdn = DN(member)
                 if ngdn['cn'] == hgdn['cn']:
                     try:
