@@ -25,7 +25,7 @@ Plugins not accessible directly through the CLI, commands used internally
 
 import json
 
-from ipalib import api, errors
+from ipalib import api
 from ipalib import Command
 from ipalib import Str
 from ipalib.output import Output
@@ -164,9 +164,8 @@ class i18n_messages(Command):
                 "memberallowcmd": _("Add Allow ${other_entity} into ${entity} ${primary_key}"),
                 "memberdenycmd": _("Add Deny ${other_entity} into ${entity} ${primary_key}"),
                 "memberof": _("Add ${entity} ${primary_key} into ${other_entity}"),
-                "sourcehost": _("Add Source ${other_entity} into ${entity} ${primary_key}"),
             },
-            "added": _("Items added"),
+            "added": _("${count} item(s) added"),
             "direct_membership": _("Direct Membership"),
             "indirect_membership": _("Indirect Membership"),
             "no_entries": _("No entries."),
@@ -179,9 +178,8 @@ class i18n_messages(Command):
                 "memberallowcmd": _("Remove Allow ${other_entity} from ${entity} ${primary_key}"),
                 "memberdenycmd": _("Remove Deny ${other_entity} from ${entity} ${primary_key}"),
                 "memberof": _("Remove ${entity} ${primary_key} from ${other_entity}"),
-                "sourcehost": _("Remove Source ${other_entity} from ${entity} ${primary_key}"),
             },
-            "removed": _("Items removed"),
+            "removed": _("${count} item(s) removed"),
             "show_results": _("Show Results"),
         },
         "buttons": {
@@ -266,6 +264,12 @@ class i18n_messages(Command):
             "search": _("Search"),
         },
         "false": _("False"),
+        "krbauthzdata": {
+            "inherited": _("Inherited from server configuration"),
+            "mspac": _("MS-PAC"),
+            "override": _("Override inherited settings"),
+            "pad": _("PAD"),
+        },
         "login": {
             "form_auth": _("To login with username and password, enter them in the fields below then click Login."),
             "header": _("Logged In As"),
@@ -312,10 +316,24 @@ class i18n_messages(Command):
                 "aa_compromise": _("AA Compromise"),
                 "affiliation_changed": _("Affiliation Changed"),
                 "ca_compromise": _("CA Compromise"),
+                "certificate": _("Certificate"),
+                "certificates": _("Certificates"),
                 "certificate_hold": _("Certificate Hold"),
                 "cessation_of_operation": _("Cessation of Operation"),
                 "common_name": _("Common Name"),
                 "expires_on": _("Expires On"),
+                "find_issuedon_from": _("Issued on from"),
+                "find_issuedon_to": _("Issued on to"),
+                "find_max_serial_number": _("Maximum serial number"),
+                "find_min_serial_number": _("Minimum serial number"),
+                "find_revocation_reason": _("Revocation reason"),
+                "find_revokedon_from": _("Revoked on from"),
+                "find_revokedon_to": _("Revoked on to"),
+                "find_subject": _("Subject"),
+                "find_validnotafter_from": _("Valid not after from"),
+                "find_validnotafter_to": _("Valid not after to"),
+                "find_validnotbefore_from": _("Valid not before from"),
+                "find_validnotbefore_to": _("Valid not before to"),
                 "fingerprints": _("Fingerprints"),
                 "issue_certificate": _("Issue New Certificate for ${entity} ${primary_key}"),
                 "issued_by": _("Issued By"),
@@ -334,14 +352,18 @@ class i18n_messages(Command):
                 "request_message": _("<ol> <li>Create a certificate database or use an existing one. To create a new database:<br/> <code># certutil -N -d &lt;database path&gt;</code> </li> <li>Create a CSR with subject <em>CN=&lt;hostname&gt;,O=&lt;realm&gt;</em>, for example:<br/> <code># certutil -R -d &lt;database path&gt; -a -g &lt;key size&gt; -s 'CN=${hostname},O=${realm}'</code> </li> <li> Copy and paste the CSR (from <em>-----BEGIN NEW CERTIFICATE REQUEST-----</em> to <em>-----END NEW CERTIFICATE REQUEST-----</em>) into the text area below: </li> </ol>"),
                 "requested": _("Certificate requested"),
                 "restore_certificate": _("Restore Certificate for ${entity} ${primary_key}"),
+                "restore_certificate_simple": _("Restore Certificate"),
                 "restore_confirmation": _("To confirm your intention to restore this certificate, click the \"Restore\" button."),
                 "restored": _("Certificate restored"),
+                "revocation_reason": _("Revocation reason"),
                 "revoke_certificate": _("Revoke Certificate for ${entity} ${primary_key}"),
+                "revoke_certificate_simple": _("Revoke Certificate"),
                 "revoke_confirmation": _("To confirm your intention to revoke this certificate, select a reason from the pull-down list, and click the \"Revoke\" button."),
                 "revoked": _("Certificate Revoked"),
                 "serial_number": _("Serial Number"),
                 "serial_number_hex": _("Serial Number (hex)"),
                 "sha1_fingerprint": _("SHA1 Fingerprint"),
+                "status": _("Status"),
                 "superseded": _("Superseded"),
                 "unspecified": _("Unspecified"),
                 "valid": _("Valid Certificate Present"),
@@ -425,7 +447,6 @@ class i18n_messages(Command):
                 "host": _("Accessing"),
                 "ipaenabledflag": _("Rule status"),
                 "service": _("Via Service"),
-                "sourcehost": _("From"),
                 "specified_hosts": _("Specified Hosts and Groups"),
                 "specified_services": _("Specified Services and Groups"),
                 "specified_users": _("Specified Users and Groups"),
@@ -518,6 +539,12 @@ class i18n_messages(Command):
                 "type_ad": _("Active Directory domain"),
                 "type_local": _("Local domain"),
             },
+            "realmdomains": {
+                "identity": _("Realm Domains"),
+                "check_dns": _("Check DNS"),
+                "check_dns_confirmation": _("Do you also want to perform DNS check?"),
+                "force_update": _("Force Update"),
+            },
             "role": {
                 "identity": _("Role Settings"),
             },
@@ -574,7 +601,7 @@ class i18n_messages(Command):
                 "host": _("Access this host"),
                 "ipaenabledflag": _("Rule status"),
                 "option_added": _("Option added"),
-                "option_removed": _("Option(s) removed"),
+                "option_removed": _("${count} option(s) removed"),
                 "options": _("Options"),
                 "runas": _("As Whom"),
                 "specified_commands": _("Specified Commands and Groups"),
@@ -586,6 +613,7 @@ class i18n_messages(Command):
             "trust": {
                 "account": _("Account"),
                 "admin_account": _("Administrative account"),
+                "blacklists": _("SID blacklists"),
                 "details": _("Trust Settings"),
                 "domain": _("Domain"),
                 "establish_using": _("Establish using"),
@@ -596,6 +624,9 @@ class i18n_messages(Command):
                 "truststatus": _("Trust status"),
                 "trusttype": _("Trust type"),
             },
+            "trustconfig": {
+                "options": _("Options"),
+            },
             "user": {
                 "account": _("Account Settings"),
                 "account_status": _("Account Status"),
@@ -626,11 +657,11 @@ class i18n_messages(Command):
         },
         "search": {
             "delete_confirm": _("Are you sure you want to delete selected entries?"),
-            "deleted": _("Selected entries were deleted."),
+            "deleted": _("${count} item(s) deleted"),
             "disable_confirm": _("Are you sure you want to disable selected entries?"),
-            "disabled": _("${count} items were disabled"),
+            "disabled": _("${count} item(s) disabled"),
             "enable_confirm": _("Are you sure you want to enable selected entries?"),
-            "enabled": _("${count} items were enabled"),
+            "enabled": _("${count} item(s) enabled"),
             "partial_delete": _("Some entries were not deleted"),
             "quick_links": _("Quick Links"),
             "select_all": _("Select All"),
@@ -648,6 +679,7 @@ class i18n_messages(Command):
             "audit": _("Audit"),
             "automember": _("Automember"),
             "automount": _("Automount"),
+            "cert": _("Certificates"),
             "dns": _("DNS"),
             "hbac": _("Host Based Access Control"),
             "identity": _("Identity"),
@@ -655,6 +687,7 @@ class i18n_messages(Command):
             "policy": _("Policy"),
             "role": _("Role Based Access Control"),
             "sudo": _("Sudo"),
+            "trust": _("Trusts"),
         },
         "true": _("True"),
         "widget": {
@@ -680,10 +713,10 @@ class i18n_messages(Command):
         },
     }
     has_output = (
-        Output('messages', dict, doc=_('Dict of I18N messages')),
+        Output('texts', dict, doc=_('Dict of I18N messages')),
     )
-    def execute(self):
-        return dict([("messages",json_serialize(self.messages))])
+    def execute(self, **options):
+        return dict(texts=json_serialize(self.messages))
 
     def output_for_cli(self, textui, result, *args, **options):
         print json.dumps(result, default=json_serialize)
