@@ -22,8 +22,10 @@ import errno
 import ldap
 import tempfile
 import uuid
-from ipaserver import ipaldap
-from ipaserver.install import installutils
+import string
+import struct
+import re
+
 from ipaserver.install import service
 from ipaserver.install.dsinstance import realm_to_serverid
 from ipaserver.install.bindinstance import get_rr, add_rr, del_rr, \
@@ -34,13 +36,9 @@ from ipapython import sysrestore
 from ipapython import ipautil
 from ipapython.ipa_log_manager import *
 from ipapython import services as ipaservices
-from ipapython.dn import DN
 
 import ipaclient.ipachangeconf
 
-import string
-import struct
-import re
 
 ALLOWED_NETBIOS_CHARS = string.ascii_uppercase + string.digits
 
@@ -160,31 +158,29 @@ class ADTRUSTInstance(service.Service):
         admin_group_dn = DN(('cn', 'admins'), api.env.container_group,
                             self.suffix)
         try:
-            dom_entry = self.admin_conn.getEntry(self.smb_dom_dn, \
-                                                 ldap.SCOPE_BASE)
+            dom_entry = self.admin_conn.get_entry(self.smb_dom_dn)
         except errors.NotFound:
             self.print_msg("Samba domain object not found")
             return
 
-        dom_sid = dom_entry.getValue(self.ATTR_SID)
+        dom_sid = dom_entry.single_value(self.ATTR_SID, None)
         if not dom_sid:
             self.print_msg("Samba domain object does not have a SID")
             return
 
         try:
-            admin_entry = self.admin_conn.getEntry(admin_dn, ldap.SCOPE_BASE)
+            admin_entry = self.admin_conn.get_entry(admin_dn)
         except:
             self.print_msg("IPA admin object not found")
             return
 
         try:
-            admin_group_entry = self.admin_conn.getEntry(admin_group_dn, \
-                                                         ldap.SCOPE_BASE)
+            admin_group_entry = self.admin_conn.get_entry(admin_group_dn)
         except:
             self.print_msg("IPA admin group object not found")
             return
 
-        if admin_entry.getValue(self.ATTR_SID):
+        if admin_entry.single_value(self.ATTR_SID, None):
             self.print_msg("Admin SID already set, nothing to do")
         else:
             try:
@@ -194,7 +190,7 @@ class ADTRUSTInstance(service.Service):
             except:
                 self.print_msg("Failed to modify IPA admin object")
 
-        if admin_group_entry.getValue(self.ATTR_SID):
+        if admin_group_entry.single_value(self.ATTR_SID, None):
             self.print_msg("Admin group SID already set, nothing to do")
         else:
             try:
@@ -220,20 +216,19 @@ class ADTRUSTInstance(service.Service):
         self.ldap_connect()
 
         try:
-            dom_entry = self.admin_conn.getEntry(self.smb_dom_dn, \
-                                                 ldap.SCOPE_BASE)
+            dom_entry = self.admin_conn.get_entry(self.smb_dom_dn)
         except errors.NotFound:
             self.print_msg("Samba domain object not found")
             return
 
-        if dom_entry.getValue(self.ATTR_FALLBACK_GROUP):
+        if dom_entry.single_value(self.ATTR_FALLBACK_GROUP, None):
             self.print_msg("Fallback group already set, nothing to do")
             return
 
         fb_group_dn = DN(('cn', self.FALLBACK_GROUP_NAME),
                          api.env.container_group, self.suffix)
         try:
-            self.admin_conn.getEntry(fb_group_dn, ldap.SCOPE_BASE)
+            self.admin_conn.get_entry(fb_group_dn)
         except errors.NotFound:
             try:
                 self._ldap_mod('default-smb-group.ldif', self.sub_dict)
@@ -244,7 +239,7 @@ class ADTRUSTInstance(service.Service):
         # _ldap_mod does not return useful error codes, so we must check again
         # if the fallback group was created properly.
         try:
-            self.admin_conn.getEntry(fb_group_dn, ldap.SCOPE_BASE)
+            self.admin_conn.get_entry(fb_group_dn)
         except errors.NotFound:
             self.print_msg("Failed to add fallback group.")
             return
@@ -263,36 +258,54 @@ class ADTRUSTInstance(service.Service):
         """
 
         try:
-            res = self.admin_conn.getList(DN(api.env.container_ranges, self.suffix),
-                                          ldap.SCOPE_ONELEVEL,
-                                          "(objectclass=ipaDomainIDRange)")
-            if len(res) != 1:
-                root_logger.critical("Found more than one ID range for the " \
-                                     "local domain.")
-                raise RuntimeError("Too many ID ranges\n")
+            # Get the ranges
+            ranges = self.admin_conn.get_entries(
+                DN(api.env.container_ranges, self.suffix),
+                ldap.SCOPE_ONELEVEL, "(objectclass=ipaDomainIDRange)")
 
-            if res[0].getValue('ipaBaseRID') or \
-               res[0].getValue('ipaSecondaryBaseRID'):
+            # Filter out ranges where RID base is already set
+            no_rid_base_set = lambda r: not any((
+                                  r.single_value('ipaBaseRID', None),
+                                  r.single_value('ipaSecondaryBaseRID', None)))
+
+            ranges_with_no_rid_base = filter(no_rid_base_set, ranges)
+
+            # Return if no range is without RID base
+            if len(ranges_with_no_rid_base) == 0:
                 self.print_msg("RID bases already set, nothing to do")
                 return
 
-            size = res[0].getValue('ipaIDRangeSize')
+            # Abort if RID base needs to be added to more than one range
+            if len(ranges_with_no_rid_base) != 1:
+                root_logger.critical("Found more than one local domain ID "
+                                     "range with no RID base set.")
+                raise RuntimeError("Too many ID ranges\n")
+
+            # Abort if RID bases are too close
+            local_range = ranges_with_no_rid_base[0]
+            size = local_range.single_value('ipaIDRangeSize', None)
+
             if abs(self.rid_base - self.secondary_rid_base) > size:
-                self.print_msg("Primary and secondary RID base are too close. " \
+                self.print_msg("Primary and secondary RID base are too close. "
                       "They have to differ at least by %d." % size)
                 raise RuntimeError("RID bases too close.\n")
 
+            # Modify the range
+            # If the RID bases would cause overlap with some other range,
+            # this will be detected by ipa-range-check DS plugin
             try:
-                self.admin_conn.modify_s(res[0].dn,
-                                         [(ldap.MOD_ADD, "ipaBaseRID", \
-                                                 str(self.rid_base)), \
-                                         (ldap.MOD_ADD, "ipaSecondaryBaseRID", \
+                self.admin_conn.modify_s(local_range.dn,
+                                         [(ldap.MOD_ADD, "ipaBaseRID",
+                                                 str(self.rid_base)),
+                                         (ldap.MOD_ADD, "ipaSecondaryBaseRID",
                                                  str(self.secondary_rid_base))])
-            except:
-                self.print_msg("Failed to add RID bases to the local range object")
+            except ldap.CONSTRAINT_VIOLATION, e:
+                self.print_msg("Failed to add RID bases to the local range "
+                               "object:\n  %s" % e[0]['info'])
+                raise RuntimeError("Constraint violation.\n")
 
         except errors.NotFound as e:
-            root_logger.critical("ID range of the local domain not found, " \
+            root_logger.critical("ID range of the local domain not found, "
                                  "define it and run again.")
             raise e
 
@@ -312,7 +325,7 @@ class ADTRUSTInstance(service.Service):
     def __create_samba_domain_object(self):
 
         try:
-            self.admin_conn.getEntry(self.smb_dom_dn, ldap.SCOPE_BASE)
+            self.admin_conn.get_entry(self.smb_dom_dn)
             if self.reset_netbios_name:
                 self.__reset_netbios_name()
             else :
@@ -325,27 +338,30 @@ class ADTRUSTInstance(service.Service):
                        DN(('cn', 'ad'), self.trust_dn), \
                        DN(api.env.container_cifsdomains, self.suffix)):
             try:
-                self.admin_conn.getEntry(new_dn, ldap.SCOPE_BASE)
+                self.admin_conn.get_entry(new_dn)
             except errors.NotFound:
-                entry = ipaldap.Entry(new_dn)
-                entry.setValues("objectclass", ["nsContainer"])
                 try:
                     name = new_dn[1].attr
                 except Exception, e:
                     self.print_msg('Cannot extract RDN attribute value from "%s": %s' % \
                           (new_dn, e))
                     return
-                entry.setValues("cn", name)
-                self.admin_conn.addEntry(entry)
+                entry = self.admin_conn.make_entry(
+                    new_dn, objectclass=['nsContainer'], cn=[name])
+                self.admin_conn.add_entry(entry)
 
-        entry = ipaldap.Entry(self.smb_dom_dn)
-        entry.setValues("objectclass", [self.OBJC_DOMAIN, "nsContainer"])
-        entry.setValues("cn", self.domain_name)
-        entry.setValues(self.ATTR_FLAT_NAME, self.netbios_name)
-        entry.setValues(self.ATTR_SID, self.__gen_sid_string())
-        entry.setValues(self.ATTR_GUID, str(uuid.uuid4()))
+        entry = self.admin_conn.make_entry(
+            self.smb_dom_dn,
+            {
+                'objectclass': [self.OBJC_DOMAIN, "nsContainer"],
+                'cn': [self.domain_name],
+                self.ATTR_FLAT_NAME: [self.netbios_name],
+                self.ATTR_SID: [self.__gen_sid_string()],
+                self.ATTR_GUID: [str(uuid.uuid4())],
+            }
+        )
         #TODO: which MAY attributes do we want to set ?
-        self.admin_conn.addEntry(entry)
+        self.admin_conn.add_entry(entry)
 
     def __write_smb_conf(self):
         self.fstore.backup_file(self.smb_conf)
@@ -364,7 +380,7 @@ class ADTRUSTInstance(service.Service):
         try:
             plugin_dn = DN(('cn', plugin_cn), ('cn', 'plugins'),
                            ('cn', 'config'))
-            self.admin_conn.getEntry(plugin_dn, ldap.SCOPE_BASE)
+            self.admin_conn.get_entry(plugin_dn)
             self.print_msg('%s plugin already configured, nothing to do' % name)
         except errors.NotFound:
             try:
@@ -414,12 +430,11 @@ class ADTRUSTInstance(service.Service):
         targets_dn = DN(('cn', 'ipa-cifs-delegation-targets'), ('cn', 's4u2proxy'),
                         ('cn', 'etc'), self.suffix)
         try:
-            targets = self.admin_conn.getEntry(targets_dn, ldap.SCOPE_BASE)
-            current = ipaldap.Entry((targets_dn, targets.toDict()))
-            members = current.getValues('memberPrincipal') or []
+            current = self.admin_conn.get_entry(targets_dn)
+            members = current.get('memberPrincipal', [])
             if not(self.cifs_principal in members):
-                current.setValues("memberPrincipal", members + [self.cifs_principal])
-                self.admin_conn.updateEntry(targets_dn, targets.toDict(), current.toDict())
+                current["memberPrincipal"] = members + [self.cifs_principal]
+                self.admin_conn.update_entry(targets_dn, current)
             else:
                 self.print_msg('cifs principal already targeted, nothing to do.')
         except errors.NotFound:
@@ -446,18 +461,19 @@ class ADTRUSTInstance(service.Service):
             # as 389-ds only operates with GroupOfNames, we have to use
             # the principal's proper dn as defined in self.cifs_agent
             try:
-                entry = self.admin_conn.getEntry(self.smb_dn, ldap.SCOPE_BASE)
-                current = ipaldap.Entry((self.smb_dn, entry.toDict()))
-                members = current.getValues('member') or []
+                current = self.admin_conn.get_entry(self.smb_dn)
+                members = current.get('member', [])
                 if not(self.cifs_agent in members):
-                    current.setValues("member", members + [self.cifs_agent])
-                    self.admin_conn.updateEntry(self.smb_dn, entry.toDict(), current.toDict())
+                    current["member"] = members + [self.cifs_agent]
+                    self.admin_conn.update_entry(self.smb_dn, current)
             except errors.NotFound:
-                entry = ipaldap.Entry(self.smb_dn)
-                entry.setValues("objectclass", ["top", "GroupOfNames"])
-                entry.setValues("cn", self.smb_dn['cn'])
-                entry.setValues("member", [self.cifs_agent])
-                self.admin_conn.addEntry(entry)
+                entry = self.admin_conn.make_entry(
+                    self.smb_dn,
+                    objectclass=["top", "GroupOfNames"],
+                    cn=[self.smb_dn['cn']],
+                    member=[self.cifs_agent],
+                )
+                self.admin_conn.add_entry(entry)
         except Exception, e:
             # CIFS principal already exists, it is not the first time adtrustinstance is managed
             # That's fine, we we'll re-extract the key again.
@@ -479,6 +495,9 @@ class ADTRUSTInstance(service.Service):
         except ipautil.CalledProcessError, e:
             root_logger.critical("Failed to add key for %s" % self.cifs_principal)
 
+    def srv_rec(self, host, port, prio):
+        return "%(prio)d 100 %(port)d %(host)s" % dict(host=host,prio=prio,port=port)
+
     def __add_dns_service_records(self):
         """
         Add DNS service records for Windows if DNS is enabled and the DNS zone
@@ -488,11 +507,12 @@ class ADTRUSTInstance(service.Service):
 
         zone = self.domain_name
         host = self.fqdn.split(".")[0]
+        priority = 0
 
         ipa_srv_rec = (
-            ("_ldap._tcp", ["0 100 389 %s" % host]),
-            ("_kerberos._tcp", ["0 100 88 %s" % host]),
-            ("_kerberos._udp", ["0 100 88 %s" % host])
+            ("_ldap._tcp", [self.srv_rec(host, 389, priority)], 389),
+            ("_kerberos._tcp", [self.srv_rec(host, 88, priority)], 88),
+            ("_kerberos._udp", [self.srv_rec(host, 88, priority)], 88),
         )
         win_srv_suffix = (".Default-First-Site-Name._sites.dc._msdcs",
                           ".dc._msdcs")
@@ -520,10 +540,12 @@ class ADTRUSTInstance(service.Service):
                     self.print_msg(" - %s%s"  % (srv, suff))
             return
 
-        for (srv, rdata) in ipa_srv_rec:
-            ipa_rdata = get_rr(zone, srv, "SRV")
-            if not ipa_rdata:
-                ipa_rdata = rdata
+        for (srv, rdata, port) in ipa_srv_rec:
+            cifs_rdata = list()
+            for fqdn in self.cifs_hosts:
+                cifs_srv = self.srv_rec(fqdn, port, priority)
+                cifs_rdata.append(cifs_srv)
+            cifs_rdata.extend(rdata)
 
             for suff in win_srv_suffix:
                 win_srv = srv+suff
@@ -531,7 +553,7 @@ class ADTRUSTInstance(service.Service):
                 if win_rdata:
                     for rec in win_rdata:
                         del_rr(zone, win_srv, "SRV", rec)
-                for rec in ipa_rdata:
+                for rec in cifs_rdata:
                     add_rr(zone, win_srv, "SRV", rec)
 
     def __configure_selinux_for_smbd(self):
@@ -574,7 +596,7 @@ class ADTRUSTInstance(service.Service):
             self.print_msg("Cannot modify /etc/krb5.conf")
 
         krbconf = ipaclient.ipachangeconf.IPAChangeConf("IPA Installer")
-        krbconf.setOptionAssignment(" = ")
+        krbconf.setOptionAssignment((" = ", " "))
         krbconf.setSectionNameDelimiters(("[", "]"))
         krbconf.setSubSectionDelimiters(("{", "}"))
         krbconf.setIndent(("", "  ", "    "))
@@ -620,7 +642,27 @@ class ADTRUSTInstance(service.Service):
             self.print_msg("'dns_lookup_kdc' already set to 'true', "
                            "nothing to do.")
 
+    def __check_replica(self):
+        try:
+            cifs_services = DN(api.env.container_service, self.suffix)
+            # Search for cifs services which also belong to adtrust agents, these are our DCs
+            res = self.admin_conn.get_entries(cifs_services,
+                ldap.SCOPE_ONELEVEL,
+                "(&(krbprincipalname=cifs/*@%s)(memberof=%s))" % (self.realm, str(self.smb_dn)))
+            if len(res) > 1:
+                # there are other CIFS services defined, we are not alone
+                for entry in res:
+                    managedBy = entry.single_value('managedBy', None)
+                    if managedBy:
+                        fqdn = DN(managedBy)['fqdn']
+                        if fqdn != unicode(self.fqdn):
+                            # this is CIFS service of a different host in our
+                            # REALM, we need to remember it to announce via
+                            # SRV records for _msdcs
+                            self.cifs_hosts.append(fqdn.split(".")[0])
 
+        except Exception, e:
+            root_logger.critical("Checking replicas for cifs principals failed with error '%s'" % e)
 
     def __start(self):
         try:
@@ -700,25 +742,27 @@ class ADTRUSTInstance(service.Service):
                              api.env.container_service,
                              self.suffix)
         self.selinux_booleans = ["samba_portmapper"]
+        self.cifs_hosts = list()
 
         self.__setup_sub_dict()
 
     def find_local_id_range(self):
         self.ldap_connect()
 
-        if self.admin_conn.search_s(DN(api.env.container_ranges, self.suffix),
-                                    ldap.SCOPE_ONELEVEL,
-                                    "objectclass=ipaDomainIDRange"):
+        if self.admin_conn.get_entries(
+                DN(api.env.container_ranges, self.suffix),
+                ldap.SCOPE_ONELEVEL,
+                "(objectclass=ipaDomainIDRange)"):
             return
 
         try:
-            entry = self.admin_conn.getEntry(DN(('cn', 'admins'), api.env.container_group, self.suffix),
-                                             ldap.SCOPE_BASE)
+            entry = self.admin_conn.get_entry(
+                DN(('cn', 'admins'), api.env.container_group, self.suffix))
         except errors.NotFound:
             raise ValueError("No local ID range and no admins group found.\n" \
                              "Add local ID range manually and try again!")
 
-        base_id = int(entry.getValue('gidNumber'))
+        base_id = int(entry.single_value('gidNumber'))
         id_range_size = 200000
 
         id_filter = "(&" \
@@ -729,20 +773,22 @@ class ADTRUSTInstance(service.Service):
                         "(gidNumber<=%d)(gidNumner>=%d)))" % \
                      ((base_id - 1), (base_id + id_range_size),
                       (base_id - 1), (base_id + id_range_size))
-        if self.admin_conn.search_s("cn=accounts," + self.suffix,
-                                   ldap.SCOPE_SUBTREE, id_filter):
+        if self.admin_conn.get_entries(DN(('cn', 'accounts'), self.suffix),
+                                       ldap.SCOPE_SUBTREE, id_filter):
             raise ValueError("There are objects with IDs out of the expected" \
                              "range.\nAdd local ID range manually and try " \
                              "again!")
 
-        entry = ipaldap.Entry(DN(('cn', ('%s_id_range' % self.realm)),
-                                 api.env.container_ranges,
-                                 self.suffix))
-        entry.setValue('objectclass', 'ipaDomainIDRange')
-        entry.setValue('cn', ('%s_id_range' % self.realm))
-        entry.setValue('ipaBaseID', str(base_id))
-        entry.setValue('ipaIDRangeSize', str(id_range_size))
-        self.admin_conn.addEntry(entry)
+        entry = self.admin_conn.make_entry(
+            DN(
+                ('cn', ('%s_id_range' % self.realm)),
+                api.env.container_ranges, self.suffix),
+            objectclass=['ipaDomainIDRange'],
+            cn=['%s_id_range' % self.realm],
+            ipaBaseID=[str(base_id)],
+            ipaIDRangeSize=[str(id_range_size)],
+        )
+        self.admin_conn.add_entry(entry)
 
     def create_instance(self):
 
@@ -754,6 +800,7 @@ class ADTRUSTInstance(service.Service):
         self.step("creating samba config registry", self.__write_smb_registry)
         self.step("writing samba config file", self.__write_smb_conf)
         self.step("adding cifs Kerberos principal", self.__setup_principal)
+        self.step("check for cifs services defined on other replicas", self.__check_replica)
         self.step("adding cifs principal to S4U2Proxy targets", self.__add_s4u2proxy_target)
         self.step("adding admin(group) SIDs", self.__add_admin_sids)
         self.step("adding RID bases", self.__add_rid_bases)
