@@ -23,31 +23,32 @@ import pwd
 import netaddr
 import re
 
-import installutils
 import ldap
+
+import installutils
 import service
-from ipaserver import ipaldap
+from ipaserver.plugins import ldap2
 from ipaserver.install.dsinstance import realm_to_serverid
-from ipaserver.install.cainstance import IPA_CA_CNAME
-from ipaserver.install.installutils import resolve_host
-from ipapython import sysrestore
-from ipapython import ipautil
-from ipalib.parameters import IA5Str
+from ipaserver.install.cainstance import IPA_CA_RECORD
+from ipapython import sysrestore, ipautil, ipaldap
+from ipapython.ipa_log_manager import *
+from ipapython.dn import DN
+import ipalib
+from ipalib import api, errors
 from ipalib.util import (validate_zonemgr, normalize_zonemgr,
         get_dns_forward_zone_update_policy, get_dns_reverse_zone_update_policy,
         normalize_zone, get_reverse_zone_default, zone_is_reverse)
-from ipapython.ipa_log_manager import *
-from ipalib.text import _
-
-import ipalib
-from ipalib import api, util, errors
-from ipapython.dn import DN
 
 NAMED_CONF = '/etc/named.conf'
 RESOLV_CONF = '/etc/resolv.conf'
 
-named_conf_ipa_re = re.compile(r'(?P<indent>\s*)arg\s+"(?P<name>\S+)\s(?P<value>[^"]+)";')
-named_conf_ipa_template = "%(indent)sarg \"%(name)s %(value)s\";\n"
+named_conf_section_ipa_start_re = re.compile('\s*dynamic-db\s+"ipa"\s+{')
+named_conf_section_options_start_re = re.compile('\s*options\s+{')
+named_conf_section_end_re = re.compile('};')
+named_conf_arg_ipa_re = re.compile(r'(?P<indent>\s*)arg\s+"(?P<name>\S+)\s(?P<value>[^"]+)";')
+named_conf_arg_options_re = re.compile(r'(?P<indent>\s*)(?P<name>\S+)\s+"(?P<value>[^"]+)"\s*;')
+named_conf_arg_ipa_template = "%(indent)sarg \"%(name)s %(value)s\";\n"
+named_conf_arg_options_template = "%(indent)s%(name)s \"%(value)s\";\n"
 
 def check_inst(unattended):
     has_bind = True
@@ -89,26 +90,36 @@ def named_conf_exists():
             return True
     return False
 
-def named_conf_get_directive(name):
+NAMED_SECTION_OPTIONS = "options"
+NAMED_SECTION_IPA = "ipa"
+def named_conf_get_directive(name, section=NAMED_SECTION_IPA):
     """Get a configuration option in bind-dyndb-ldap section of named.conf"""
+    if section == NAMED_SECTION_IPA:
+        named_conf_section_start_re = named_conf_section_ipa_start_re
+        named_conf_arg_re = named_conf_arg_ipa_re
+    elif section == NAMED_SECTION_OPTIONS:
+        named_conf_section_start_re = named_conf_section_options_start_re
+        named_conf_arg_re = named_conf_arg_options_re
+    else:
+        raise NotImplementedError('Section "%s" is not supported' % section)
 
     with open(NAMED_CONF, 'r') as f:
-        ipa_section = False
+        target_section = False
         for line in f:
-            if line.startswith('dynamic-db "ipa"'):
-                ipa_section = True
+            if named_conf_section_start_re.match(line):
+                target_section = True
                 continue
-            if line.startswith('};'):
-                if ipa_section:
+            if named_conf_section_end_re.match(line):
+                if target_section:
                     break
 
-            if ipa_section:
-                match = named_conf_ipa_re.match(line)
+            if target_section:
+                match = named_conf_arg_re.match(line)
 
                 if match and name == match.group('name'):
                     return match.group('value')
 
-def named_conf_set_directive(name, value):
+def named_conf_set_directive(name, value, section=NAMED_SECTION_IPA):
     """
     Set configuration option in bind-dyndb-ldap section of named.conf.
 
@@ -120,25 +131,37 @@ def named_conf_set_directive(name, value):
     """
     new_lines = []
 
+    if section == NAMED_SECTION_IPA:
+        named_conf_section_start_re = named_conf_section_ipa_start_re
+        named_conf_arg_re = named_conf_arg_ipa_re
+        named_conf_arg_template = named_conf_arg_ipa_template
+    elif section == NAMED_SECTION_OPTIONS:
+        named_conf_section_start_re = named_conf_section_options_start_re
+        named_conf_arg_re = named_conf_arg_options_re
+        named_conf_arg_template = named_conf_arg_options_template
+    else:
+        raise NotImplementedError('Section "%s" is not supported' % section)
+
     with open(NAMED_CONF, 'r') as f:
-        ipa_section = False
+        target_section = False
         matched = False
         last_indent = "\t"
         for line in f:
-            if line.startswith('dynamic-db "ipa"'):
-                ipa_section = True
-            if line.startswith('};'):
-                if ipa_section and not matched:
+            if named_conf_section_start_re.match(line):
+                target_section = True
+            if named_conf_section_end_re.match(line):
+                if target_section and not matched and \
+                        value is not None:
                     # create a new conf
-                    new_conf = named_conf_ipa_template \
+                    new_conf = named_conf_arg_template \
                             % dict(indent=last_indent,
                                    name=name,
                                    value=value)
                     new_lines.append(new_conf)
-                ipa_section = False
+                target_section = False
 
-            if ipa_section and not matched:
-                match = named_conf_ipa_re.match(line)
+            if target_section and not matched:
+                match = named_conf_arg_re.match(line)
 
                 if match:
                     last_indent = match.group('indent')
@@ -147,7 +170,7 @@ def named_conf_set_directive(name, value):
                         if value is not None:
                             if not isinstance(value, basestring):
                                 value = str(value)
-                            new_conf = named_conf_ipa_template \
+                            new_conf = named_conf_arg_template \
                                     % dict(indent=last_indent,
                                            name=name,
                                            value=value)
@@ -170,8 +193,8 @@ def dns_container_exists(fqdn, suffix, dm_password=None, ldapi=False, realm=None
         """
         assert isinstance(dn, DN)
         try:
-            conn.search_ext_s(dn, ldap.SCOPE_BASE)
-        except ldap.NO_SUCH_OBJECT:
+            conn.get_entry(dn)
+        except errors.NotFound:
             return False
         else:
             return True
@@ -193,7 +216,7 @@ def dns_container_exists(fqdn, suffix, dm_password=None, ldapi=False, realm=None
         raise RuntimeError('LDAP server on %s is not responding. Is IPA installed?' % fqdn)
 
     ret = object_exists(DN(('cn', 'dns'), suffix))
-    conn.unbind_s()
+    conn.unbind()
 
     return ret
 
@@ -319,11 +342,10 @@ def add_fwd_rr(zone, host, ip_address):
 
 def add_ptr_rr(zone, ip_address, fqdn, dns_backup=None):
     name = get_reverse_record_name(zone, ip_address)
-    add_rr(zone, name, "PTR", fqdn+".", dns_backup)
+    add_rr(zone, name, "PTR", normalize_zone(fqdn), dns_backup)
 
 def add_ns_rr(zone, hostname, dns_backup=None, force=True):
-    if not hostname.endswith('.'):
-        hostname += '.'
+    hostname = normalize_zone(hostname)
     add_rr(zone, "@", "NS", hostname, dns_backup=dns_backup,
             force=force)
 
@@ -334,6 +356,13 @@ def del_rr(zone, name, type, rdata):
     except (errors.NotFound, errors.AttrValueNotFound, errors.EmptyModlist):
         pass
 
+def del_fwd_rr(zone, host, ip_address):
+    addr = netaddr.IPAddress(ip_address)
+    if addr.version == 4:
+        del_rr(zone, host, "A", ip_address)
+    elif addr.version == 6:
+        del_rr(zone, host, "AAAA", ip_address)
+
 def get_rr(zone, name, type):
     rectype = '%srecord' % unicode(type.lower())
     ret = api.Command.dnsrecord_find(unicode(zone), unicode(name))
@@ -344,6 +373,9 @@ def get_rr(zone, name, type):
 
     return []
 
+def get_fwd_rr(zone, host):
+    return [x for t in ("A", "AAAA") for x in get_rr(zone, host, t)]
+
 def zonemgr_callback(option, opt_str, value, parser):
     """
     Properly validate and convert --zonemgr Option to IA5String
@@ -453,11 +485,15 @@ class BindInstance(service.Service):
         else:
             self.zonemgr = normalize_zonemgr(zonemgr)
 
+        self.first_instance = not dns_container_exists(
+            self.fqdn, self.suffix, realm=self.realm, ldapi=True,
+            dm_password=self.dm_password)
+
         self.__setup_sub_dict()
 
     @property
     def host_domain(self):
-        return '.'.join(self.fqdn.split(".")[1:])
+        return self.fqdn.split(".", 1)[1]
 
     @property
     def host_in_rr(self):
@@ -490,17 +526,20 @@ class BindInstance(service.Service):
         if installutils.record_in_hosts(self.ip_address, self.fqdn) is None:
             installutils.add_record_to_hosts(self.ip_address, self.fqdn)
 
-        if not dns_container_exists(self.fqdn, self.suffix, realm=self.realm,
-                                    ldapi=True, dm_password=self.dm_password):
+        if self.first_instance:
             self.step("adding DNS container", self.__setup_dns_container)
+
         if dns_zone_exists(self.domain):
             self.step("adding NS record to the zone", self.__add_self_ns)
         else:
             self.step("setting up our zone", self.__setup_zone)
         if self.reverse_zone is not None:
             self.step("setting up reverse zone", self.__setup_reverse_zone)
+
         self.step("setting up our own record", self.__add_self)
-        self.step("setting up CA CNAME record", self.__add_ipa_ca_cname)
+        if self.first_instance:
+            self.step("setting up records for other masters", self.__add_others)
+        self.step("setting up CA record", self.__add_ipa_ca_record)
 
         self.step("setting up kerberos principal", self.__setup_principal)
         self.step("setting up named.conf", self.__setup_named_conf)
@@ -545,6 +584,15 @@ class BindInstance(service.Service):
         else:
             optional_ntp = ""
 
+        addr = netaddr.IPAddress(self.ip_address)
+        if addr.version in (4, 6):
+            ipa_ca = "%s\t\t\tIN %s\t\t\t%s\n" % (
+                IPA_CA_RECORD,
+                "A" if addr.version == 4 else "AAAA",
+                self.ip_address)
+        else:
+            ipa_ca = ""
+
         boolean_var = {}
         for var in ('persistent_search', 'serial_autoincrement'):
             boolean_var[var] = "yes" if getattr(self, var, False) else "no"
@@ -560,7 +608,7 @@ class BindInstance(service.Service):
                              OPTIONAL_NTP=optional_ntp,
                              ZONEMGR=self.zonemgr,
                              ZONE_REFRESH=self.zone_refresh,
-                             IPA_CA_CNAME=IPA_CA_CNAME,
+                             IPA_CA_RECORD=ipa_ca,
                              PERSISTENT_SEARCH=boolean_var['persistent_search'],
                              SERIAL_AUTOINCREMENT=boolean_var['serial_autoincrement'],)
 
@@ -570,13 +618,6 @@ class BindInstance(service.Service):
     def __setup_zone(self):
         nameserver_ip_address = self.ip_address
         if not self.host_in_default_domain():
-            # add DNS domain for host first
-            root_logger.debug("Host domain (%s) is different from DNS domain (%s)!" \
-                    % (self.host_domain, self.domain))
-            root_logger.debug("Add DNS zone for host first.")
-
-            add_zone(self.host_domain, self.zonemgr, dns_backup=self.dns_backup,
-                    ns_hostname=api.env.host, ns_ip_address=self.ip_address, force=True)
             # Nameserver is in self.host_domain, no forward record added to self.domain
             nameserver_ip_address = None
         # Always use force=True as named is not set up yet
@@ -584,62 +625,139 @@ class BindInstance(service.Service):
                 ns_hostname=api.env.host, ns_ip_address=nameserver_ip_address,
                 force=True)
 
+        add_rr(self.domain, "_kerberos", "TXT", self.realm)
+
     def __add_self_ns(self):
         add_ns_rr(self.domain, api.env.host, self.dns_backup, force=True)
 
-    def __add_ipa_ca_cname(self):
-        if self.ca_configured is False:
-            root_logger.debug("CA is not configured, skip this step")
-            return
-        elif self.ca_configured is None:
-            # we do not know if CA is configured for this host and we can
-            # add the CA CNAME record. So we need to find out
-            root_logger.debug("Check if CA is enabled for this host")
-            base_dn = DN(('cn', api.env.host), ('cn', 'masters'), ('cn', 'ipa'),
-                         ('cn', 'etc'), api.env.basedn)
-            ldap_filter = '(&(objectClass=ipaConfigObject)(cn=CA))'
-            try:
-                api.Backend.ldap2.find_entries(filter=ldap_filter, base_dn=base_dn)
-            except ipalib.errors.NotFound:
-                # CA is not configured
-                root_logger.debug("CA is not configured")
-                return
-            else:
-                root_logger.debug("CA is configured for this host, continue")
-
-        add_rr(self.domain, IPA_CA_CNAME, "CNAME", self.host_in_rr)
-
-    def __add_self(self):
-        zone = self.domain
-        resource_records = (
-            ("_ldap._tcp", "SRV", "0 100 389 %s" % self.host_in_rr),
-            ("_kerberos", "TXT", self.realm),
-            ("_kerberos._tcp", "SRV", "0 100 88 %s" % self.host_in_rr),
-            ("_kerberos._udp", "SRV", "0 100 88 %s" % self.host_in_rr),
-            ("_kerberos-master._tcp", "SRV", "0 100 88 %s" % self.host_in_rr),
-            ("_kerberos-master._udp", "SRV", "0 100 88 %s" % self.host_in_rr),
-            ("_kpasswd._tcp", "SRV", "0 100 464 %s" % self.host_in_rr),
-            ("_kpasswd._udp", "SRV", "0 100 464 %s" % self.host_in_rr),
-        )
-
-        for (host, type, rdata) in resource_records:
-            if type == "SRV":
-                add_rr(zone, host, type, rdata, self.dns_backup)
-            else:
-                add_rr(zone, host, type, rdata)
-        if self.ntp:
-            add_rr(zone, "_ntp._udp", "SRV", "0 100 123 %s" % self.host_in_rr)
-
-        # Add forward and reverse records to self
-        add_fwd_rr(self.host_domain, self.host, self.ip_address)
-        if self.reverse_zone is not None and dns_zone_exists(self.reverse_zone):
-            add_ptr_rr(self.reverse_zone, self.ip_address, self.fqdn)
-
     def __setup_reverse_zone(self):
         # Always use force=True as named is not set up yet
         add_zone(self.reverse_zone, self.zonemgr, ns_hostname=api.env.host,
                 dns_backup=self.dns_backup, force=True)
 
+    def __add_master_records(self, fqdn, addrs):
+        host, zone = fqdn.split(".", 1)
+
+        if normalize_zone(zone) == normalize_zone(self.domain):
+            host_in_rr = host
+        else:
+            host_in_rr = normalize_zone(fqdn)
+
+        srv_records = (
+            ("_ldap._tcp", "0 100 389 %s" % host_in_rr),
+            ("_kerberos._tcp", "0 100 88 %s" % host_in_rr),
+            ("_kerberos._udp", "0 100 88 %s" % host_in_rr),
+            ("_kerberos-master._tcp", "0 100 88 %s" % host_in_rr),
+            ("_kerberos-master._udp", "0 100 88 %s" % host_in_rr),
+            ("_kpasswd._tcp", "0 100 464 %s" % host_in_rr),
+            ("_kpasswd._udp", "0 100 464 %s" % host_in_rr),
+        )
+        if self.ntp:
+            srv_records += (
+                ("_ntp._udp", "0 100 123 %s" % host_in_rr),
+            )
+
+        for (rname, rdata) in srv_records:
+            add_rr(self.domain, rname, "SRV", rdata, self.dns_backup)
+
+        if not dns_zone_exists(zone):
+            # add DNS domain for host first
+            root_logger.debug(
+                "Host domain (%s) is different from DNS domain (%s)!" % (
+                    zone, self.domain))
+            root_logger.debug("Add DNS zone for host first.")
+
+            if normalize_zone(zone) == normalize_zone(self.host_domain):
+                ns_ip_address = self.ip_address
+            else:
+                ns_ip_address = None
+
+            add_zone(zone, self.zonemgr, dns_backup=self.dns_backup,
+                     ns_hostname=self.fqdn, ns_ip_address=ns_ip_address,
+                     force=True)
+
+        # Add forward and reverse records to self
+        for addr in addrs:
+            add_fwd_rr(zone, host, addr)
+
+            reverse_zone = find_reverse_zone(addr)
+            if reverse_zone:
+                add_ptr_rr(reverse_zone, addr, fqdn)
+
+    def __add_self(self):
+        self.__add_master_records(self.fqdn, [self.ip_address])
+
+    def __add_others(self):
+        entries = self.admin_conn.get_entries(
+            DN(('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'),
+               self.suffix),
+            self.admin_conn.SCOPE_ONELEVEL, None, ['dn'])
+
+        for entry in entries:
+            fqdn = entry.dn[0]['cn']
+            if fqdn == self.fqdn:
+                continue
+
+            addrs = installutils.resolve_host(fqdn)
+
+            root_logger.debug("Adding DNS records for master %s" % fqdn)
+            self.__add_master_records(fqdn, addrs)
+
+    def __add_ipa_ca_records(self, fqdn, addrs, ca_configured):
+        if ca_configured is False:
+            root_logger.debug("CA is not configured")
+            return
+        elif ca_configured is None:
+            # we do not know if CA is configured for this host and we can
+            # add the CA record. So we need to find out
+            root_logger.debug("Check if CA is enabled for this host")
+            base_dn = DN(('cn', fqdn), ('cn', 'masters'), ('cn', 'ipa'),
+                         ('cn', 'etc'), api.env.basedn)
+            ldap_filter = '(&(objectClass=ipaConfigObject)(cn=CA))'
+            try:
+                api.Backend.ldap2.find_entries(filter=ldap_filter, base_dn=base_dn)
+            except ipalib.errors.NotFound:
+                root_logger.debug("CA is not configured")
+                return
+            else:
+                root_logger.debug("CA is configured for this host")
+
+        try:
+            for addr in addrs:
+                add_fwd_rr(self.domain, IPA_CA_RECORD, addr)
+        except errors.ValidationError:
+            # there is a CNAME record in ipa-ca, we can't add A/AAAA records
+            pass
+
+    def __add_ipa_ca_record(self):
+        self.__add_ipa_ca_records(self.fqdn, [self.ip_address],
+                                  self.ca_configured)
+
+        if self.first_instance:
+            ldap = api.Backend.ldap2
+            try:
+                entries = ldap.get_entries(
+                    DN(('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'),
+                       api.env.basedn),
+                    ldap.SCOPE_SUBTREE, '(&(objectClass=ipaConfigObject)(cn=CA))',
+                    ['dn'])
+            except errors.NotFound:
+                root_logger.debug('No server with CA found')
+                entries = []
+
+            for entry in entries:
+                fqdn = entry.dn[1]['cn']
+                if fqdn == self.fqdn:
+                    continue
+
+                host, zone = fqdn.split('.', 1)
+                if dns_zone_exists(zone):
+                    addrs = get_fwd_rr(zone, host)
+                else:
+                    addrs = installutils.resolve_host(fqdn)
+
+                self.__add_ipa_ca_records(fqdn, addrs, True)
+
     def __setup_principal(self):
         dns_principal = "DNS/" + self.fqdn + "@" + self.realm
         installutils.kadmin_addprinc(dns_principal)
@@ -718,25 +836,75 @@ class BindInstance(service.Service):
         self.ntp = ntp
         self.reverse_zone = reverse_zone
         self.ca_configured = ca_configured
+        self.first_instance = False
 
         self.__add_self()
-        self.__add_ipa_ca_cname()
+        self.__add_ipa_ca_record()
+
+    def add_ipa_ca_dns_records(self, fqdn, domain_name, ca_configured=True):
+        host, zone = fqdn.split(".", 1)
+        if dns_zone_exists(zone):
+            addrs = get_fwd_rr(zone, host)
+        else:
+            addrs = installutils.resolve_host(fqdn)
 
-    def add_ipa_ca_cname(self, fqdn, domain_name, ca_configured=True):
-        self.host = fqdn.split(".")[0]
-        self.fqdn = fqdn
         self.domain = domain_name
-        self.ca_configured = ca_configured
-        self.__add_ipa_ca_cname()
+
+        self.__add_ipa_ca_records(fqdn, addrs, ca_configured)
+
+    def convert_ipa_ca_cnames(self, domain_name):
+        # get ipa-ca CNAMEs
+        cnames = get_rr(domain_name, IPA_CA_RECORD, "CNAME")
+        if not cnames:
+            return
+
+        root_logger.info('Converting IPA CA CNAME records to A/AAAA records')
+
+        # create CNAME to FQDN mapping
+        cname_fqdn = {}
+        for cname in cnames:
+            if cname.endswith('.'):
+                fqdn = cname[:-1]
+            else:
+                fqdn = '%s.%s' % (cname, domain_name)
+            cname_fqdn[cname] = fqdn
+
+        # get FQDNs of all IPA masters
+        ldap = api.Backend.ldap2
+        try:
+            entries = ldap.get_entries(
+                DN(('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'),
+                   api.env.basedn),
+                ldap.SCOPE_ONELEVEL, None, ['cn'])
+            masters = set(e['cn'][0] for e in entries)
+        except errors.NotFound:
+            masters = set()
+
+        # check if all CNAMEs point to IPA masters
+        for cname in cnames:
+            fqdn = cname_fqdn[cname]
+            if fqdn not in masters:
+                root_logger.warning(
+                    "Cannot convert IPA CA CNAME records to A/AAAA records, "
+                    "please convert them manually if necessary")
+                return
+
+        # delete all CNAMEs
+        for cname in cnames:
+            del_rr(domain_name, IPA_CA_RECORD, "CNAME", cname)
+
+        # add A/AAAA records
+        for cname in cnames:
+            fqdn = cname_fqdn[cname]
+            self.add_ipa_ca_dns_records(fqdn, domain_name, None)
 
     def remove_master_dns_records(self, fqdn, realm_name, domain_name):
-        host = fqdn.split(".")[0]
+        host, zone = fqdn.split(".", 1)
         self.host = host
         self.fqdn = fqdn
         self.domain = domain_name
         suffix = ipautil.realm_to_suffix(realm_name)
 
-        zone = domain_name
         resource_records = (
             ("_ldap._tcp", "SRV", "0 100 389 %s" % self.host_in_rr),
             ("_kerberos._tcp", "SRV", "0 100 88 %s" % self.host_in_rr),
@@ -746,23 +914,32 @@ class BindInstance(service.Service):
             ("_kpasswd._tcp", "SRV", "0 100 464 %s" % self.host_in_rr),
             ("_kpasswd._udp", "SRV", "0 100 464 %s" % self.host_in_rr),
             ("_ntp._udp", "SRV", "0 100 123 %s" % self.host_in_rr),
-            (IPA_CA_CNAME, "CNAME", self.host_in_rr),
-            ("@", "NS", fqdn+"."),
+            ("@", "NS", normalize_zone(fqdn)),
         )
 
         for (record, type, rdata) in resource_records:
-            del_rr(zone, record, type, rdata)
+            del_rr(self.domain, record, type, rdata)
 
-        areclist = [("A", x) for x in get_rr(zone, host, "A")] + [("AAAA", x) for x in get_rr(zone, host, "AAAA")]
-        for (type, rdata) in areclist:
-            del_rr(zone, host, type, rdata)
+        areclist = get_fwd_rr(zone, host)
+        for rdata in areclist:
+            del_fwd_rr(zone, host, rdata)
 
             rzone = find_reverse_zone(rdata)
             if rzone is not None:
                 record = get_reverse_record_name(rzone, rdata)
-                del_rr(rzone, record, "PTR", fqdn+".")
+                del_rr(rzone, record, "PTR", normalize_zone(fqdn))
                 # remove also master NS record from the reverse zone
-                del_rr(rzone, "@", "NS", fqdn+".")
+                del_rr(rzone, "@", "NS", normalize_zone(fqdn))
+
+    def remove_ipa_ca_dns_records(self, fqdn, domain_name):
+        host, zone = fqdn.split(".", 1)
+        if dns_zone_exists(zone):
+            addrs = get_fwd_rr(zone, host)
+        else:
+            addrs = installutils.resolve_host(fqdn)
+
+        for addr in addrs:
+            del_fwd_rr(domain_name, IPA_CA_RECORD, addr)
 
     def check_global_configuration(self):
         """
