@@ -19,32 +19,26 @@
 #
 
 import shutil
-from ipapython.ipa_log_manager import *
 import pwd
-import glob
 import sys
 import os
 import re
 import time
 import tempfile
+import base64
+import stat
 
-from ipapython import ipautil
-from ipapython import sysrestore
+from ipapython.ipa_log_manager import *
+from ipapython import ipautil, sysrestore, dogtag, ipaldap
 from ipapython import services as ipaservices
-from ipapython import dogtag
-
 import service
 import installutils
 import certs
 import ldap
-from ipaserver import ipaldap
 from ipaserver.install import ldapupdate
-from ipaserver.install import httpinstance
 from ipaserver.install import replication
-from ipalib import util, errors
+from ipalib import errors
 from ipapython.dn import DN
-from ipaserver.plugins.ldap2 import ldap2
-import base64
 
 SERVER_ROOT_64 = "/usr/lib64/dirsrv"
 SERVER_ROOT_32 = "/usr/lib/dirsrv"
@@ -161,14 +155,15 @@ info: IPA V2.0
 """
 
 class DsInstance(service.Service):
-    def __init__(self, realm_name=None, domain_name=None, dm_password=None, fstore=None):
+    def __init__(self, realm_name=None, domain_name=None, dm_password=None,
+                 fstore=None, cert_nickname='Server-Cert'):
         service.Service.__init__(self, "dirsrv",
             service_desc="directory server",
             dm_password=dm_password,
             ldapi=False,
             autobind=service.DISABLED
             )
-        self.nickname = 'Server-Cert'
+        self.nickname = cert_nickname
         self.dm_password = dm_password
         self.realm_name = realm_name
         self.sub_dict = None
@@ -209,6 +204,7 @@ class DsInstance(service.Service):
         self.step("configuring uniqueness plugin", self.__set_unique_attrs)
         self.step("configuring uuid plugin", self.__config_uuid_module)
         self.step("configuring modrdn plugin", self.__config_modrdn_module)
+        self.step("configuring DNS plugin", self.__config_dns_module)
         self.step("enabling entryUSN plugin", self.__enable_entryusn)
         self.step("configuring lockout plugin", self.__config_lockout_module)
         self.step("creating indices", self.__create_indices)
@@ -218,6 +214,7 @@ class DsInstance(service.Service):
         self.step("configuring certmap.conf", self.__certmap_conf)
         self.step("configure autobind for root", self.__root_autobind)
         self.step("configure new location for managed entries", self.__repoint_managed_entries)
+        self.step("configure dirsrv ccache", self.configure_dirsrv_ccache)
         self.step("restarting directory server", self.__restart_instance)
 
     def __common_post_setup(self):
@@ -225,35 +222,42 @@ class DsInstance(service.Service):
         self.step("adding master entry", self.__add_master_entry)
         self.step("configuring Posix uid/gid generation",
                   self.__config_uidgid_gen)
+        self.step("adding replication acis", self.__add_replication_acis)
         self.step("enabling compatibility plugin",
                   self.__enable_compat_plugin)
         self.step("tuning directory server", self.__tuning)
 
         self.step("configuring directory to start on boot", self.__enable)
 
-    def create_instance(self, realm_name, fqdn, domain_name,
-                        dm_password, pkcs12_info=None, self_signed_ca=False,
-                        idstart=1100, idmax=999999, subject_base=None,
-                        hbac_allow=True):
+    def init_info(self, realm_name, fqdn, domain_name, dm_password,
+                  subject_base, idstart, idmax, pkcs12_info, ca_file=None):
         self.realm_name = realm_name.upper()
         self.serverid = realm_to_serverid(self.realm_name)
         self.suffix = ipautil.realm_to_suffix(self.realm_name)
         self.fqdn = fqdn
         self.dm_password = dm_password
         self.domain = domain_name
-        self.pkcs12_info = pkcs12_info
-        self.self_signed_ca = self_signed_ca
-        self.idstart = idstart
-        self.idmax = idmax
         self.principal = "ldap/%s@%s" % (self.fqdn, self.realm_name)
         self.subject_base = subject_base
+        self.idstart = idstart
+        self.idmax = idmax
+        self.pkcs12_info = pkcs12_info
+        self.ca_file = ca_file
 
         self.__setup_sub_dict()
+
+    def create_instance(self, realm_name, fqdn, domain_name,
+                        dm_password, pkcs12_info=None,
+                        idstart=1100, idmax=999999, subject_base=None,
+                        hbac_allow=True, ca_file=None):
+        self.init_info(
+            realm_name, fqdn, domain_name, dm_password,
+            subject_base, idstart, idmax, pkcs12_info, ca_file=ca_file)
+
         self.__common_setup()
 
         self.step("adding default layout", self.__add_default_layout)
         self.step("adding delegation layout", self.__add_delegation_layout)
-        self.step("adding replication acis", self.__add_replication_acis)
         self.step("creating container for managed entries", self.__managed_entries)
         self.step("configuring user private groups", self.__user_private_groups)
         self.step("configuring netgroups from hostgroups", self.__host_nis_groups)
@@ -262,38 +266,29 @@ class DsInstance(service.Service):
         self.step("adding range check plugin", self.__add_range_check_plugin)
         if hbac_allow:
             self.step("creating default HBAC rule allow_all", self.add_hbac)
-        self.step("Upload CA cert to the directory", self.__upload_ca_cert)
 
         self.__common_post_setup()
 
         self.start_creation(runtime=60)
 
     def create_replica(self, realm_name, master_fqdn, fqdn,
-                       domain_name, dm_password, pkcs12_info=None):
-        self.realm_name = realm_name.upper()
-        self.serverid = realm_to_serverid(self.realm_name)
-        self.suffix = ipautil.realm_to_suffix(self.realm_name)
-        self.master_fqdn = master_fqdn
-        self.fqdn = fqdn
-        self.dm_password = dm_password
-        self.domain = domain_name
-        self.pkcs12_info = pkcs12_info
-        self.principal = "ldap/%s@%s" % (self.fqdn, self.realm_name)
-
-        self.self_signed_ca = False
-        self.subject_base = None
+                       domain_name, dm_password, pkcs12_info=None,
+                       ca_file=None):
         # idstart and idmax are configured so that the range is seen as
         # depleted by the DNA plugin and the replica will go and get a
         # new range from the master.
         # This way all servers use the initially defined range by default.
-        self.idstart = 1101
-        self.idmax = 1100
+        idstart = 1101
+        idmax = 1100
+
+        self.init_info(
+            realm_name, fqdn, domain_name, dm_password, None,
+            idstart, idmax, pkcs12_info, ca_file=ca_file)
+        self.master_fqdn = master_fqdn
 
-        self.__setup_sub_dict()
         self.__common_setup(True)
 
         self.step("setting up initial replication", self.__setup_replica)
-        self.step("adding replication acis", self.__add_replication_acis)
         # See LDIFs for automember configuration during replica install
         self.step("setting Auto Member configuration", self.__add_replica_automember_config)
         self.step("enabling S4U2Proxy delegation", self.__setup_s4u2proxy)
@@ -416,7 +411,8 @@ class DsInstance(service.Service):
                              "60basev3.ldif",
                              "60ipadns.ldif",
                              "61kerberos-ipav3.ldif",
-                             "65ipasudo.ldif"):
+                             "65ipasudo.ldif",
+                             "70ipaotp.ldif"):
             target_fname = schema_dirname(self.serverid) + schema_fname
             shutil.copyfile(ipautil.SHARE_DIR + schema_fname, target_fname)
             os.chmod(target_fname, 0440)    # read access for dirsrv user/group
@@ -467,16 +463,17 @@ class DsInstance(service.Service):
         root_logger.debug("Waiting for memberof task to complete.")
         conn = ipaldap.IPAdmin(self.fqdn)
         if self.dm_password:
-            conn.simple_bind_s(DN(('cn', 'directory manager')), self.dm_password)
+            conn.do_simple_bind(
+                DN(('cn', 'directory manager')), self.dm_password)
         else:
             conn.do_sasl_gssapi_bind()
-        conn.checkTask(dn, dowait=True)
+        replication.wait_for_task(conn, dn)
         conn.unbind()
 
     def apply_updates(self):
         ld = ldapupdate.LDAPUpdate(dm_password=self.dm_password, sub_dict=self.sub_dict, plugins=True)
         files = ld.get_all_files(ldapupdate.UPDATES_DIR)
-        ld.update(files)
+        ld.update(files, ordered=True)
 
     def __add_referint_module(self):
         self._ldap_mod("referint-conf.ldif")
@@ -510,12 +507,33 @@ class DsInstance(service.Service):
         self._ldap_mod("modrdn-conf.ldif")
         self._ldap_mod("modrdn-krbprinc.ldif", self.sub_dict)
 
+    def __config_dns_module(self):
+        # Configure DNS plugin unconditionally as we would otherwise have
+        # troubles if other replica just configured DNS with ipa-dns-install
+        self._ldap_mod("ipa-dns-conf.ldif")
+
     def __config_lockout_module(self):
         self._ldap_mod("lockout-conf.ldif")
 
     def __repoint_managed_entries(self):
         self._ldap_mod("repoint-managed-entries.ldif", self.sub_dict)
 
+    def configure_dirsrv_ccache(self):
+        pent = pwd.getpwnam("dirsrv")
+        ccache = '/tmp/krb5cc_%d' % pent.pw_uid
+        filepath = '/etc/sysconfig/dirsrv'
+        if not os.path.exists(filepath):
+            # file doesn't exist; create it with correct ownership & mode
+            open(filepath, 'a').close()
+            os.chmod(filepath,
+                stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
+            os.chown(filepath, 0, 0)
+
+        replacevars = {'KRB5CCNAME': ccache}
+        old_values = ipautil.backup_config_and_replace_variables(
+            self.fstore, filepath, replacevars=replacevars)
+        ipaservices.restore_context(filepath)
+
     def __managed_entries(self):
         self._ldap_mod("managed-entries.ldif", self.sub_dict)
 
@@ -535,7 +553,8 @@ class DsInstance(service.Service):
         dirname = config_dirname(self.serverid)
         dsdb = certs.CertDB(self.realm_name, nssdir=dirname, subject_base=self.subject_base)
         if self.pkcs12_info:
-            dsdb.create_from_pkcs12(self.pkcs12_info[0], self.pkcs12_info[1])
+            dsdb.create_from_pkcs12(self.pkcs12_info[0], self.pkcs12_info[1],
+                                    ca_file=self.ca_file)
             server_certs = dsdb.find_server_certs()
             if len(server_certs) == 0:
                 raise RuntimeError("Could not find a suitable server cert in import in %s" % self.pkcs12_info[0])
@@ -543,25 +562,24 @@ class DsInstance(service.Service):
             # We only handle one server cert
             nickname = server_certs[0][0]
             self.dercert = dsdb.get_cert_from_db(nickname, pem=False)
-            dsdb.track_server_cert(nickname, self.principal, dsdb.passwd_fname, 'restart_dirsrv %s' % self.serverid )
+            dsdb.track_server_cert(nickname, self.principal, dsdb.passwd_fname,
+                                   'restart_dirsrv %s' % self.serverid)
         else:
-            nickname = "Server-Cert"
+            nickname = self.nickname
             cadb = certs.CertDB(self.realm_name, host_name=self.fqdn, subject_base=self.subject_base)
-            if self.self_signed_ca:
-                dsdb.create_from_cacert(cadb.cacert_fname, passwd=None)
-                self.dercert = dsdb.create_server_cert("Server-Cert", self.fqdn, cadb)
-                dsdb.track_server_cert("Server-Cert", self.principal, dsdb.passwd_fname, 'restart_dirsrv %s' % self.serverid)
-                dsdb.create_pin_file()
-            else:
-                # FIXME, need to set this nickname in the RA plugin
-                cadb.export_ca_cert('ipaCert', False)
-                dsdb.create_from_cacert(cadb.cacert_fname, passwd=None)
-                self.dercert = dsdb.create_server_cert("Server-Cert", self.fqdn, cadb)
-                dsdb.track_server_cert("Server-Cert", self.principal, dsdb.passwd_fname, 'restart_dirsrv %s' % self.serverid)
-                dsdb.create_pin_file()
+
+            # FIXME, need to set this nickname in the RA plugin
+            cadb.export_ca_cert('ipaCert', False)
+            dsdb.create_from_cacert(cadb.cacert_fname, passwd=None)
+            self.dercert = dsdb.create_server_cert(
+                nickname, self.fqdn, cadb)
+            dsdb.track_server_cert(
+                nickname, self.principal, dsdb.passwd_fname,
+                'restart_dirsrv %s' % self.serverid)
+            dsdb.create_pin_file()
 
         conn = ipaldap.IPAdmin(self.fqdn)
-        conn.simple_bind_s(DN(('cn', 'directory manager')), self.dm_password)
+        conn.do_simple_bind(DN(('cn', 'directory manager')), self.dm_password)
 
         mod = [(ldap.MOD_REPLACE, "nsSSLClientAuth", "allowed"),
                (ldap.MOD_REPLACE, "nsSSL3Ciphers",
@@ -574,30 +592,45 @@ class DsInstance(service.Service):
         mod = [(ldap.MOD_ADD, "nsslapd-security", "on")]
         conn.modify_s(DN(('cn', 'config')), mod)
 
-        entry = ipaldap.Entry(DN(('cn', 'RSA'), ('cn', 'encryption'), ('cn', 'config')))
-
-        entry.setValues("objectclass", "top", "nsEncryptionModule")
-        entry.setValues("cn", "RSA")
-        entry.setValues("nsSSLPersonalitySSL", nickname)
-        entry.setValues("nsSSLToken", "internal (software)")
-        entry.setValues("nsSSLActivation", "on")
-
-        conn.addEntry(entry)
+        entry = conn.make_entry(
+            DN(('cn', 'RSA'), ('cn', 'encryption'), ('cn', 'config')),
+            objectclass=["top", "nsEncryptionModule"],
+            cn=["RSA"],
+            nsSSLPersonalitySSL=[nickname],
+            nsSSLToken=["internal (software)"],
+            nsSSLActivation=["on"],
+        )
+        conn.add_entry(entry)
 
         conn.unbind()
 
         # check for open secure port 636 from now on
         self.open_ports.append(636)
 
-    def __upload_ca_cert(self):
+    def export_ca_cert(self, nickname, location):
+        dirname = config_dirname(self.serverid)
+        dsdb = certs.NSSDatabase(nssdir=dirname)
+        dsdb.export_pem_cert(nickname, location)
+
+    def upload_ca_cert(self, cacert_name=None):
         """
-        Upload the CA certificate in DER form in the LDAP directory.
+        Upload the CA certificate from the NSS database to the LDAP directory.
         """
 
         dirname = config_dirname(self.serverid)
         certdb = certs.CertDB(self.realm_name, nssdir=dirname, subject_base=self.subject_base)
 
-        dercert = certdb.get_cert_from_db(certdb.cacert_name, pem=False)
+        if cacert_name is None:
+            cacert_name = certdb.cacert_name
+        dercert = certdb.get_cert_from_db(cacert_name, pem=False)
+        self.upload_ca_dercert(dercert)
+
+    def upload_ca_dercert(self, dercert):
+        """Upload the CA DER certificate to the LDAP directory
+        """
+        # Note: Don't try to optimize if base64 data is already available.
+        # We want to re-encode using Python's b64encode to ensure the
+        # data is normalized (no extra newlines in the ldif)
         self.sub_dict['CADERCERT'] = base64.b64encode(dercert)
 
         self._ldap_mod('upload-cacert.ldif', self.sub_dict)
@@ -684,11 +717,7 @@ class DsInstance(service.Service):
 
         serverid = self.restore_state("serverid")
         if not serverid is None:
-            # drop the trailing / off the config_dirname so the directory
-            # will match what is in certmonger
-            dirname = config_dirname(serverid)[:-1]
-            dsdb = certs.CertDB(self.realm_name, nssdir=dirname)
-            dsdb.untrack_server_cert("Server-Cert")
+            self.stop_tracking_certificates(serverid)
             erase_ds_instance_data(serverid)
 
         # At one time we removed this user on uninstall. That can potentially
@@ -710,6 +739,16 @@ class DsInstance(service.Service):
             except Exception, e:
                 root_logger.error('Unable to restart ds instance %s: %s', ds_instance, e)
 
+    def stop_tracking_certificates(self, serverid=None):
+        if serverid is None:
+            serverid = self.get_state("serverid")
+        if not serverid is None:
+            # drop the trailing / off the config_dirname so the directory
+            # will match what is in certmonger
+            dirname = config_dirname(serverid)[:-1]
+            dsdb = certs.CertDB(self.realm_name, nssdir=dirname)
+            dsdb.untrack_server_cert(self.nickname)
+
     # we could probably move this function into the service.Service
     # class - it's very generic - all we need is a way to get an
     # instance of a particular Service
@@ -836,8 +875,8 @@ class DsInstance(service.Service):
 
         dn = DN(('cn', 'default'), ('ou', 'profile'), self.suffix)
         try:
-            entry = self.admin_conn.getEntry(dn, ldap.SCOPE_BASE, '(objectclass=*)')
-            srvlist = entry.getValue('defaultServerList', '')
+            entry = self.admin_conn.get_entry(dn)
+            srvlist = entry.single_value('defaultServerList', '')
             srvlist = srvlist.split()
             if not self.fqdn in srvlist:
                 srvlist.append(self.fqdn)
