@@ -20,19 +20,19 @@
 import os
 import os.path
 import tempfile
-from ipapython.ipa_log_manager import *
 import pwd
 import shutil
+import stat
 
 import service
 import certs
-import dsinstance
 import installutils
 from ipapython import sysrestore
 from ipapython import ipautil
 from ipapython import services as ipaservices
 from ipapython import dogtag
-from ipalib import util, api
+from ipapython.ipa_log_manager import *
+from ipalib import api
 
 HTTPD_DIR = "/etc/httpd"
 SSL_CONF = HTTPD_DIR + "/conf.d/ssl.conf"
@@ -51,23 +51,26 @@ class WebGuiInstance(service.SimpleServiceInstance):
         service.SimpleServiceInstance.__init__(self, "ipa_webgui")
 
 class HTTPInstance(service.Service):
-    def __init__(self, fstore = None):
+    def __init__(self, fstore=None, cert_nickname='Server-Cert'):
         service.Service.__init__(self, "httpd", service_desc="the web interface")
         if fstore:
             self.fstore = fstore
         else:
             self.fstore = sysrestore.FileStore('/var/lib/ipa/sysrestore')
 
+        self.cert_nickname = cert_nickname
+
     subject_base = ipautil.dn_attribute_property('_subject_base')
 
-    def create_instance(self, realm, fqdn, domain_name, dm_password=None, autoconfig=True, pkcs12_info=None, self_signed_ca=False, subject_base=None, auto_redirect=True):
+    def create_instance(self, realm, fqdn, domain_name, dm_password=None,
+                        autoconfig=True, pkcs12_info=None,
+                        subject_base=None, auto_redirect=True, ca_file=None):
         self.fqdn = fqdn
         self.realm = realm
         self.domain = domain_name
         self.dm_password = dm_password
         self.suffix = ipautil.realm_to_suffix(self.realm)
         self.pkcs12_info = pkcs12_info
-        self.self_signed_ca = self_signed_ca
         self.principal = "HTTP/%s@%s" % (self.fqdn, self.realm)
         self.dercert = None
         self.subject_base = subject_base
@@ -78,6 +81,7 @@ class HTTPInstance(service.Service):
             AUTOREDIR='' if auto_redirect else '#',
             CRL_PUBLISH_PATH=dogtag.install_constants.CRL_PUBLISH_PATH,
         )
+        self.ca_file = ca_file
 
         # get a connection to the DS
         self.ldap_connect()
@@ -96,6 +100,7 @@ class HTTPInstance(service.Service):
         self.step("creating a keytab for httpd", self.__create_http_keytab)
         self.step("clean up any existing httpd ccache", self.remove_httpd_ccache)
         self.step("configuring SELinux for httpd", self.configure_selinux_for_httpd)
+        self.step("configure httpd ccache", self.configure_httpd_ccache)
         self.step("restarting httpd", self.__start)
         self.step("configuring httpd to start on boot", self.__enable)
 
@@ -189,6 +194,22 @@ class HTTPInstance(service.Service):
         pent = pwd.getpwnam("apache")
         installutils.remove_file('/tmp/krb5cc_%d' % pent.pw_uid)
 
+    def configure_httpd_ccache(self):
+        pent = pwd.getpwnam("apache")
+        ccache = '/tmp/krb5cc_%d' % pent.pw_uid
+        filepath = '/etc/sysconfig/httpd'
+        if not os.path.exists(filepath):
+            # file doesn't exist; create it with correct ownership & mode
+            open(filepath, 'a').close()
+            os.chmod(filepath,
+                stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IROTH)
+            os.chown(filepath, 0, 0)
+
+        replacevars = {'KRB5CCNAME': ccache}
+        old_values = ipautil.backup_config_and_replace_variables(
+            self.fstore, filepath, replacevars=replacevars)
+        ipaservices.restore_context(filepath)
+
     def __configure_http(self):
         target_fname = '/etc/httpd/conf.d/ipa.conf'
         http_txt = ipautil.template_file(ipautil.SHARE_DIR + "ipa.conf", self.sub_dict)
@@ -232,33 +253,35 @@ class HTTPInstance(service.Service):
             print "Adding Include conf.d/ipa-rewrite to %s failed." % NSS_CONF
 
     def __setup_ssl(self):
-        fqdn = None
-        if not self.self_signed_ca:
-            fqdn = self.fqdn
+        fqdn = self.fqdn
 
         ca_db = certs.CertDB(self.realm, host_name=fqdn, subject_base=self.subject_base)
 
         db = certs.CertDB(self.realm, subject_base=self.subject_base)
         if self.pkcs12_info:
-            db.create_from_pkcs12(self.pkcs12_info[0], self.pkcs12_info[1], passwd=None)
+            db.create_from_pkcs12(self.pkcs12_info[0], self.pkcs12_info[1],
+                                  passwd=None, ca_file=self.ca_file)
             server_certs = db.find_server_certs()
             if len(server_certs) == 0:
                 raise RuntimeError("Could not find a suitable server cert in import in %s" % self.pkcs12_info[0])
 
             db.create_password_conf()
+
             # We only handle one server cert
             nickname = server_certs[0][0]
             self.dercert = db.get_cert_from_db(nickname, pem=False)
-            db.track_server_cert(nickname, self.principal, db.passwd_fname, 'restart_httpd')
+
+            if api.env.enable_ra:
+                db.track_server_cert(nickname, self.principal, db.passwd_fname, 'restart_httpd')
 
             self.__set_mod_nss_nickname(nickname)
         else:
-            if self.self_signed_ca:
-                db.create_from_cacert(ca_db.cacert_fname)
 
             db.create_password_conf()
-            self.dercert = db.create_server_cert("Server-Cert", self.fqdn, ca_db)
-            db.track_server_cert("Server-Cert", self.principal, db.passwd_fname, 'restart_httpd')
+            self.dercert = db.create_server_cert(self.cert_nickname, self.fqdn,
+                                                 ca_db)
+            db.track_server_cert(self.cert_nickname, self.principal,
+                                 db.passwd_fname, 'restart_httpd')
             db.create_signing_cert("Signing-Cert", "Object Signing Cert", ca_db)
 
         # Fix the database permissions
@@ -277,13 +300,6 @@ class HTTPInstance(service.Service):
         ipaservices.restore_context(certs.NSS_DIR + "/cert8.db")
         ipaservices.restore_context(certs.NSS_DIR + "/key3.db")
 
-        # In case this got generated as part of the install, reset the
-        # context
-        if ipautil.file_exists(certs.CA_SERIALNO):
-            ipaservices.restore_context(certs.CA_SERIALNO)
-            os.chown(certs.CA_SERIALNO, 0, pent.pw_gid)
-            os.chmod(certs.CA_SERIALNO, 0664)
-
     def __setup_autoconfig(self):
         target_fname = '/usr/share/ipa/html/preferences.html'
         ipautil.copy_template_file(
@@ -365,8 +381,7 @@ class HTTPInstance(service.Service):
         if not running is None:
             self.stop()
 
-        db = certs.CertDB(api.env.realm)
-        db.untrack_server_cert("Server-Cert")
+        self.stop_tracking_certificates()
         if not enabled is None and not enabled:
             self.disable()
 
@@ -393,3 +408,7 @@ class HTTPInstance(service.Service):
 
         if not running is None and running:
             self.start()
+
+    def stop_tracking_certificates(self):
+        db = certs.CertDB(api.env.realm)
+        db.untrack_server_cert(self.cert_nickname)
