@@ -30,21 +30,13 @@ import installutils
 from ipapython import sysrestore
 from ipapython import ipautil
 from ipapython import services as ipaservices
-from ipalib import util
 from ipalib import errors
 from ipapython.ipa_log_manager import *
 from ipapython.dn import DN
 
-from ipaserver import ipaldap
 from ipaserver.install import replication
 from ipaserver.install import dsinstance
 
-import ldap
-from ldap import LDAPError
-from ldap import ldapobject
-
-from pyasn1.type import univ, namedtype
-import pyasn1.codec.ber.encoder
 import pyasn1.codec.ber.decoder
 import struct
 
@@ -89,7 +81,6 @@ class KrbInstance(service.Service):
         self.kdc_password = None
         self.sub_dict = None
         self.pkcs12_info = None
-        self.self_signed_ca = None
 
         if fstore:
             self.fstore = fstore
@@ -109,27 +100,35 @@ class KrbInstance(service.Service):
         """
 
         service_dn = DN(('krbprincipalname', principal), self.get_realm_suffix())
-        service_entry = self.admin_conn.getEntry(service_dn, ldap.SCOPE_BASE)
-        self.admin_conn.deleteEntry(service_dn)
+        service_entry = self.admin_conn.get_entry(service_dn)
+        self.admin_conn.delete_entry(service_entry)
 
         # Create a host entry for this master
-        host_dn = DN(('fqdn', self.fqdn), ('cn', 'computers'), ('cn', 'accounts'), self.suffix)
-        host_entry = ipaldap.Entry(host_dn)
-        host_entry.setValues('objectclass', ['top', 'ipaobject', 'nshost', 'ipahost', 'ipaservice', 'pkiuser', 'krbprincipalaux', 'krbprincipal', 'krbticketpolicyaux', 'ipasshhost'])
-        host_entry.setValues('krbextradata', service_entry.getValues('krbextradata'))
-        host_entry.setValue('krblastpwdchange', service_entry.getValue('krblastpwdchange'))
+        host_dn = DN(
+            ('fqdn', self.fqdn), ('cn', 'computers'), ('cn', 'accounts'),
+            self.suffix)
+        host_entry = self.admin_conn.make_entry(
+            host_dn,
+            objectclass=[
+               'top', 'ipaobject', 'nshost', 'ipahost', 'ipaservice',
+               'pkiuser', 'krbprincipalaux', 'krbprincipal',
+               'krbticketpolicyaux', 'ipasshhost'],
+            krbextradata=service_entry['krbextradata'],
+            krblastpwdchange=service_entry['krblastpwdchange'],
+            krbprincipalname=service_entry['krbprincipalname'],
+            krbprincipalkey=service_entry['krbprincipalkey'],
+            serverhostname=[self.fqdn.split('.',1)[0]],
+            cn=[self.fqdn],
+            fqdn=[self.fqdn],
+            ipauniqueid=['autogenerate'],
+            managedby=[host_dn],
+        )
         if 'krbpasswordexpiration' in service_entry.toDict():
-            host_entry.setValue('krbpasswordexpiration', service_entry.getValue('krbpasswordexpiration'))
-        host_entry.setValue('krbprincipalname', service_entry.getValue('krbprincipalname'))
+            host_entry['krbpasswordexpiration'] = service_entry[
+                'krbpasswordexpiration']
         if 'krbticketflags' in service_entry.toDict():
-            host_entry.setValue('krbticketflags', service_entry.getValue('krbticketflags'))
-        host_entry.setValue('krbprincipalkey', service_entry.getValue('krbprincipalkey'))
-        host_entry.setValue('serverhostname', self.fqdn.split('.',1)[0])
-        host_entry.setValue('cn', self.fqdn)
-        host_entry.setValue('fqdn', self.fqdn)
-        host_entry.setValue('ipauniqueid', 'autogenerate')
-        host_entry.setValue('managedby', host_dn)
-        self.admin_conn.addEntry(host_entry)
+            host_entry['krbticketflags'] = service_entry['krbticketflags']
+        self.admin_conn.add_entry(host_entry)
 
     def __common_setup(self, realm_name, host_name, domain_name, admin_password):
         self.fqdn = host_name
@@ -158,10 +157,9 @@ class KrbInstance(service.Service):
         self.step("starting the KDC", self.__start_instance)
         self.step("configuring KDC to start on boot", self.__enable)
 
-    def create_instance(self, realm_name, host_name, domain_name, admin_password, master_password, setup_pkinit=False, pkcs12_info=None, self_signed_ca=False, subject_base=None):
+    def create_instance(self, realm_name, host_name, domain_name, admin_password, master_password, setup_pkinit=False, pkcs12_info=None, subject_base=None):
         self.master_password = master_password
         self.pkcs12_info = pkcs12_info
-        self.self_signed_ca = self_signed_ca
         self.subject_base = subject_base
 
         self.__common_setup(realm_name, host_name, domain_name, admin_password)
@@ -189,9 +187,8 @@ class KrbInstance(service.Service):
                        master_fqdn, host_name,
                        domain_name, admin_password,
                        setup_pkinit=False, pkcs12_info=None,
-                       self_signed_ca=False, subject_base=None):
+                       subject_base=None):
         self.pkcs12_info = pkcs12_info
-        self.self_signed_ca = self_signed_ca
         self.subject_base = subject_base
         self.master_fqdn = master_fqdn
 
@@ -256,44 +253,45 @@ class KrbInstance(service.Service):
         # they may conflict.
 
         try:
-            res = self.admin_conn.search_s(DN(('cn', 'mapping'), ('cn', 'sasl'), ('cn', 'config')),
-                                     ldap.SCOPE_ONELEVEL,
-                                     "(objectclass=nsSaslMapping)")
+            res = self.admin_conn.get_entries(
+                DN(('cn', 'mapping'), ('cn', 'sasl'), ('cn', 'config')),
+                self.admin_conn.SCOPE_ONELEVEL,
+                "(objectclass=nsSaslMapping)")
             for r in res:
                 try:
-                    self.admin_conn.delete_s(r.dn)
-                except LDAPError, e:
-                    root_logger.critical("Error during SASL mapping removal: %s" % str(e))
-                    raise e
-        except LDAPError, e:
-            root_logger.critical("Error while enumerating SASL mappings %s" % str(e))
-            raise e
+                    self.admin_conn.delete_entry(r)
+                except Exception, e:
+                    root_logger.critical(
+                        "Error during SASL mapping removal: %s", e)
+                    raise
+        except Exception, e:
+            root_logger.critical("Error while enumerating SASL mappings %s", e)
+            raise
 
-        entry = ipaldap.Entry(DN(('cn', 'Full Principal'), ('cn', 'mapping'), ('cn', 'sasl'), ('cn', 'config')))
-        entry.setValues("objectclass", "top", "nsSaslMapping")
-        entry.setValues("cn", "Full Principal")
-        entry.setValues("nsSaslMapRegexString", '\(.*\)@\(.*\)')
-        entry.setValues("nsSaslMapBaseDNTemplate", self.suffix)
-        entry.setValues("nsSaslMapFilterTemplate", '(krbPrincipalName=\\1@\\2)')
+        entry = self.admin_conn.make_entry(
+            DN(
+                ('cn', 'Full Principal'), ('cn', 'mapping'), ('cn', 'sasl'),
+                ('cn', 'config')),
+            objectclass=["top", "nsSaslMapping"],
+            cn=["Full Principal"],
+            nsSaslMapRegexString=['\(.*\)@\(.*\)'],
+            nsSaslMapBaseDNTemplate=[self.suffix],
+            nsSaslMapFilterTemplate=['(krbPrincipalName=\\1@\\2)'],
+        )
+        self.admin_conn.add_entry(entry)
 
-        try:
-            self.admin_conn.addEntry(entry)
-        except ldap.ALREADY_EXISTS:
-            root_logger.critical("failed to add Full Principal Sasl mapping")
-            raise e
-
-        entry = ipaldap.Entry(DN(('cn', 'Name Only'), ('cn', 'mapping'), ('cn', 'sasl'), ('cn', 'config')))
-        entry.setValues("objectclass", "top", "nsSaslMapping")
-        entry.setValues("cn", "Name Only")
-        entry.setValues("nsSaslMapRegexString", '^[^:@]+$')
-        entry.setValues("nsSaslMapBaseDNTemplate", self.suffix)
-        entry.setValues("nsSaslMapFilterTemplate", '(krbPrincipalName=&@%s)' % self.realm)
-
-        try:
-            self.admin_conn.addEntry(entry)
-        except ldap.ALREADY_EXISTS:
-            root_logger.critical("failed to add Name Only Sasl mapping")
-            raise e
+        entry = self.admin_conn.make_entry(
+            DN(
+                ('cn', 'Name Only'), ('cn', 'mapping'), ('cn', 'sasl'),
+                ('cn', 'config')),
+            objectclass=["top", "nsSaslMapping"],
+            cn=["Name Only"],
+            nsSaslMapRegexString=['^[^:@]+$'],
+            nsSaslMapBaseDNTemplate=[self.suffix],
+            nsSaslMapFilterTemplate=[
+                '(krbPrincipalName=&@%s)' % self.realm],
+        )
+        self.admin_conn.add_entry(entry)
 
     def __add_krb_container(self):
         self._ldap_mod("kerberos.ldif", self.sub_dict)
@@ -357,13 +355,16 @@ class KrbInstance(service.Service):
 
     def __write_stash_from_ds(self):
         try:
-            entry = self.admin_conn.getEntry(self.get_realm_suffix(),
-                                             ldap.SCOPE_SUBTREE)
+            entries = self.admin_conn.get_entries(
+                self.get_realm_suffix(), self.admin_conn.SCOPE_SUBTREE)
+            # TODO: Ensure we got only one entry
+            entry = entries[0]
         except errors.NotFound, e:
             root_logger.critical("Could not find master key in DS")
             raise e
 
-        krbMKey = pyasn1.codec.ber.decoder.decode(entry.getValue('krbmkey'))
+        krbMKey = pyasn1.codec.ber.decoder.decode(
+            entry.single_value('krbmkey', None))
         keytype = int(krbMKey[0][1][0])
         keydata = str(krbMKey[0][1][1])
 
@@ -390,7 +391,6 @@ class KrbInstance(service.Service):
         installutils.create_keytab("/etc/dirsrv/ds.keytab", ldap_principal)
 
         update_key_val_in_file("/etc/sysconfig/dirsrv", "KRB5_KTNAME", "/etc/dirsrv/ds.keytab")
-        update_key_val_in_file("/etc/sysconfig/dirsrv", "export KRB5_KTNAME", "/etc/dirsrv/ds.keytab")
         pent = pwd.getpwnam(dsinstance.DS_USER)
         os.chown("/etc/dirsrv/ds.keytab", pent.pw_uid, pent.pw_gid)
 
@@ -408,23 +408,15 @@ class KrbInstance(service.Service):
         self.move_service_to_host(host_principal)
 
     def __setup_pkinit(self):
-        if self.self_signed_ca:
-            ca_db = certs.CertDB(self.realm,
-                                 subject_base=self.subject_base)
-        else:
-            ca_db = certs.CertDB(self.realm, host_name=self.fqdn,
-                                 subject_base=self.subject_base)
+        ca_db = certs.CertDB(self.realm, host_name=self.fqdn,
+                                subject_base=self.subject_base)
 
         if self.pkcs12_info:
             ca_db.install_pem_from_p12(self.pkcs12_info[0],
                                        self.pkcs12_info[1],
                                        "/var/kerberos/krb5kdc/kdc.pem")
         else:
-            if self.self_signed_ca:
-                ca_db.create_kdc_cert("KDC-Cert", self.fqdn,
-                                      "/var/kerberos/krb5kdc")
-            else:
-                raise RuntimeError("PKI not supported yet\n")
+            raise RuntimeError("PKI not supported yet\n")
 
         # Finally copy the cacert in the krb directory so we don't
         # have any selinux issues with the file context
@@ -437,7 +429,9 @@ class KrbInstance(service.Service):
         # Create the special anonymous principal
         installutils.kadmin_addprinc(princ_realm)
         dn = DN(('krbprincipalname', princ_realm), self.get_realm_suffix())
-        self.admin_conn.inactivateEntry(dn, False)
+        entry = self.admin_conn.get_entry(dn)
+        entry['nsAccountlock'] = ['TRUE']
+        self.admin_conn.update_entry(entry)
 
     def __convert_to_gssapi_replication(self):
         repl = replication.ReplicationManager(self.realm,
