@@ -25,29 +25,29 @@
 UPDATES_DIR="/usr/share/ipa/updates/"
 
 import sys
-from ipaserver.install import installutils
-from ipaserver.install import service
-from ipaserver import ipaldap
-from ipapython import entity, ipautil
 import uuid
-from ipalib import util
-from ipalib import errors
-from ipalib import api
-from ipapython.dn import DN
-import ldap
-from ldap.schema.models import ObjectClass, AttributeType
-from ipapython.ipa_log_manager import *
-import krbV
 import platform
 import time
-import random
 import os
 import pwd
 import fnmatch
 import csv
 import inspect
+import re
+
+import krbV
+import ldap
+from ldap.schema.models import ObjectClass, AttributeType
+
+from ipaserver.install import installutils
+from ipapython import ipautil, ipaldap
+from ipalib import errors
+from ipalib import api
+from ipapython.dn import DN
+from ipapython.ipa_log_manager import *
 from ipaserver.install.plugins import PRE_UPDATE, POST_UPDATE
-from ipaserver.install.plugins import FIRST, MIDDLE, LAST
+from ipaserver.plugins import ldap2
+
 
 class BadSyntax(installutils.ScriptError):
     def __init__(self, value):
@@ -246,18 +246,9 @@ class LDAPUpdate:
         return text
 
     def _entry_to_entity(self, ent):
-        """Tne Entry class is a bare LDAP entry. The Entity class has a lot more
-           helper functions that we need, so convert to dict and then to Entity.
-        """
-        entry = dict(ent.data)
-        entry['dn'] = ent.dn
-        for key,value in entry.iteritems():
-            if isinstance(value,list) or isinstance(value,tuple):
-                if len(value) == 0:
-                    entry[key] = ''
-                elif len(value) == 1:
-                    entry[key] = value[0]
-        return entity.Entity(entry)
+        entry = ent.copy()
+        entry.commit()
+        return entry
 
     def _combine_updates(self, all_updates, update):
         'Combine a new update with the list of total updates'
@@ -426,18 +417,19 @@ class LDAPUpdate:
         cn = "indextask_%s_%s_%s" % (attribute, cn_uuid.time, cn_uuid.clock_seq)
         dn = DN(('cn', cn), ('cn', 'index'), ('cn', 'tasks'), ('cn', 'config'))
 
-        e = ipaldap.Entry(dn)
-
-        e.setValues('objectClass', ['top', 'extensibleObject'])
-        e.setValue('cn', cn)
-        e.setValue('nsInstance', 'userRoot')
-        e.setValues('nsIndexAttribute', attribute)
+        e = self.conn.make_entry(
+            dn,
+            objectClass=['top', 'extensibleObject'],
+            cn=[cn],
+            nsInstance=['userRoot'],
+            nsIndexAttribute=[attribute],
+        )
 
         self.info("Creating task to index attribute: %s", attribute)
         self.debug("Task id: %s", dn)
 
         if self.live_run:
-            self.conn.addEntry(e)
+            self.conn.add_entry(e)
 
         return dn
 
@@ -459,7 +451,7 @@ class LDAPUpdate:
 
         while True:
             try:
-                entry = self.conn.getEntry(dn, ldap.SCOPE_BASE, "(objectclass=*)", attrlist)
+                entry = self.conn.get_entry(dn, attrlist)
             except errors.NotFound, e:
                 self.error("Task not found: %s", dn)
                 return
@@ -467,7 +459,7 @@ class LDAPUpdate:
                 self.error("Task lookup failure %s", e)
                 return
 
-            status = entry.getValue('nstaskstatus')
+            status = entry.single_value('nstaskstatus', None)
             if status is None:
                 # task doesn't have a status yet
                 time.sleep(1)
@@ -485,10 +477,10 @@ class LDAPUpdate:
     def _create_default_entry(self, dn, default):
         """Create the default entry from the values provided.
 
-           The return type is entity.Entity
+           The return type is ipaldap.LDAPEntry
         """
         assert isinstance(dn, DN)
-        entry = ipaldap.Entry(dn)
+        entry = self.conn.make_entry(dn)
 
         if not default:
             # This means that the entire entry needs to be created with add
@@ -497,28 +489,28 @@ class LDAPUpdate:
         for item in default:
             # We already do syntax-parsing so this is safe
             (attr, value) = item.split(':',1)
-            e = entry.getValues(attr)
+            e = entry.get(attr)
             if e:
                 # multi-valued attribute
                 e = list(e)
                 e.append(value)
             else:
-                e = value
-            entry.setValues(attr, e)
+                e = [value]
+            entry[attr] = e
 
         return self._entry_to_entity(entry)
 
     def _get_entry(self, dn):
         """Retrieve an object from LDAP.
 
-           The return type is ipaldap.Entry
+           The return type is ipaldap.LDAPEntry
         """
         assert isinstance(dn, DN)
         searchfilter="objectclass=*"
         sattrs = ["*", "aci", "attributeTypes", "objectClasses"]
         scope = ldap.SCOPE_BASE
 
-        return self.conn.getList(dn, scope, searchfilter, sattrs)
+        return self.conn.get_entries(dn, scope, searchfilter, sattrs)
 
     def _apply_update_disposition(self, updates, entry):
         """
@@ -541,7 +533,7 @@ class LDAPUpdate:
             if self.conn.has_dn_syntax(attr):
                 update_values = [DN(x) for x in update_values]
 
-            entry_values = entry.getValues(attr)
+            entry_values = entry.get(attr)
             if not isinstance(entry_values, list):
                 if entry_values is None:
                     entry_values = []
@@ -586,7 +578,7 @@ class LDAPUpdate:
                     except ValueError:
                         self.warning("remove: '%s' not in %s", update_value, attr)
                         pass
-                    entry.setValues(attr, entry_values)
+                    entry[attr] = entry_values
                     self.debug('remove: updated value %s', entry_values)
                 elif action == 'add':
                     self.debug("add: '%s' to %s, current value %s", update_value, attr, entry_values)
@@ -597,7 +589,7 @@ class LDAPUpdate:
                         pass
                     entry_values.append(update_value)
                     self.debug('add: updated value %s', entry_values)
-                    entry.setValues(attr, entry_values)
+                    entry[attr] = entry_values
                 elif action == 'addifnew':
                     self.debug("addifnew: '%s' to %s, current value %s", update_value, attr, entry_values)
                     # Only add the attribute if it doesn't exist. Only works
@@ -605,15 +597,15 @@ class LDAPUpdate:
                     if len(entry_values) == 0:
                         entry_values.append(update_value)
                         self.debug('addifnew: set %s to %s', attr, entry_values)
-                        entry.setValues(attr, entry_values)
+                        entry[attr] = entry_values
                 elif action == 'addifexist':
                     self.debug("addifexist: '%s' to %s, current value %s", update_value, attr, entry_values)
                     # Only add the attribute if the entry doesn't exist. We
                     # determine this based on whether it has an objectclass
-                    if entry.getValues('objectclass'):
+                    if entry.get('objectclass'):
                         entry_values.append(update_value)
                         self.debug('addifexist: set %s to %s', attr, entry_values)
-                        entry.setValues(attr, entry_values)
+                        entry[attr] = entry_values
                 elif action == 'only':
                     self.debug("only: set %s to '%s', current value %s", attr, update_value, entry_values)
                     if only.get(attr):
@@ -621,20 +613,20 @@ class LDAPUpdate:
                     else:
                         entry_values = [update_value]
                         only[attr] = True
-                    entry.setValues(attr, entry_values)
+                    entry[attr] = entry_values
                     self.debug('only: updated value %s', entry_values)
                 elif action == 'onlyifexist':
                     self.debug("onlyifexist: '%s' to %s, current value %s", update_value, attr, entry_values)
                     # Only set the attribute if the entry exist's. We
                     # determine this based on whether it has an objectclass
-                    if entry.getValues('objectclass'):
+                    if entry.get('objectclass'):
                         if only.get(attr):
                             entry_values.append(update_value)
                         else:
                             entry_values = [update_value]
                             only[attr] = True
                         self.debug('onlyifexist: set %s to %s', attr, entry_values)
-                        entry.setValues(attr, entry_values)
+                        entry[attr] = entry_values
                 elif action == 'deleteentry':
                     # skip this update type, it occurs in  __delete_entries()
                     return None
@@ -670,7 +662,7 @@ class LDAPUpdate:
                             entry_values.remove(old)
                         entry_values.append(new)
                         self.debug('replace: updated value %s', entry_values)
-                        entry.setValues(attr, entry_values)
+                        entry[attr] = entry_values
                     except ValueError:
                         self.debug('replace: %s not found, skipping', old)
 
@@ -682,9 +674,7 @@ class LDAPUpdate:
         if message:
             self.debug("%s", message)
         self.debug("dn: %s", e.dn)
-        attr = e.attrList()
-        for a in attr:
-            value = e.getValues(a)
+        for a, value in e.items():
             if isinstance(value, (list, tuple)):
                 self.debug('%s:', a)
                 for l in value:
@@ -775,12 +765,12 @@ class LDAPUpdate:
             # entry.orig_data = {}
             try:
                 if self.live_run:
-                    if len(entry.toTupleList()) > 0:
+                    if len(entry):
                         # addifexist may result in an entry with only a
                         # dn defined. In that case there is nothing to do.
                         # It means the entry doesn't exist, so skip it.
                         try:
-                            self.conn.addEntry(entry)
+                            self.conn.add_entry(entry)
                         except errors.NotFound:
                             # parent entry of the added entry does not exist
                             # this may not be an error (e.g. entries in NIS container)
@@ -825,7 +815,7 @@ class LDAPUpdate:
         if entry.dn.endswith(DN(('cn', 'index'), ('cn', 'userRoot'),
                                 ('cn', 'ldbm database'), ('cn', 'plugins'),
                                 ('cn', 'config'))) and (added or updated):
-            taskid = self.create_index_task(entry.getValue('cn'))
+            taskid = self.create_index_task(entry.single_value('cn'))
             self.monitor_index_task(taskid)
         return
 
@@ -846,7 +836,7 @@ class LDAPUpdate:
         try:
             self.info("Deleting entry %s", dn)
             if self.live_run:
-                self.conn.deleteEntry(dn)
+                self.conn.delete_entry(dn)
             self.modified = True
         except errors.NotFound, e:
             self.info("%s did not exist:%s", dn, e)
@@ -911,21 +901,46 @@ class LDAPUpdate:
         for dn, update in sorted_updates:
             self._delete_record(update)
 
-    def update(self, files):
+    def update(self, files, ordered=False):
         """Execute the update. files is a list of the update files to use.
 
+           If ordered is True then the updates the file must be of the form
+           ##-name.update where ## is an integer between 10 and 89. The
+           changes are applied to LDAP at the end of each value divisible
+           by 10, so after 20, 30, etc.
+
            returns True if anything was changed, otherwise False
         """
 
+        pat = re.compile(r'(\d+)-.*\.update')
         all_updates = {}
+        r = 20
         if self.plugins:
             self.info('PRE_UPDATE')
             updates = api.Backend.updateclient.update(PRE_UPDATE, self.dm_password, self.ldapi, self.live_run)
             self.merge_updates(all_updates, updates)
         try:
             self.create_connection()
+            if ordered and all_updates:
+                # flush out PRE_UPDATE plugin updates before we begin
+                self._run_updates(all_updates)
+                all_updates = {}
 
             for f in files:
+                name = os.path.basename(f)
+                if ordered:
+                    m = pat.match(name)
+                    if not m:
+                        raise RuntimeError("Filename does not match format #-name.update: %s" % f)
+                    index = int(m.group(1))
+                    if index < 10 or index > 90:
+                        raise RuntimeError("Index not legal range: %d" % index)
+
+                    if index >= r:
+                        self._run_updates(all_updates)
+                        all_updates = {}
+                        r += 10
+
                 try:
                     self.info("Parsing update file '%s'" % f)
                     data = self.read_file(f)
