@@ -21,19 +21,14 @@ import sys
 import os, socket
 import tempfile
 import pwd
-from ipapython import sysrestore
-from ipapython import ipautil
-from ipapython import dogtag
-from ipapython import services as ipaservices
-from ipalib import errors
-from ipapython.dn import DN
-import ldap
-from ipaserver import ipaldap
-import base64
 import time
 import datetime
-from ipaserver.install import installutils
+
+from ipapython import sysrestore, ipautil, dogtag, ipaldap
+from ipapython import services as ipaservices
+from ipapython.dn import DN
 from ipapython.ipa_log_manager import *
+from ipalib import errors
 
 CACERT = "/etc/ipa/ca.crt"
 
@@ -45,14 +40,15 @@ DISABLED = 3
 # The service name as stored in cn=masters,cn=ipa,cn=etc. In the tuple
 # the first value is the *nix service name, the second the start order.
 SERVICE_LIST = {
-    'KDC':('krb5kdc', 10),
-    'KPASSWD':('kadmin', 20),
-    'DNS':('named', 30),
-    'MEMCACHE':('ipa_memcached', 39),
-    'HTTP':('httpd', 40),
-    'CA':('%sd' % dogtag.configured_constants().PKI_INSTANCE_NAME, 50),
-    'ADTRUST':('smb', 60),
-    'EXTID':('winbind', 70)
+    'KDC': ('krb5kdc', 10),
+    'KPASSWD': ('kadmin', 20),
+    'DNS': ('named', 30),
+    'MEMCACHE': ('ipa_memcached', 39),
+    'HTTP': ('httpd', 40),
+    'CA': ('%sd' % dogtag.configured_constants().PKI_INSTANCE_NAME, 50),
+    'ADTRUST': ('smb', 60),
+    'EXTID': ('winbind', 70),
+    'OTPD': ('ipa-otpd', 80),
 }
 
 def print_msg(message, output_fd=sys.stdout):
@@ -120,7 +116,7 @@ class Service(object):
                 conn.do_sasl_gssapi_bind()
         except Exception, e:
             root_logger.debug("Could not connect to the Directory Server on %s: %s" % (self.fqdn, str(e)))
-            raise e
+            raise
 
         self.admin_conn = conn
 
@@ -153,7 +149,7 @@ class Service(object):
         # use URI of admin connection
         if not self.admin_conn:
             self.ldap_connect()
-        args += ["-H", self.admin_conn.uri]
+        args += ["-H", self.admin_conn.ldap_uri]
 
         auth_parms = []
         if self.dm_password:
@@ -188,21 +184,21 @@ class Service(object):
 
         dn = DN(('krbprincipalname', principal), ('cn', self.realm), ('cn', 'kerberos'), self.suffix)
         try:
-            entry = self.admin_conn.getEntry(dn, ldap.SCOPE_BASE)
+            entry = self.admin_conn.get_entry(dn)
         except errors.NotFound:
             # There is no service in the wrong location, nothing to do.
             # This can happen when installing a replica
             return None
         newdn = DN(('krbprincipalname', principal), ('cn', 'services'), ('cn', 'accounts'), self.suffix)
         hostdn = DN(('fqdn', self.fqdn), ('cn', 'computers'), ('cn', 'accounts'), self.suffix)
-        self.admin_conn.deleteEntry(dn)
+        self.admin_conn.delete_entry(entry)
         entry.dn = newdn
-        classes = entry.getValues("objectclass")
+        classes = entry.get("objectclass")
         classes = classes + ["ipaobject", "ipaservice", "pkiuser"]
-        entry.setValues("objectclass", list(set(classes)))
-        entry.setValue("ipauniqueid", 'autogenerate')
-        entry.setValue("managedby", hostdn)
-        self.admin_conn.addEntry(entry)
+        entry["objectclass"] = list(set(classes))
+        entry["ipauniqueid"] = ['autogenerate']
+        entry["managedby"] = [hostdn]
+        self.admin_conn.add_entry(entry)
         return newdn
 
     def add_simple_service(self, principal):
@@ -216,12 +212,16 @@ class Service(object):
 
         dn = DN(('krbprincipalname', principal), ('cn', 'services'), ('cn', 'accounts'), self.suffix)
         hostdn = DN(('fqdn', self.fqdn), ('cn', 'computers'), ('cn', 'accounts'), self.suffix)
-        entry = ipaldap.Entry(dn)
-        entry.setValues("objectclass", ["krbprincipal", "krbprincipalaux", "krbticketpolicyaux", "ipaobject", "ipaservice", "pkiuser"])
-        entry.setValue("krbprincipalname", principal)
-        entry.setValue("ipauniqueid", 'autogenerate')
-        entry.setValue("managedby", hostdn)
-        self.admin_conn.addEntry(entry)
+        entry = self.admin_conn.make_entry(
+            dn,
+            objectclass=[
+                "krbprincipal", "krbprincipalaux", "krbticketpolicyaux",
+                "ipaobject", "ipaservice", "pkiuser"],
+            krbprincipalname=[principal],
+            ipauniqueid=['autogenerate'],
+            managedby=[hostdn],
+        )
+        self.admin_conn.add_entry(entry)
         return dn
 
     def add_cert_to_service(self):
@@ -245,10 +245,12 @@ class Service(object):
             self.ldap_disconnect()
         self.ldap_connect()
 
-        dn = DN(('krbprincipalname', self.principal), ('cn', 'services'), ('cn', 'accounts'), self.suffix)
-        mod = [(ldap.MOD_ADD, 'userCertificate', self.dercert)]
+        dn = DN(('krbprincipalname', self.principal), ('cn', 'services'),
+                ('cn', 'accounts'), self.suffix)
+        entry = self.admin_conn.get_entry(dn)
+        entry.setdefault('userCertificate', []).append(self.dercert)
         try:
-            self.admin_conn.modify_s(dn, mod)
+            self.admin_conn.update_entry(entry)
         except Exception, e:
             root_logger.critical("Could not add certificate to service %s entry: %s" % (self.principal, str(e)))
 
@@ -291,6 +293,9 @@ class Service(object):
     def restore_state(self, key):
         return self.sstore.restore_state(self.service_name, key)
 
+    def get_state(self, key):
+        return self.sstore.get_state(self.service_name, key)
+
     def print_msg(self, message):
         print_msg(message, self.output_fd)
 
@@ -373,16 +378,17 @@ class Service(object):
 
         entry_name = DN(('cn', name), ('cn', fqdn), ('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'), ldap_suffix)
         order = SERVICE_LIST[name][1]
-        entry = ipaldap.Entry(entry_name)
-        entry.setValues("objectclass",
-                        "nsContainer", "ipaConfigObject")
-        entry.setValues("cn", name)
-        entry.setValues("ipaconfigstring",
-                        "enabledService", "startOrder " + str(order))
+        entry = self.admin_conn.make_entry(
+            entry_name,
+            objectclass=["nsContainer", "ipaConfigObject"],
+            cn=[name],
+            ipaconfigstring=[
+                "enabledService", "startOrder " + str(order)],
+        )
 
         try:
-            self.admin_conn.addEntry(entry)
-        except (ldap.ALREADY_EXISTS, errors.DuplicateEntry), e:
+            self.admin_conn.add_entry(entry)
+        except (errors.DuplicateEntry), e:
             root_logger.debug("failed to add %s Service startup entry" % name)
             raise e
 
