@@ -22,15 +22,14 @@ Test the `ipaserver/install/ldapupdate.py` module.
 
 import unittest
 import os
-import sys
-import ldap
+
 import nose
+
 from ipalib import api
 from ipalib import errors
-from ipaserver.install.ldapupdate import LDAPUpdate, BadSyntax, UPDATES_DIR
+from ipaserver.install.ldapupdate import LDAPUpdate, BadSyntax
 from ipaserver.install import installutils
-from ipaserver import ipaldap
-from ipapython import ipautil
+from ipapython import ipautil, ipaldap
 from ipapython.dn import DN
 
 """
@@ -89,10 +88,12 @@ class test_update(unittest.TestCase):
         self.assertTrue(modified)
 
         with self.assertRaises(errors.NotFound):
-            entries = self.ld.getList(self.container_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+            entries = self.ld.get_entries(
+                self.container_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
 
         with self.assertRaises(errors.NotFound):
-            entries = self.ld.getList(self.user_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+            entries = self.ld.get_entries(
+                self.user_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
 
     def test_1_add(self):
         """
@@ -102,28 +103,30 @@ class test_update(unittest.TestCase):
 
         self.assertTrue(modified)
 
-        entries = self.ld.getList(self.container_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+        entries = self.ld.get_entries(
+            self.container_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
         self.assertEqual(len(entries), 1)
         entry = entries[0]
 
-        objectclasses = entry.getValues('objectclass')
+        objectclasses = entry.get('objectclass')
         for item in ('top', 'nsContainer'):
             self.assertTrue(item in objectclasses)
 
-        self.assertEqual(entry.getValue('cn'), 'test')
+        self.assertEqual(entry.single_value('cn'), 'test')
 
-        entries = self.ld.getList(self.user_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+        entries = self.ld.get_entries(
+            self.user_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
         self.assertEqual(len(entries), 1)
         entry = entries[0]
 
-        objectclasses = entry.getValues('objectclass')
+        objectclasses = entry.get('objectclass')
         for item in ('top', 'person', 'posixaccount', 'krbprincipalaux', 'inetuser'):
             self.assertTrue(item in objectclasses)
 
-        self.assertEqual(entry.getValue('loginshell'), '/bin/bash')
-        self.assertEqual(entry.getValue('sn'), 'User')
-        self.assertEqual(entry.getValue('uid'), 'tuser')
-        self.assertEqual(entry.getValue('cn'), 'Test User')
+        self.assertEqual(entry.single_value('loginshell'), '/bin/bash')
+        self.assertEqual(entry.single_value('sn'), 'User')
+        self.assertEqual(entry.single_value('uid'), 'tuser')
+        self.assertEqual(entry.single_value('cn'), 'Test User')
 
 
     def test_2_update(self):
@@ -133,10 +136,11 @@ class test_update(unittest.TestCase):
         modified = self.updater.update([self.testdir + "2_update.update"])
         self.assertTrue(modified)
 
-        entries = self.ld.getList(self.user_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+        entries = self.ld.get_entries(
+            self.user_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
         self.assertEqual(len(entries), 1)
         entry = entries[0]
-        self.assertEqual(entry.getValue('gecos'), 'Test User')
+        self.assertEqual(entry.single_value('gecos'), 'Test User')
 
     def test_3_update(self):
         """
@@ -145,10 +149,11 @@ class test_update(unittest.TestCase):
         modified = self.updater.update([self.testdir + "3_update.update"])
         self.assertTrue(modified)
 
-        entries = self.ld.getList(self.user_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+        entries = self.ld.get_entries(
+            self.user_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
         self.assertEqual(len(entries), 1)
         entry = entries[0]
-        self.assertEqual(entry.getValue('gecos'), 'Test User New')
+        self.assertEqual(entry.single_value('gecos'), 'Test User New')
 
     def test_4_update(self):
         """
@@ -157,10 +162,11 @@ class test_update(unittest.TestCase):
         modified = self.updater.update([self.testdir + "4_update.update"])
         self.assertTrue(modified)
 
-        entries = self.ld.getList(self.user_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+        entries = self.ld.get_entries(
+            self.user_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
         self.assertEqual(len(entries), 1)
         entry = entries[0]
-        self.assertEqual(entry.getValue('gecos'), 'Test User New2')
+        self.assertEqual(entry.single_value('gecos'), 'Test User New2')
 
     def test_5_update(self):
         """
@@ -169,10 +175,11 @@ class test_update(unittest.TestCase):
         modified = self.updater.update([self.testdir + "5_update.update"])
         self.assertTrue(modified)
 
-        entries = self.ld.getList(self.user_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+        entries = self.ld.get_entries(
+            self.user_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
         self.assertEqual(len(entries), 1)
         entry = entries[0]
-        self.assertEqual(sorted(entry.getValues('cn')), sorted(['Test User', 'Test User New']))
+        self.assertEqual(sorted(entry.get('cn')), sorted(['Test User', 'Test User New']))
 
     def test_6_update(self):
         """
@@ -181,10 +188,11 @@ class test_update(unittest.TestCase):
         modified = self.updater.update([self.testdir + "6_update.update"])
         self.assertTrue(modified)
 
-        entries = self.ld.getList(self.user_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+        entries = self.ld.get_entries(
+            self.user_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
         self.assertEqual(len(entries), 1)
         entry = entries[0]
-        self.assertEqual(sorted(entry.getValues('cn')), sorted(['Test User']))
+        self.assertEqual(sorted(entry.get('cn')), sorted(['Test User']))
 
     def test_6_update_1(self):
         """
@@ -193,10 +201,11 @@ class test_update(unittest.TestCase):
         modified = self.updater.update([self.testdir + "6_update.update"])
         self.assertFalse(modified)
 
-        entries = self.ld.getList(self.user_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+        entries = self.ld.get_entries(
+            self.user_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
         self.assertEqual(len(entries), 1)
         entry = entries[0]
-        self.assertEqual(sorted(entry.getValues('cn')), sorted(['Test User']))
+        self.assertEqual(sorted(entry.get('cn')), sorted(['Test User']))
 
     def test_7_cleanup(self):
         """
@@ -211,10 +220,12 @@ class test_update(unittest.TestCase):
         self.assertTrue(modified)
 
         with self.assertRaises(errors.NotFound):
-            entries = self.ld.getList(self.container_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+            entries = self.ld.get_entries(
+                self.container_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
 
         with self.assertRaises(errors.NotFound):
-            entries = self.ld.getList(self.user_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+            entries = self.ld.get_entries(
+                self.user_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
 
     def test_8_badsyntax(self):
         """
@@ -239,10 +250,12 @@ class test_update(unittest.TestCase):
 
         # First make sure we're clean
         with self.assertRaises(errors.NotFound):
-            entries = self.ld.getList(self.container_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+            entries = self.ld.get_entries(
+                self.container_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
 
         with self.assertRaises(errors.NotFound):
-            entries = self.ld.getList(self.user_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+            entries = self.ld.get_entries(
+                self.user_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
 
 
         update = {
@@ -274,28 +287,30 @@ class test_update(unittest.TestCase):
         modified = self.updater.update_from_dict(update)
         self.assertTrue(modified)
 
-        entries = self.ld.getList(self.container_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+        entries = self.ld.get_entries(
+            self.container_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
         self.assertEqual(len(entries), 1)
         entry = entries[0]
 
-        objectclasses = entry.getValues('objectclass')
+        objectclasses = entry.get('objectclass')
         for item in ('top', 'nsContainer'):
             self.assertTrue(item in objectclasses)
 
-        self.assertEqual(entry.getValue('cn'), 'test')
+        self.assertEqual(entry.single_value('cn'), 'test')
 
-        entries = self.ld.getList(self.user_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+        entries = self.ld.get_entries(
+            self.user_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
         self.assertEqual(len(entries), 1)
         entry = entries[0]
 
-        objectclasses = entry.getValues('objectclass')
+        objectclasses = entry.get('objectclass')
         for item in ('top', 'person', 'posixaccount', 'krbprincipalaux', 'inetuser'):
             self.assertTrue(item in objectclasses)
 
-        self.assertEqual(entry.getValue('loginshell'), '/bin/bash')
-        self.assertEqual(entry.getValue('sn'), 'User')
-        self.assertEqual(entry.getValue('uid'), 'tuser')
-        self.assertEqual(entry.getValue('cn'), 'Test User')
+        self.assertEqual(entry.single_value('loginshell'), '/bin/bash')
+        self.assertEqual(entry.single_value('sn'), 'User')
+        self.assertEqual(entry.single_value('uid'), 'tuser')
+        self.assertEqual(entry.single_value('cn'), 'Test User')
 
         # Now delete
 
@@ -314,7 +329,9 @@ class test_update(unittest.TestCase):
         self.assertTrue(modified)
 
         with self.assertRaises(errors.NotFound):
-            entries = self.ld.getList(self.container_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+            entries = self.ld.get_entries(
+                self.container_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
 
         with self.assertRaises(errors.NotFound):
-            entries = self.ld.getList(self.user_dn, ldap.SCOPE_BASE, 'objectclass=*', ['*'])
+            entries = self.ld.get_entries(
+                self.user_dn, self.ld.SCOPE_BASE, 'objectclass=*', ['*'])
