@@ -27,7 +27,7 @@ from tests.util import assert_equal
 from ipalib.constants import TYPE_ERROR
 from ipalib.base import NameSpace
 from ipalib import frontend, backend, plugable, errors, parameters, config
-from ipalib import output
+from ipalib import output, messages
 from ipalib.parameters import Str
 from ipapython.version import API_VERSION
 
@@ -243,10 +243,14 @@ class test_Command(ClassChecker):
         """
         Test the `ipalib.frontend.Command.get_options` method.
         """
-        assert list(self.cls().get_options()) == []
+        options = list(self.cls().get_options())
+        assert len(options) == 1
+        assert options[0].name == 'version'
         options = ('verbose', 'debug')
         o = self.get_instance(options=options)
-        assert tuple(o.get_options()) == options
+        assert len(tuple(o.get_options())) == 3
+        assert 'verbose' in tuple(o.get_options())
+        assert 'debug' in tuple(o.get_options())
 
     def test_args(self):
         """
@@ -305,12 +309,12 @@ class test_Command(ClassChecker):
         o = self.cls()
         o.finalize()
         assert type(o.options) is plugable.NameSpace
-        assert len(o.options) == 0
+        assert len(o.options) == 1
         options = ('target', 'files*')
         ns = self.get_instance(options=options).options
         assert type(ns) is plugable.NameSpace
-        assert len(ns) == len(options)
-        assert list(ns) == ['target', 'files']
+        assert len(ns) == len(options) + 1
+        assert list(ns) == ['target', 'files', 'version']
         assert type(ns.target) is parameters.Str
         assert type(ns.files) is parameters.Str
         assert ns.target.required is True
@@ -377,12 +381,13 @@ class test_Command(ClassChecker):
         cmd = user_add()
         cmd.env = config.Env(context='cli')
         cmd.finalize()
-        assert list(cmd.params) == ['givenname', 'sn', 'uid']
+        assert list(cmd.params) == ['givenname', 'sn', 'uid', 'version']
         ret = cmd.soft_validate({})
-        assert len(ret['values']) == 0
-        assert len(ret['errors']) == 3
+        assert sorted(ret['values']) == ['version']
+        assert sorted(ret['errors']) == ['givenname', 'sn', 'uid']
         assert cmd.soft_validate(dict(givenname=u'First', sn=u'Last')) == dict(
-            values=dict(givenname=u'First', sn=u'Last', uid=u'flast'),
+            values=dict(givenname=u'First', sn=u'Last', uid=u'flast',
+                        version=None),
             errors=dict(),
         )
 
@@ -604,7 +609,6 @@ class test_Command(ClassChecker):
         o.set_api(api)
         assert o.run.im_func is self.cls.run.im_func
         out = o.run(*args, **kw)
-        del kw['version']
         assert ('execute', args, kw) == out
 
         # Test in non-server context
@@ -615,6 +619,47 @@ class test_Command(ClassChecker):
         assert o.run.im_func is self.cls.run.im_func
         assert ('forward', args, kw) == o.run(*args, **kw)
 
+    def test_messages(self):
+        """
+        Test correct handling of messages
+        """
+        class TestMessage(messages.PublicMessage):
+            type = 'info'
+            format = 'This is a message.'
+            errno = 1234
+
+        class my_cmd(self.cls):
+            def execute(self, *args, **kw):
+                result = {'name': 'execute'}
+                messages.add_message(kw['version'], result, TestMessage())
+                return result
+
+            def forward(self, *args, **kw):
+                result = {'name': 'forward'}
+                messages.add_message(kw['version'], result, TestMessage())
+                return result
+
+        args = ('Hello,', 'world,')
+        kw = dict(how_are='you', on_this='fine day?', version=API_VERSION)
+
+        expected = [TestMessage().to_dict()]
+
+        # Test in server context:
+        (api, home) = create_test_api(in_server=True)
+        api.finalize()
+        o = my_cmd()
+        o.set_api(api)
+        assert o.run.im_func is self.cls.run.im_func
+        assert {'name': 'execute', 'messages': expected} == o.run(*args, **kw)
+
+        # Test in non-server context
+        (api, home) = create_test_api(in_server=False)
+        api.finalize()
+        o = my_cmd()
+        o.set_api(api)
+        assert o.run.im_func is self.cls.run.im_func
+        assert {'name': 'forward', 'messages': expected} == o.run(*args, **kw)
+
     def test_validate_output_basic(self):
         """
         Test the `ipalib.frontend.Command.validate_output` method.
@@ -749,7 +794,7 @@ class test_LocalOrRemote(ClassChecker):
         o = self.cls()
         o.finalize()
         assert list(o.args) == []
-        assert list(o.options) == ['server']
+        assert list(o.options) == ['server', 'version']
         op = o.options.server
         assert op.required is False
         assert op.default is False
@@ -772,17 +817,17 @@ class test_LocalOrRemote(ClassChecker):
         api.register(example)
         api.finalize()
         cmd = api.Command.example
-        assert cmd() == dict(
-            result=('execute', (None,), dict(server=False))
+        assert cmd(version=u'2.47') == dict(
+            result=('execute', (None,), dict(version=u'2.47', server=False))
         )
-        assert cmd(u'var') == dict(
-            result=('execute', (u'var',), dict(server=False))
+        assert cmd(u'var', version=u'2.47') == dict(
+            result=('execute', (u'var',), dict(version=u'2.47', server=False))
         )
-        assert cmd(server=True) == dict(
-            result=('forward', (None,), dict(server=True))
+        assert cmd(server=True, version=u'2.47') == dict(
+            result=('forward', (None,), dict(version=u'2.47', server=True))
         )
-        assert cmd(u'var', server=True) == dict(
-            result=('forward', (u'var',), dict(server=True))
+        assert cmd(u'var', server=True, version=u'2.47') == dict(
+            result=('forward', (u'var',), dict(version=u'2.47', server=True))
         )
 
         # Test when in_server=True (should always call execute):
@@ -790,17 +835,17 @@ class test_LocalOrRemote(ClassChecker):
         api.register(example)
         api.finalize()
         cmd = api.Command.example
-        assert cmd() == dict(
-            result=('execute', (None,), dict(server=False))
+        assert cmd(version=u'2.47') == dict(
+            result=('execute', (None,), dict(version=u'2.47', server=False))
         )
-        assert cmd(u'var') == dict(
-            result=('execute', (u'var',), dict(server=False))
+        assert cmd(u'var', version=u'2.47') == dict(
+            result=('execute', (u'var',), dict(version=u'2.47', server=False))
         )
-        assert cmd(server=True) == dict(
-            result=('execute', (None,), dict(server=True))
+        assert cmd(server=True, version=u'2.47') == dict(
+            result=('execute', (None,), dict(version=u'2.47', server=True))
         )
-        assert cmd(u'var', server=True) == dict(
-            result=('execute', (u'var',), dict(server=True))
+        assert cmd(u'var', server=True, version=u'2.47') == dict(
+            result=('execute', (u'var',), dict(version=u'2.47', server=True))
         )
 
 
