@@ -203,8 +203,6 @@ class test_Param(ClassChecker):
         assert o.flags == frozenset()
         assert o.sortorder == 2
         assert o.csv is False
-        assert o.csv_separator == ','
-        assert o.csv_skipspace is True
 
         # Test that doc defaults from label:
         o = self.cls('my_param', doc=_('Hello world'))
@@ -469,11 +467,11 @@ class test_Param(ClassChecker):
         assert str(e) == 'value: empty tuple must be converted to None'
 
         # Test with wrong (scalar) type:
-        e = raises(ValidationError, o.validate, (None, None, 42, None), 'cli')
-        assert str(e) == 'invalid %s' % (TYPE_ERROR % ('\'my_param\'', NoneType, 42, int))
+        e = raises(TypeError, o.validate, (None, None, 42, None), 'cli')
+        assert str(e) == TYPE_ERROR % ('my_param', NoneType, 42, int)
         o = self.cls('my_param')
-        e = raises(ValidationError, o.validate, 'Hello', 'cli')
-        assert str(e) == 'invalid %s' % (TYPE_ERROR % ('\'my_param\'', NoneType, 'Hello', str))
+        e = raises(TypeError, o.validate, 'Hello', 'cli')
+        assert str(e) == TYPE_ERROR % ('my_param', NoneType, 'Hello', str)
 
         class Example(self.cls):
             type = int
@@ -535,10 +533,10 @@ class test_Param(ClassChecker):
         o = MyParam('my_param', okay)
 
         # Test that TypeError is appropriately raised:
-        e = raises(ValidationError, o._validate_scalar, 0)
-        assert str(e) == 'invalid %s' % (TYPE_ERROR % ('\'my_param\'', bool, 0, int))
-        e = raises(ValidationError, o._validate_scalar, 'Hi', index=4)
-        assert str(e) == 'invalid %s' % (TYPE_ERROR % ('\'my_param\'', bool, 'Hi', str))
+        e = raises(TypeError, o._validate_scalar, 0)
+        assert str(e) == TYPE_ERROR % ('my_param', bool, 0, int)
+        e = raises(TypeError, o._validate_scalar, 'Hi', index=4)
+        assert str(e) == TYPE_ERROR % ('my_param', bool, 'Hi', str)
         e = raises(TypeError, o._validate_scalar, True, index=3.0)
         assert str(e) == TYPE_ERROR % ('index', int, 3.0, float)
 
@@ -618,48 +616,6 @@ class test_Param(ClassChecker):
         assert o._convert_scalar.value is default
         assert o.normalizer.value is default
 
-    def test_split_csv(self):
-        """
-        Test the `ipalib.parameters.Param.split_csv` method with csv.
-        """
-        o = self.cls('my_list+', csv=True)
-        n = o.split_csv('a,b')
-        assert type(n) is tuple
-        assert len(n) is 2
-
-        n = o.split_csv('bar,   "hi, there",foo')
-        assert type(n) is tuple
-        assert len(n) is 3
-
-    def test_split_csv_separator(self):
-        """
-        Test the `ipalib.parameters.Param.split_csv` method with csv and a separator.
-        """
-        o = self.cls('my_list+', csv=True, csv_separator='|')
-
-        n = o.split_csv('a')
-        assert type(n) is tuple
-        assert len(n) is 1
-
-        n = o.split_csv('a|b')
-        assert type(n) is tuple
-        assert len(n) is 2
-
-    def test_split_csv_skipspace(self):
-        """
-        Test the `ipalib.parameters.Param.split_csv` method with csv without skipping spaces.
-        """
-        o = self.cls('my_list+', csv=True, csv_skipspace=False)
-
-        n = o.split_csv('a')
-        assert type(n) is tuple
-        assert len(n) is 1
-
-        n = o.split_csv('a, "b,c", d')
-        assert type(n) is tuple
-        # the output w/o skipspace is ['a',' "b','c"',' d']
-        assert len(n) is 4
-
 
 class test_Flag(ClassChecker):
     """
@@ -1574,4 +1530,4 @@ class test_IA5Str(ClassChecker):
             e = raises(errors.ConversionError, mthd, value)
             assert e.name == 'my_str'
             assert e.index is None
-            assert_equal(e.error, "The character \''\\xc3'\' is not allowed.")
+            assert_equal(e.error, "The character '\\xc3' is not allowed.")
