@@ -34,6 +34,7 @@ dnszone1_rname = u'root.%s.' % dnszone1
 dnszone1_permission = u'Manage DNS zone %s' % dnszone1
 dnszone1_permission_dn = DN(('cn',dnszone1_permission),
                             api.env.container_permission,api.env.basedn)
+dnszone1_txtrec_dn = DN(('idnsname', '_kerberos'), dnszone1_dn)
 dnszone2 = u'dnszone2.test'
 dnszone2_dn = DN(('idnsname', dnszone2), api.env.container_dns, api.env.basedn)
 dnszone2_mname = u'ns1.%s.' % dnszone2
@@ -53,6 +54,8 @@ dnsrev2 = u'81'
 dnsrev2_dn = DN(('idnsname',dnsrev2), revdnszone1_dn)
 dnsrescname = u'testcnamerec'
 dnsrescname_dn = DN(('idnsname',dnsrescname), dnszone1_dn)
+dnsresdname = u'testdns-dname'
+dnsresdname_dn = DN(('idnsname',dnsresdname), dnszone1_dn)
 
 class test_dns(Declarative):
 
@@ -223,15 +226,6 @@ class test_dns(Declarative):
             },
         ),
 
-        dict(
-            desc='Delete zone %r' % dnszone2,
-            command=('dnszone_del', [dnszone2], {}),
-            expected={
-                'value': dnszone2,
-                'summary': None,
-                'result': {'failed': u''},
-            },
-        ),
 
         dict(
             desc='Retrieve zone %r' % dnszone1,
@@ -401,7 +395,7 @@ class test_dns(Declarative):
             command=('dnszone_del', [revdnszone1], {}),
             expected={
                 'value': revdnszone1,
-                'summary': None,
+                'summary': u'Deleted DNS zone "%s"' % revdnszone1,
                 'result': {'failed': u''},
             },
         ),
@@ -533,7 +527,7 @@ class test_dns(Declarative):
             command=('dnsrecord_find', [dnszone1], {}),
             expected={
                 'summary': None,
-                'count': 3,
+                'count': 4,
                 'truncated': False,
                 'result': [
                     {
@@ -541,6 +535,11 @@ class test_dns(Declarative):
                         'nsrecord': (dnszone1_mname,),
                         'idnsname': [u'@'],
                     },
+                    {
+                        'dn': dnszone1_txtrec_dn,
+                        'txtrecord': [api.env.realm],
+                        'idnsname': [u'_kerberos'],
+                    },
                     {
                         'dn': dnszone1_mname_dn,
                         'idnsname': [u'ns1'],
@@ -678,7 +677,7 @@ class test_dns(Declarative):
                                                                  'srv_part_port' : 123,
                                                                  'srv_part_target' : u'foo bar'}),
             expected=errors.ValidationError(name='srv_target',
-                error=u'invalid domain-name: only letters, numbers, and - ' +
+                error=u'invalid domain-name: only letters, numbers, _, and - ' +
                     u'are allowed. DNS label may not start or end with -'),
         ),
 
@@ -773,17 +772,26 @@ class test_dns(Declarative):
             desc='Try to add CNAME record to %r using dnsrecord_add' % (dnsres1),
             command=('dnsrecord_add', [dnszone1, dnsres1], {'cnamerecord': u'foo-1.example.com.'}),
             expected=errors.ValidationError(name='cnamerecord',
-                error=u'CNAME record is not allowed to coexist with any other records except PTR'),
+                error=u'CNAME record is not allowed to coexist with any other '
+                      u'record (RFC 1034, section 3.6.2)'),
         ),
 
         dict(
             desc='Try to add invalid CNAME record %r using dnsrecord_add' % (dnsrescname),
             command=('dnsrecord_add', [dnszone1, dnsrescname], {'cnamerecord': u'-.example.com'}),
             expected=errors.ValidationError(name='hostname',
-                error=u'invalid domain-name: only letters, numbers, and - ' +
+                error=u'invalid domain-name: only letters, numbers, _, and - ' +
                     u'are allowed. DNS label may not start or end with -'),
         ),
 
+        dict(
+            desc='Try to add multiple CNAME record %r using dnsrecord_add' % (dnsrescname),
+            command=('dnsrecord_add', [dnszone1, dnsrescname], {'cnamerecord':
+                [u'1.example.com.', u'2.example.com.']}),
+            expected=errors.ValidationError(name='cnamerecord',
+                error=u'only one CNAME record is allowed per name (RFC 2136, section 1.1.5)'),
+        ),
+
         dict(
             desc='Add CNAME record to %r using dnsrecord_add' % (dnsrescname),
             command=('dnsrecord_add', [dnszone1, dnsrescname], {'cnamerecord': u'foo-1.example.com.'}),
@@ -803,14 +811,16 @@ class test_dns(Declarative):
             desc='Try to add other record to CNAME record %r using dnsrecord_add' % (dnsrescname),
             command=('dnsrecord_add', [dnszone1, dnsrescname], {'arecord': u'10.0.0.1'}),
             expected=errors.ValidationError(name='cnamerecord',
-                error=u'CNAME record is not allowed to coexist with any other records except PTR'),
+                error=u'CNAME record is not allowed to coexist with any other '
+                      u'record (RFC 1034, section 3.6.2)'),
         ),
 
         dict(
             desc='Try to add other record to CNAME record %r using dnsrecord_mod' % (dnsrescname),
             command=('dnsrecord_mod', [dnszone1, dnsrescname], {'arecord': u'10.0.0.1'}),
             expected=errors.ValidationError(name='cnamerecord',
-                error=u'CNAME record is not allowed to coexist with any other records except PTR'),
+                error=u'CNAME record is not allowed to coexist with any other '
+                      u'record (RFC 1034, section 3.6.2)'),
         ),
 
         dict(
@@ -827,6 +837,85 @@ class test_dns(Declarative):
             },
         ),
 
+        dict(
+            desc='Try to add multiple DNAME records to %r using dnsrecord_add' % (dnsresdname),
+            command=('dnsrecord_add', [dnszone1, dnsres1], {'dnamerecord':
+                [u'foo-1.example.com.', u'foo-2.example.com.']}),
+            expected=errors.ValidationError(name='dnamerecord',
+                error=u'only one DNAME record is allowed per name (RFC 6672, section 2.4)'),
+        ),
+
+        dict(
+            desc='Try to add invalid DNAME record %r using dnsrecord_add' % (dnsresdname),
+            command=('dnsrecord_add', [dnszone1, dnsresdname], {'dnamerecord': u'-.example.com.'}),
+            expected=errors.ValidationError(name='target',
+                error=u'invalid domain-name: only letters, numbers, _, and - ' +
+                    u'are allowed. DNS label may not start or end with -'),
+        ),
+
+        dict(
+            desc='Add DNAME record to %r using dnsrecord_add' % (dnsresdname),
+            command=('dnsrecord_add', [dnszone1, dnsresdname],
+                {'dnamerecord': u'd.example.com.', 'arecord': u'10.0.0.1'}),
+            expected={
+                'value': dnsresdname,
+                'summary': None,
+                'result': {
+                    'objectclass': objectclasses.dnsrecord,
+                    'dn': dnsresdname_dn,
+                    'idnsname': [dnsresdname],
+                    'dnamerecord': [u'd.example.com.'],
+                    'arecord': [u'10.0.0.1'],
+                },
+            },
+        ),
+
+        dict(
+            desc='Try to add CNAME record to %r using dnsrecord_add' % (dnsresdname),
+            command=('dnsrecord_add', [dnszone1, dnsresdname], {'cnamerecord': u'foo-1.example.com.'}),
+            expected=errors.ValidationError(name='cnamerecord',
+                error=u'CNAME record is not allowed to coexist with any other '
+                      u'record (RFC 1034, section 3.6.2)'),
+        ),
+
+        dict(
+            desc='Try to add NS record to %r using dnsrecord_add' % (dnsresdname),
+            command=('dnsrecord_add', [dnszone1, dnsresdname],
+                {'nsrecord': u'%s.%s.' % (dnsres1, dnszone1)}),
+            expected=errors.ValidationError(name='dnamerecord',
+                error=u'DNAME record is not allowed to coexist with an NS '
+                      u'record except when located in a zone root record (RFC 6672, section 2.3)'),
+        ),
+
+        dict(
+            desc='Add NS+DNAME record to %r zone record using dnsrecord_add' % (dnszone2),
+            command=('dnsrecord_add', [dnszone2, u'@'],
+                {'dnamerecord': u'd.example.com.',
+                 'nsrecord': dnszone1_mname}),
+            expected = {
+                'value': u'@',
+                'summary': None,
+                'result': {
+                    'objectclass': objectclasses.dnszone,
+                    'dnamerecord': [u'd.example.com.'],
+                    'dn': dnszone2_dn,
+                    'nsrecord': [dnszone2_mname, dnszone1_mname],
+                    'idnsname': [u'@']
+                }
+            },
+        ),
+
+
+        dict(
+            desc='Delete zone %r' % dnszone2,
+            command=('dnszone_del', [dnszone2], {}),
+            expected={
+                'value': dnszone2,
+                'summary': u'Deleted DNS zone "%s"' % dnszone2,
+                'result': {'failed': u''},
+            },
+        ),
+
         dict(
             desc='Try to add invalid KX record %r using dnsrecord_add' % (dnsres1),
             command=('dnsrecord_add', [dnszone1, dnsres1], {'kxrecord': u'foo-1.example.com' }),
@@ -1063,17 +1152,24 @@ class test_dns(Declarative):
         ),
 
         dict(
-            desc='Test that CNAME/PTR record type combination in record %r is allowed' % (dnsrev1),
-            command=('dnsrecord_add', [revdnszone1, dnsrev1], {'cnamerecord': u'foo-1.example.com.' }),
+            desc='Show record %r in zone %r with --structured and --all options'\
+                    % (dnsrev1, revdnszone1),
+            command=('dnsrecord_show', [revdnszone1, dnsrev1],
+                {'structured': True, 'all': True}),
             expected={
                 'value': dnsrev1,
                 'summary': None,
                 'result': {
-                    'objectclass': objectclasses.dnsrecord,
                     'dn': dnsrev1_dn,
                     'idnsname': [dnsrev1],
-                    'ptrrecord': [u'foo-1.example.com.'],
-                    'cnamerecord': [u'foo-1.example.com.'],
+                    'objectclass': objectclasses.dnsrecord,
+                    'dnsrecords': [
+                        {
+                            'dnstype': u'PTR',
+                            'dnsdata': u'foo-1.example.com.',
+                            'ptr_part_hostname': u'foo-1.example.com.'
+                        },
+                    ],
                 },
             },
         ),
@@ -1095,7 +1191,7 @@ class test_dns(Declarative):
             desc='Try to add invalid allow-query to zone %r' % dnszone1,
             command=('dnszone_mod', [dnszone1], {'idnsallowquery': u'foo'}),
             expected=errors.ValidationError(name='allow_query',
-                error=u"failed to detect a valid IP address from u'foo'"),
+                error=u"failed to detect a valid IP address from 'foo'"),
         ),
 
         dict(
@@ -1128,7 +1224,7 @@ class test_dns(Declarative):
             desc='Try to add invalid allow-transfer to zone %r' % dnszone1,
             command=('dnszone_mod', [dnszone1], {'idnsallowtransfer': u'10.'}),
             expected=errors.ValidationError(name='allow_transfer',
-                error=u"failed to detect a valid IP address from u'10.'"),
+                error=u"failed to detect a valid IP address from '10.'"),
         ),
 
         dict(
@@ -1307,7 +1403,7 @@ class test_dns(Declarative):
             command=('dnszone_del', [dnszone1], {}),
             expected={
                 'value': dnszone1,
-                'summary': None,
+                'summary': u'Deleted DNS zone "%s"' % dnszone1,
                 'result': {'failed': u''},
             },
         ),
@@ -1370,7 +1466,7 @@ class test_dns(Declarative):
             command=('dnszone_del', [dnszone1], {}),
             expected={
                 'value': dnszone1,
-                'summary': None,
+                'summary': u'Deleted DNS zone "%s"' % dnszone1,
                 'result': {'failed': u''},
             },
         ),
