@@ -187,6 +187,8 @@ class test_host(Declarative):
                     ipauniqueid=[fuzzy_uuid],
                     has_keytab=False,
                     has_password=False,
+                    ipakrbokasdelegate=False,
+                    ipakrbrequirespreauth=True,
                 ),
             ),
         ),
@@ -241,6 +243,8 @@ class test_host(Declarative):
                         managing_host=[u'%s' % fqdn1],
                         has_keytab=False,
                         has_password=False,
+                        ipakrbokasdelegate=False,
+                        ipakrbrequirespreauth=True,
                     ),
                 ],
             ),
@@ -696,6 +700,7 @@ class test_host(Declarative):
                 dict(
                     description=u'Test host 2',
                     l=u'Undisclosed location 2',
+                    userclass=[u'webserver', u'mailserver'],
                     force=True,
                 ),
             ),
@@ -711,6 +716,7 @@ class test_host(Declarative):
                     objectclass=objectclasses.host,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[fqdn2],
+                    userclass=[u'webserver', u'mailserver'],
                     has_keytab=False,
                     has_password=False,
                 ),
@@ -718,6 +724,27 @@ class test_host(Declarative):
         ),
 
 
+        dict(
+            desc='Retrieve %r' % fqdn2,
+            command=('host_show', [fqdn2], {}),
+            expected=dict(
+                value=fqdn2,
+                summary=None,
+                result=dict(
+                    dn=dn2,
+                    fqdn=[fqdn2],
+                    description=[u'Test host 2'],
+                    l=[u'Undisclosed location 2'],
+                    krbprincipalname=[u'host/%s@%s' % (fqdn2, api.env.realm)],
+                    has_keytab=False,
+                    has_password=False,
+                    managedby_host=[fqdn2],
+                    userclass=[u'webserver', u'mailserver'],
+                ),
+            ),
+        ),
+
+
         # This test will only succeed when running against lite-server.py
         # on same box as IPA install.
         dict(
