@@ -228,7 +228,9 @@ class test_service(Declarative):
                     objectclass=objectclasses.service,
                     ipauniqueid=[fuzzy_uuid],
                     managedby_host=[fqdn1],
-                    has_keytab=False
+                    has_keytab=False,
+                    ipakrbrequirespreauth=True,
+                    ipakrbokasdelegate=False,
                 ),
             ),
         ),
@@ -269,6 +271,8 @@ class test_service(Declarative):
                         ipauniqueid=[fuzzy_uuid],
                         has_keytab=False,
                         managedby_host=[fqdn1],
+                        ipakrbrequirespreauth=True,
+                        ipakrbokasdelegate=False,
                     ),
                 ],
             ),
@@ -461,6 +465,84 @@ class test_service(Declarative):
         ),
 
 
+        dict(
+            desc='Enable %r OK_AS_DELEGATE Kerberos ticket flag' % service1,
+            command=('service_mod', [service1], dict(ipakrbokasdelegate=True)),
+            expected=dict(
+                value=service1,
+                summary=u'Modified service "%s"' % service1,
+                result=dict(
+                    usercertificate=[base64.b64decode(servercert)],
+                    krbprincipalname=[service1],
+                    managedby_host=[fqdn1],
+                    ipakrbauthzdata=[u'MS-PAC'],
+                    valid_not_before=fuzzy_date,
+                    valid_not_after=fuzzy_date,
+                    subject=DN(('CN',api.env.host),x509.subject_base()),
+                    serial_number=fuzzy_digits,
+                    serial_number_hex=fuzzy_hex,
+                    md5_fingerprint=fuzzy_hash,
+                    sha1_fingerprint=fuzzy_hash,
+                    issuer=fuzzy_issuer,
+                    krbticketflags=[u'1048704'],
+                    ipakrbokasdelegate=True,
+                ),
+            ),
+        ),
+
+
+        dict(
+            desc='Update %r Kerberos ticket flags with setattr' % service1,
+            command=('service_mod', [service1],
+                     dict(setattr=[u'krbTicketFlags=1048577'])),
+            expected=dict(
+                value=service1,
+                summary=u'Modified service "%s"' % service1,
+                result=dict(
+                    usercertificate=[base64.b64decode(servercert)],
+                    krbprincipalname=[service1],
+                    managedby_host=[fqdn1],
+                    ipakrbauthzdata=[u'MS-PAC'],
+                    valid_not_before=fuzzy_date,
+                    valid_not_after=fuzzy_date,
+                    subject=DN(('CN',api.env.host),x509.subject_base()),
+                    serial_number=fuzzy_digits,
+                    serial_number_hex=fuzzy_hex,
+                    md5_fingerprint=fuzzy_hash,
+                    sha1_fingerprint=fuzzy_hash,
+                    issuer=fuzzy_issuer,
+                    krbticketflags=[u'1048577'],
+                ),
+            ),
+        ),
+
+
+        dict(
+            desc='Disable %r OK_AS_DELEGATE Kerberos ticket flag' % service1,
+            command=('service_mod', [service1], dict(ipakrbokasdelegate=False)),
+            expected=dict(
+                value=service1,
+                summary=u'Modified service "%s"' % service1,
+                result=dict(
+                    usercertificate=[base64.b64decode(servercert)],
+                    krbprincipalname=[service1],
+                    managedby_host=[fqdn1],
+                    ipakrbauthzdata=[u'MS-PAC'],
+                    valid_not_before=fuzzy_date,
+                    valid_not_after=fuzzy_date,
+                    subject=DN(('CN',api.env.host),x509.subject_base()),
+                    serial_number=fuzzy_digits,
+                    serial_number_hex=fuzzy_hex,
+                    md5_fingerprint=fuzzy_hash,
+                    sha1_fingerprint=fuzzy_hash,
+                    issuer=fuzzy_issuer,
+                    krbticketflags=[u'1'],
+                    ipakrbokasdelegate=False,
+                ),
+            ),
+        ),
+
+
         dict(
             desc='Delete %r' % service1,
             command=('service_del', [service1], {}),
